/* file     : ab_ball_filter.c
 *          : inherited from ab_filter
 *          : changed smooth and predict functions
 *          : to take care of constant-factor velocity
 *          : reduction
 * author   : Jussi Pajala 25.6.1997
 */           
#include <stdlib.h>
#include <math.h>
#include "Ab_ball_filter.h"
#include "decls.h"

Ab_ball_filter::Ab_ball_filter(float ball_decay, float new_alpha)
          :Ab_filter(new_alpha)
{
     int i;
     float vel = 1.0;
     float dist = 0.0;
     
     for( i = 0; i < ball_filt_array; i++ ){
          end_dist[i] = dist; // distance = 0, when dt = 0
          end_vel[i] = vel;  // vel = 1, when dt = 0

          dist += vel;
          vel *= ball_decay;
     }
}

void Ab_ball_filter::smooth (int curr_time, float measured_position,
                        float& new_smooth_pos, float& new_smooth_vel)
{
  int dt = curr_time - update_time;
  update_time = curr_time;

  if( dt <= 0 ) {
       new_smooth_pos = measured_position;
       new_smooth_vel = smooth_vel * .96;
       return;
  }
 
  
  
  /* if filter has been just resetted, initialize */
  if (count < 1) {            /* 1st time */
       smooth_pos = measured_position;
  }
  else if (count < 4) {       /* 2nd and 3rd time */
    smooth_vel
      = (smooth_vel + (measured_position - smooth_pos) / dt) / 2.0;
    smooth_pos = (measured_position + smooth_pos) / 2.0;
  } else {                            /* 4th, 5th, ... */
    /* predict current position and velocity from old smooth values */
       float pred_pos = smooth_pos;
       /*for(i = 0; i < dt && i < 10; i++){
            pred_pos +=  smooth_vel;
            smooth_vel *= .96;
       }
       if(i<dt && i == 10) pred_pos += smooth_vel*(dt-i);
       */
       // calculate distance traveled and speed after dt time
       if(dt < ball_filt_array){
            pred_pos += smooth_vel*end_dist[dt];
            smooth_vel *= end_vel[dt];
       } else {
            pred_pos += smooth_vel*end_dist[ball_filt_array - 1];
            smooth_vel *= end_vel[ball_filt_array - 1];
       }
              
       float pred_vel = smooth_vel;
       
       /* smooth predicted current state based on measurements */
       float pred_error = measured_position - pred_pos;
       smooth_pos = pred_pos + alpha * pred_error;
       smooth_vel = pred_vel + beta * pred_error / dt;
  }
  count++;
  new_smooth_pos = smooth_pos;
  new_smooth_vel = smooth_vel;

  last_pred_dt = 0;
  
}

void Ab_ball_filter::predict (int curr_time, int pred_delay,
                         float& new_pred_pos, float& new_pred_vel)
{
     int dt = curr_time - update_time + pred_delay;
     int i, counter = 0;
     
     if( last_pred_dt == 0){
          last_pred_pos = smooth_pos;
          last_pred_vel = smooth_vel;
     }

     new_pred_pos = last_pred_pos;
     new_pred_vel = last_pred_vel;
     
     if(dt <= 0) return;

     for( i = last_pred_dt; i < dt && counter < 10; i++){
          counter ++;
          new_pred_pos += new_pred_vel;
          new_pred_vel *= .96;
     }
     
     // calculate distance traveled and speed after dt time
/*     if(dt < ball_filt_array){
          last_pred_pos += smooth_vel*end_dist[dt];
          last_pred_vel *= end_vel[dt];
     } else {
          last_pred_pos += smooth_vel*end_dist[ball_filt_array - 1];
          last_pred_vel *= end_vel[ball_filt_array - 1];
     }
     */
     last_pred_pos = new_pred_pos;
     last_pred_vel = new_pred_vel;
     last_pred_dt = dt;
}

/* end of file */

  
