// -*- mode:C++ -*-
// File: Actionmap.cc
// Author: Antti Tikanmki, Jussi Pajala
// Contact:sunday@ee.oulu.fi, jussip@ee.oulu.fi
#include <Am/Actionmap.h>
float Actionmap::CUR_MAX_SPEED;
float *Actionmap::discr_cos = NULL;

Actionmap::Actionmap()
{
     Maptype type = Not; 
     amap.set_type(type); 
     min_reach_t = Cons2::min_reach_time();
     max_reach_t = Cons2::max_reach_time();

     // initialize dscrete cosine array
     if(!discr_cos){
         discr_cos = new float[amap.NRows];
         for( int i = 0; i < amap.NRows; i++ ){
             CUR_MAX_SPEED = Cons2::player_spd_limit();
             discr_cos[i] = cos( (float)i * 2 * M_PI / (float)amap.NRows);
         }
     }
}

void Actionmap::filter_times_longer_than(float time_longer_than)
{
     float weight_lower_than = time2weight(time_longer_than);
     amap.filter_weight_lower_than(weight_lower_than);
}


void Actionmap::mav(Actionmap& avoidmap, float accuracy_time)
{
     // MAV = Maximum of Absolute Values 
     // Calculate difference of weight in accuracy time
     float weight_diff = time2weight(max_reach_t-accuracy_time) - time2weight(max_reach_t);
     amap.mav(avoidmap.amap, weight_diff);
}

float Actionmap::linear_value(float cur_x,float min_x, float max_x,
                               float begin_y, float end_y)
{
     float cur_y;
     if (cur_x <= min_x) cur_y = begin_y;
     else if (cur_x >= max_x) cur_y = end_y;
     else cur_y = begin_y + (end_y - begin_y)*(cur_x - min_x)/(max_x - min_x);
     return(cur_y);
}

float Actionmap::time2weight(float reach_time)
{
     if(reach_time < max_reach_t)
          return linear_value(reach_time, min_reach_t, max_reach_t,1.0,0.1);
     return linear_value(reach_time, max_reach_t, 500,0.1,0.0);
}
float Actionmap::weight2time(float weight)
{
     if(weight<0.1) return linear_value(weight, 0.1, 0.0, max_reach_t, 500);
     else return linear_value(weight,1.0, 0.1, min_reach_t, max_reach_t);
}

// START SPEED:
// Returns needed start speed for given end speed at given distance
float Actionmap::ball_sspd_vd(float end_speed, float distance)
{
     return distance*(1.0-Cons2::ball_decay())+end_speed;
}
// Returns needed start speed for ball when reach time and distance is known
float Actionmap::ball_sspd_td(float reach_time, float distance)
{
     float ball_decay = Cons2::ball_decay();
     float exp_decay = ball_decay;
     for(int i=1;i<(int)reach_time;i++){
          exp_decay*=ball_decay;
     }
     return distance*((1.0-ball_decay )/(1.0-exp_decay));
}

// END SPEED:
// Returns end speed at distance when start speed is given
float Actionmap::ball_espd_vd(float start_speed, float distance)
{
     return start_speed - distance*(1.0-Cons2::ball_decay());
} 

// REACHE TIME
// Returns time that is needed to reache given distance when start speed is known
// There is a better way to caluculate this. send it to sunday@ee.oulu.fi!
float Actionmap::ball_reach_time(float cur_speed, float distance)
{
     float time;
     float cur_dist =0.0;
     for(time = 1.0; time <= 50.0;time++) {
          cur_speed *= Cons2::ball_decay();
          cur_dist += cur_speed;
          if( cur_dist > distance ) {
               float time_over = distance - cur_dist + cur_speed;
               time += time_over;
               return time;
          }
     }
     return 501.0;
}

// SPEED TO IMPULSE
// Calculate the impulse that equals given speed
float Actionmap::ball_spd2imp(float speed)
{
     return speed/Cons2::kick_power_rate();
}

// *********************************************
//        PROJECT                              *
// *********************************************
// There is two kinds of projections:
// projection for a dynamic (moving) object and projection for a static (non-moving) object.
// For dynamic object projection is done to one direction only, but for static object weights
// are calculated to both directions

// dir must be either 1 or -1, and it tells the direction of the projection
// row and col are position of the best action in the map
void Actionmap::project_dyn(int sign, int row, int col, float reach_time)
{
    int dir_change = 1;
    int spd = 0;
    float wgt = 0.0;
    float cur_reach_t = 0;
    float opt_spd = amap.col2mgn(col);
    
    if(reach_time < 1) reach_time = 1;
    while(dir_change < amap.NRows/4.0){
        float cosine = discr_cos[dir_change];
        for(spd = col; spd >= 0; spd--){
            float my_spd = cosine*(col2mgn(spd));
            // prevent greater approach speed than the optimal
            //if(my_spd > best_spd-0.1) my_spd = best_spd-0.1;
            cur_reach_t = reach_time * (opt_spd/my_spd);
            
            wgt = time2weight(cur_reach_t);
//            if( wgt == max_wgt) break;           
            
            amap.add_point(row + sign * dir_change, spd, wgt,0);
        }
        dir_change ++;
    }
    if(dir_change < amap.NRows/4.0) {
        cerr << "WRONG IT WENT!";
        assert(0);
    }      
}

void Actionmap::project_stat(int row, int col)
{
    int dir_change = 0;
    int spd = 0;
    float wgt;
    
    while(dir_change < amap.NRows/4 ){// NRows = 2*M_PI => NRows/4 = 0.5*M_PI
        float cosine = discr_cos[dir_change];
        for( spd = col; spd >= 0; spd--){
            float my_spd = cosine*col2mgn(spd);
            // prevent greater approach speed than the optimal
            //if(my_spd > opt_spd-0.1) my_spd = opt_spd - 0.1;
            
            wgt = time2weight(obj_dist/my_spd);
            amap.add_point(row + dir_change, spd, wgt,0);
            amap.add_point(row - dir_change, spd, wgt,0);
        }
        dir_change ++;
    }
}
int Actionmap::fill_collision_angle(int row, int col, float reach_time,
                                    float dist, float radius)
{
    int dir;
    int mgn;
    
    // calculate the greatest angle where we still collide to object
    float angle;
    float dtc; // distance to collision
    float coll_x, coll_y, temp_x, temp_y;
    
    if(dist-radius > 0 )
        angle = atan(radius/(dist-radius));
    else{
        angle = M_PI/2.0;
        dist = radius + 0.0001;
    }
    
    // calculate how many rows the greatest collide angle is
    int fill_rows = amap.dir2row(angle) - amap.NRows/2;
    
    assert(col>=0); // column must be at least 0!
    int intersec_points;

    // two loops: direction loop contains magnitude loop
    // i.e. for each direction all magnitudes are calculated
    for(dir = 0; (dir < fill_rows || dir == 0); dir++){
        // calculate distance to collision
        intersec_points = 
            line_circle_intersection(0.0,0.0,
                                     amap.row2dir(dir+((amap.NRows-1)/2)),
                                     dist, 0.0, radius,
                                     temp_x, temp_y,
                                     coll_x, coll_y);
        
        if(!intersec_points){
            if(dir == fill_rows-1) {
                // ok, this was the last round of this loop anyway,
                // sometimes intersection points are not found 
                fill_rows = dir;
                continue;
            }
            fill_rows = dir;
            continue;
        }
        
        dtc = sqrt(sqr(coll_x)+sqr(coll_y));
        float ttc = (dtc/dist) * reach_time;
    
        for(mgn = col; mgn>=0; mgn--){
            // this causes some inaccuracy because we have already
            // calculated accurate weight for optimal action.
            // with this formula we'll change this value a bit...
            float wgt = time2weight(ttc*(col2mgn(col)/col2mgn(mgn)));
           
            amap.add_point(row + dir, mgn, wgt, 0);
            amap.add_point(row - dir, mgn, wgt, 0);
        }
        // if this is avoid map and all columns are not filled,
        // fill greater speeds than latest filled column
        if(avoid && col < amap.NCols-1 ){
            amap.fill_faster_speeds( row + dir, col, 1.0 );            
            if(dir) amap.fill_faster_speeds( row - dir, col, 1.0 );
        }
    }
    if(!fill_rows) return 0;
    return fill_rows-1;
}
