// -*- mode:C++ -*-
// File:   Kickto_Goal_map.cc
// Author: Antti Tikanmki
// Contact:sunday@ee.oulu.fi
// Last c.:3. Jun 1998
// INFO:
// - Calculates vector that changes ball velocity toward opponents goal
//   (kick_to-vector) 
// - Changes that to impulse and add it to map
#include <Am/Kickto_Far_map.h>

Kickto_Far_map::Kickto_Far_map(void)
{
     obj_dir = obj_dist = ball_dir = ball_dist=0.0;
     amap.reset();
     amap.set_type(Kick);
     FAR_SECTOR = Cons2::far_sector();
}

void Kickto_Far_map::calculate_map(Vector ball_loc, Vector ball_vel, Vector goal_loc)
{
     reset();
     if(!kicks_calculated) calculate_possible_kicks(ball_loc, ball_vel);
     goal_loc = goal_loc - ball_loc;
     obj_dir = goal_loc.get_dir();
     obj_dist = goal_loc.get_len();
     
     // calculate direction limits
     float right_dir = goal_loc.get_dir() - 0.5*FAR_SECTOR;
     normalize_angle(right_dir);
     float left_dir  = goal_loc.get_dir() + 0.5*FAR_SECTOR;
     normalize_angle(left_dir);

     // Check the table over case
     int over = 0;
     if(left_dir<right_dir) over = 1;
     
     // Find the match kicks
     int col = NCols-1;
     float dir_diff, wgt, impulse;
     float find_max_vel_to_dir(float dir);
     
     int temp_col;
     if(over)
          for(int row = 0;row < NRows; row++) {
               if((kicks[row][col].dir < left_dir || kicks[row][col].dir > right_dir)&& 
                  kicks[row][col].vel >= ball_vel.get_len() )
                    {
                    dir_diff = fabs(calc_angle_difference(kicks[row][col].dir,obj_dir));
                    wgt = linear_value(dir_diff, 0.0, ANGLE_DIFF_LIMIT, 0.5, 0.0);
                    impulse = linear_value(dir_diff, 0.0, ANGLE_DIFF_LIMIT, MAX_IMP, MAX_IMP*0.5);
                    temp_col =  amap.mgn2col(impulse);
                    if(kicks[row][temp_col].dir < left_dir || kicks[row][temp_col].dir > right_dir)
                         amap.add_point(row, temp_col, wgt,0);
               }
          }
     else  
          for(int row = 0;row < NRows; row++) {
               if((kicks[row][col].dir < left_dir && kicks[row][col].dir > right_dir)&& 
                  kicks[row][col].vel >= ball_vel.get_len() ){
                    dir_diff = fabs(calc_angle_difference(kicks[row][col].dir,obj_dir));
                    wgt = linear_value(dir_diff, 0.0, ANGLE_DIFF_LIMIT, 0.5, 0.0);
                    impulse = linear_value(dir_diff, 0.0, ANGLE_DIFF_LIMIT, MAX_IMP, MAX_IMP*0.5);
                    temp_col =  amap.mgn2col(impulse);
                    if(kicks[row][temp_col].dir < left_dir && kicks[row][temp_col].dir > right_dir)
                         amap.add_point(row, temp_col, wgt,0);
               }
          }
}
float Kickto_Far_map::FAR_SECTOR;
