// -*- mode:C++ -*-
// File:   Polargrid.cc
// Author: Antti Tikanmki (sunday@ee.oulu.fi)
//         Jussi Pajala (jussip@ee.oulu.fi)

#include <Am/Polargrid.h>

#include <stdlib.h> // abs(i)
#include <iostream.h>// cout
#include "Cons2.h"

Polargrid::Polargrid()
{
     reset();
     MAX_IMPULSE = 100.0;
     MAX_SPD = Cons2::player_max_spd();
     mytype=Not;   
     cpp_out = NULL;
     tried_cpp_open = 0;

         
}
Polargrid::Polargrid(Maptype newtype)
{
     reset();
     MAX_IMPULSE = 100.0;
     MAX_SPD = Cons2::player_max_spd();
     mytype=newtype; 
     cpp_out = NULL;
     tried_cpp_open = 0;
}
Polargrid::~Polargrid()
{
     cpp_close();
}

void Polargrid::scan_nonzero()
{
     for(int j = 0;j < NRows; j++) {
          for(int i = 0;i < NCols; i++) {
               if(map[j][i]>0.0)
                    cerr<<"("<<i<<","<<j<<") ";
          }
     }
}
// *************************************************
// Methods to convert magnitude (speed or impulse) * 
// to column, and direction to row                 *
// *************************************************

int Polargrid::mgn2col(float spd)
{
    int column;
    if(mytype==Move){
        column = max(0, min(NCols-1, round( (NCols-1)*(spd/MAX_SPD) )));
    } else if(mytype==Kick){
        column = max(0, min(NCols-1, round( (NCols-1)*(spd/MAX_IMPULSE) )));
    } else{
        assert(0);// serious error! map has no type!
        return 0;
    }
    return column;
}

int Polargrid::dir2row(float dir)
{
    normalize_angle(dir);
    return round((dir/M_PI + 1.0 )*((NRows-1)*0.5) );
}
    
// *********************************************
//        ADD_POINT                            *
// *********************************************
// direction index j, distance index i
void Polargrid::add_point(float dir, float dst, float wgt, int fill )
{
     if(fabs(dir)>=10) cerr << __FILE__ << __LINE__ << "dir>10"<<endl;
     int new_j = dir2row(dir);
     int new_i = mgn2col(dst);

     add_point(new_j, new_i, wgt, fill );
}

void Polargrid::add_point(int j, int i, float wgt, int fill /*= 1*/)
{
     while(j>NRows-1) j-=(NRows-1);
     if(j<0)     j+=(NRows-1);
     if(i>NCols-1) i=NCols-1;
     if(i<0)     i=0;

     if(i==prev_i && j == prev_j && wgt < map[j][i]) return;
     map[j][i]=wgt;

     // 0 and NRows-1 are equal
     if(j==NRows-1 &&  wgt > map[0][i])map[0][i]=wgt;
     if(j==0 &&  wgt > map[NRows-1][i])map[NRows-1][i]=wgt;

     if( fill  && (i!=prev_i||j!=prev_j) )
          fill_gap(i, j);
     if(wgt>best_wgt)
          { best_wgt=wgt;best_i=i;best_j=j;};
     if(prev_i==-1 || prev_j==-1){ // If unused map
          range[0]=range[1]=j; range[2]=range[3]=i;
     }
     if(j<range[0])range[0]=j;     if(j>range[1])range[1]=j;
     if(i<range[2])range[2]=i;     if(i>range[3])range[3]=i;
     prev_i = i; prev_j = j;
}

void  Polargrid::add_mask_point(float dir, float dst, float wgt)
{
    int row = dir2row(dir);
    int col = mgn2col(dst);
    map[row][col] = wgt;
    if(row==NRows-1 && wgt > map[0][col])  map[0][col]=wgt;
    if(row==0 &&  wgt > map[NRows-1][col]) map[NRows-1][col]=wgt;
}

// *********************************************
//        OPERATORS                            *
// *********************************************
void operator+=(Polargrid& gridA, const Polargrid& gridB)
{
    gridA.best_j = gridA.best_i = -1;
     if(gridA.NRows!=gridB.NRows||gridA.NCols!=gridB.NCols) return;
     if(gridA.get_type()==Kick) 
          for(int j = 0;j < gridB.NRows; j++) {
               for(int i = 0;i < gridB.NCols; i++) {
                    gridA.map[j][i] += gridB.map[j][i];
               }
          } 
     else if(gridA.get_type()==Move)
          for(int j=0;j<gridA.NRows;j++) {
               for(int i=0;i<gridA.NCols;i++) {
                    if(fabs(gridA.map[j][i])<gridB.map[j][i])
                         gridA.map[j][i] = gridB.map[j][i];
               }
          }
     else assert(0); // serious error, wrong type
     
     gridA.range[0] = min(gridA.range[0],gridB.range[0]);
     gridA.range[1] = max(gridA.range[1],gridB.range[1]);
     gridA.range[2] = min(gridA.range[2],gridB.range[2]);
     gridA.range[3] = max(gridA.range[3],gridB.range[3]);
}
void operator-=(Polargrid& gridA, const Polargrid& gridB)
{
     gridA.best_j = gridA.best_i = -1;
     if(gridA.NRows!=gridB.NRows||gridA.NCols!=gridB.NCols) return;
     if(gridA.get_type()==Kick)
          for(int j = 0;j < gridB.NRows; j++) {
               for(int i = 0;i < gridB.NCols; i++) {
                    gridA.map[j][i] -= gridB.map[j][i];
               }
          }
     else if(gridA.get_type()==Move)// Combination style
          for(int j=0;j<gridA.NRows;j++) {
               for(int i=0;i<gridA.NCols;i++) {
                    if(fabs(gridA.map[j][i])<gridB.map[j][i])
                         gridA.map[j][i]=-1*gridB.map[j][i];
               }
          }
     else assert(0); // serious error, no type!
     
     gridA.range[0] = min(gridA.range[0],gridB.range[0]);
     gridA.range[1] = max(gridA.range[1],gridB.range[1]);
     gridA.range[2] = min(gridA.range[2],gridB.range[2]);
     gridA.range[3] = max(gridA.range[3],gridB.range[3]);
}

// Combination style
void operator <<(Polargrid& gridA, const Polargrid& gridB)
{
     gridA.best_j = gridA.best_i = -1;
    
     if(gridA.NRows!=gridB.NRows||gridA.NCols!=gridB.NCols) return;     
     for(int j=0;j<gridA.NRows;j++) {
          for(int i=0;i<gridA.NCols;i++) {
              if(fabs(gridA.map[j][i])<=fabs(gridB.map[j][i]))
                  gridA.map[j][i]=gridB.map[j][i];
          }
     }
     gridA.range[0] = min(gridA.range[0],gridB.range[0]);
     gridA.range[1] = max(gridA.range[1],gridB.range[1]);
     gridA.range[2] = min(gridA.range[2],gridB.range[2]);
     gridA.range[3] = max(gridA.range[3],gridB.range[3]);
}

Polargrid operator*(float r, const Polargrid& gridA)
{
     Polargrid temp_grid;
     if(r == 0.0) { temp_grid.reset(); return temp_grid; }
     else for(int j = 0;j < gridA.NRows; j++) {
          for(int i = 0;i < gridA.NCols; i++) {
               temp_grid.map[j][i] = r*gridA.map[j][i];
          }
     }
     temp_grid.best_wgt = r*gridA.best_wgt;
     return temp_grid;
}

void Polargrid::use_mask(Polargrid& mask)
{
     for(int j = 0;j < NRows; j++) {
          for(int i = 0;i < NCols; i++) {
               if(mask.map[j][i]==0.0)
                    map[j][i] = 0.0;
          }
     }
     // best value is not valid anymore
     best_i = best_j = -1;
     best_wgt = 0.0;
}
void Polargrid::use_invmask(Polargrid& mask)
{
     for(int j = 0;j < NRows; j++) {
          for(int i = 0;i < NCols; i++) {
               if(mask.map[j][i]==1.0){
                    map[j][i] = 0.0;
               }    
          }
     }
     // best value is not valid anymore
     mask.best_i = mask.best_j = -1;
     best_wgt = 0.0;
}

void Polargrid::invert()
{
     for(int j = 0;j < NRows; j++) {
          for(int i = 0;i < NCols; i++) {
               map[j][i] *= -1.0;
          }    
     }
     
// best value is not valid anymore
     best_i = best_j = -1;
     best_wgt = 0.0;
}

void Polargrid::filter_weight_lower_than(float weight_lower_than)
{
     for(int j = 0;j < NRows; j++) {
          for(int i = 0;i < NCols; i++) {
               if(map[j][i]<weight_lower_than)
                    map[j][i] = 0.0;
          }
     }
     best_i = best_j = -1;
}

void Polargrid::filter_speeds_faster_than(float curr_max_speed)
{
    best_j = best_i = -1;
    if(mytype==Move){
        if( curr_max_speed > MAX_SPD ) return;
        int max_i = mgn2col(curr_max_speed);
        for(int j = 0;j < NRows; j++) {
            for(int i = max_i;i < NCols; i++)
                map[j][i] = 0.0;
        }
    } else assert(0); // cant filter kick maps this way...
}

void Polargrid::mav(Polargrid& avoidgrid, float weight_diff)
{
     best_j = best_i = -1;
     for(int j = 0;j < NRows; j++) {
          for(int i = 0;i < NCols; i++) {
               if(map[j][i] - weight_diff < avoidgrid.map[j][i])
                    map[j][i] = -avoidgrid.map[j][i];
          }
     }
}

// *********************************************
//        FIND_BEST_ACTION                    *
// *******************************************

int Polargrid::find_best_action(Vector &best,float CUR_MAX_SPEED)
{
     // best_col array is used in selecting the best action
     // when several actions have an equal weight. in such
     // a case, the median is selcted. the index of best_col 
     // defines the row of the best weight, the value the column.
     // map[j][i] => best_col[j] = i
     //
     int cnt=0, best_col[NRows], nbest=0;
     for (cnt=0; cnt<NRows; cnt++) best_col[cnt] = 0;
     
     // If best value hasn yet found, scan the map
     if (best_i == -1|| best_j == -1) {
          best_wgt = 0.0;
          int maxcol = NCols-1;
          int mincol = 1;
          if(mytype == Move) maxcol = mgn2col(CUR_MAX_SPEED);
          if(mytype == Kick) mincol = mgn2col(15.0);
 
          for (int j=0;j<NRows;j++)
               for (int i = mincol; i <= maxcol; i++) {
                    if (map[j][i] > best_wgt) {
                         best_wgt = map[j][i];
                         for (cnt=0; cnt<NRows; cnt++) best_col[cnt] = 0;
                         best_col[j] = i;
                         nbest = 1;
                    }
                    else if (map[j][i] && map[j][i] == best_wgt) {
                         // if several best weights in a direction,
                         // select the smallest magnitude
                         if(!best_col[j]) {
                              best_col[j] = i;
                              nbest++;
                         }
                    }
               }
     }

     int selected = (int) (0.5 + nbest/2.0);
     int curr_best=0;
     for (cnt=0; cnt<NRows; cnt++) {
          if (best_col[cnt]) curr_best++;
          if (curr_best == selected) {
               best_i = best_col[cnt];
               best_j = cnt;
               break;
          }
     }
     
     // If best not found return zero
     if(best_i == -1|| best_j == -1) {
          cerr << "   but not found!" << endl;
          return 0;
     }
     
     // Calculate directions and values
     float dir = row2dir(best_j);
     float len = col2mgn(best_i);
     // Set the best vector
     best.set_dir_len(dir,len);
     // if map is good enought
     if(best_wgt>0.05)
          return 1;
     else return 0;
}


// *********************************************
//        FILL_GAP                             *
// *********************************************
// Gap filling-method. in: new point
// previous point will get from prev_i and prev_j
// Special case gap is over zero speed  Just go through gap
// in two sequence:
// - first from prev_i to zero speed, at same direction
// - second from zero speed to i
// uses float indexs for smoother behavior
// PROBLEM!
// - If we want to fill sector that is M_PI wide, 
//   fill_gap think it is over center point and uses special fill
//   -> sector is not filled
void Polargrid::fill_gap(int i, int j)
{
     int col_gap = abs(i-prev_i);  // Gap's lenght
     int row_gap = abs(j-prev_j);
     int col_gap_dir = sign(i-prev_i); //Gap's direction
     int row_gap_dir = sign(j-prev_j);
     float gap_wgt = 0.0;
     //If row gap is wider than half of the table = direction is wrong
     //chose another direction
     if (row_gap > (NRows-1)/2){
          row_gap = abs(row_gap - (NRows-1));
          row_gap_dir = -row_gap_dir;
     }
     // If there is previous point and gap is wide enough.
     if ((prev_j>=0 && row_gap > 1) || (prev_j>=0 && col_gap > 1)){
          gap_wgt = map[prev_j][prev_i];
          float gap_wgt_diff = map[j][i] - map[prev_j][prev_i];
          // special case: Gap is over center point
          if (row_gap == (NRows-1)/2) {
               float fill_i = 0.0;
               float fill_j = (float)prev_j;
               for (fill_i = prev_i-1;fill_i>=0;fill_i--) {
                    gap_wgt = gap_wgt + (float)gap_wgt_diff / (float)prev_i;
                    if(map[round(fill_j)][round(fill_i)]<gap_wgt)
                         map[round(fill_j)][round(fill_i)] = gap_wgt;}
               fill_j = (float)j;
               for (fill_i = 0; fill_i < i; fill_i++) {
                    gap_wgt = gap_wgt + (float)gap_wgt_diff / (float)i;
                    if(map[round(fill_j)][round(fill_i)]<gap_wgt)
                         map[round(fill_j)][round(fill_i)] = gap_wgt;}
          } else { // normal case  
               int step = max(abs(row_gap), abs(col_gap));
               float fill_i = (float)prev_i;
               float fill_j = (float)prev_j; 
               for (int count = step;count>=1;count--) {
                    fill_i = fill_i + (float)col_gap_dir*(float)col_gap / (float)step;
                    fill_j = fill_j + (float)row_gap_dir*(float)row_gap / (float)step;
                    // if fill goes over table end
                    if (round(fill_j) >= (NRows-1))
                         fill_j = fill_j - (float)(NRows-1);
                    if (round(fill_j) < 0)
                         fill_j = fill_j + (float)(NRows-1);
                    gap_wgt += (float)gap_wgt_diff / (float)step;
                    if(map[round(fill_j)][round(fill_i)]<gap_wgt)
                         map[round(fill_j)][round(fill_i)] = gap_wgt;
               } 
          } 
     }
}

void Polargrid::fill_faster_speeds(float dir,float dst,float wgt)
{
     int new_j = dir2row(dir);
     int new_i = mgn2col(dst);

     fill_faster_speeds(new_j, new_i, wgt);
}

void Polargrid::fill_faster_speeds(int row, int col, float wgt)
{  
    if(row < 0) row += (NRows-1);
    else if(row>NRows-1) row -= (NRows-1);
    
    assert( row >= 0 && row < NRows );
    assert( col >= 0 && col < NCols );
    
    for(int i = col; i < NCols;i++){
          if( map[row][i] < wgt) map[row][i] = wgt;
    }
    if(row == 0 || row == NRows-1){
        if(row) row = 0;
        else row = NRows - 1;
        for(int i = col; i <NCols;i++){
            if(map[row][i] < wgt) map[row][i] = wgt;
        }
    }
    range[3]=NCols-1;
}
    

// *********************************************
//        LIFT_SECTOR                          *
// *********************************************
void Polargrid::lift_sector(float sector, float amount)
{
     //don't lift empty map
     if(prev_i==-1 || prev_j==-1)return;
     // calculate size of  currentsector
     int current_sector_i = abs(range[1] - range[0]);
     // Calculate size of new sector
     int new_sector_i =(int)((float)((NRows-1)*(fabs(sector)/(2.0*M_PI))));
     // Calculate difference
     int diff_i = new_sector_i - current_sector_i;
     // If new is smaller
     if(diff_i <= 0) return;
     
     range[0] = range[0] - (diff_i / 2);
     if(range[0]<0)range[0] += NRows-1;
     range[1] =  range[1] + (diff_i / 2);
     if(range[1] >= NRows-1)range[1] -= NRows-1;
     int cur_j;
     for(int j = 0;j < new_sector_i;j++){
          for(int i=max(1,best_i-3);i<min(NCols,best_i+3);i++){
               cur_j = range[0]+j;
               if(cur_j >= NRows-1)cur_j-=NRows-1;
               if(map[cur_j][i]>amount)continue;
               else map[cur_j][i] += amount;
          }
     }
}

// *********************************************
//        PROPAGATE                            *
// *********************************************
// ALL index are in temp_map coords.
// right and left are names of propagation ranges ends in directions
void Polargrid::propagate(int ptimes, float amount)
{
     if(prev_i==-1||prev_j==-1)return;
     ptimes = min(ptimes,15);// 15 is limit of propagation times

     int right_row = range[0]+1;
     int left_row  = range[1]+1;
     int min_col   = range[2]+1;
     int max_col   = range[3]+1;
     int temp_right_row = 1;
     int temp_left_row  = NRows-1;
     float temp, new_wgt;
     int row, col, prop_row, prop_col, zone_left, zone_right;

     static float temp_map[NRows+1][NCols+2];
     memset(temp_map, 0, (NRows+1)*(NCols+2)*sizeof(float)); //reset temporary map

     for(int count=1;count<=ptimes;count++) {
          zone_left  = left_row + 2;
          if (zone_left > NRows-1) zone_left-=(NRows-1);
          zone_right = right_row-2;
          if (zone_right < 1) zone_right += (NRows-1);
   
          // Let's copy the area that is two index bigger than present range
          if (zone_right > zone_left) {// overflow case left & right separately
               for(col=min_col;col<=max_col;col++) {    
                    for(row=zone_right;row<=NRows;row++) {
                         if (col==0 || col==NCols+1)
                              temp_map[row][col] = 0.0;
                         if (col>=1 && row>=1 && col<=NCols)
                              temp_map[row][col]  = map[row-1][col-1];
                    }
               } 
               for(col=min_col; col<=max_col; col++) {
                    for(row = 1; row <= zone_left; row++) {
                         if (col==0 || col==NCols+1)
                              temp_map[row][col] = 0;
                         if (col>=1 && row>=1 && col<=NCols)
                              temp_map[row][col] = map[row-1][col-1];
                    }
               }
          } else {
               for(col=min_col;col<=max_col;col++) {
                    for(row=zone_right; row<=zone_left; row++) {
                         if (col==0 || col==NCols+1)
                              temp_map[row][col] = 0;
                         if (col>=1 && row>=1 && col<=NCols)
                              temp_map[row][col] = map[row-1][col-1];
                    }
               }    
          }
          for(col=min_col; col<=max_col; col++)
               temp_map[0][col] = map[NRows-2][col-1];

          right_row = right_row - 1;      
          left_row  = left_row + 1;
          min_col   = max(1, min_col-1);   
          max_col   = min(NCols, max_col+1);
          if (right_row < 1 )      right_row = NRows-1;
          if (left_row  > NRows-1) left_row  = 1;          
          if (right_row > left_row) {
               for (prop_row = right_row; prop_row <= temp_left_row; prop_row++) {
                    for (prop_col = min_col; prop_col <= max_col; prop_col++) {
                         temp=temp_map[prop_row][prop_col];          
                         temp=max(temp,temp_map[prop_row-1][prop_col]);      
                         temp=max(temp,temp_map[prop_row][prop_col-1]);
                         temp=max(temp,temp_map[prop_row+1][prop_col]);
                         temp=max(temp,temp_map[prop_row][prop_col+1]);   
                         new_wgt = (1-amount)*(temp_map[prop_row][prop_col]) + (amount*temp);
                         map[prop_row-1][prop_col-1] = new_wgt;
                         if(prop_row==NRows-1)map[0][prop_col-1] = new_wgt;
                    }
               }
               for (prop_row = temp_right_row; prop_row <= left_row; prop_row++) {
                      for (prop_col = min_col; prop_col <= max_col; prop_col++) {
                         temp=temp_map[prop_row][prop_col];          
                         temp=max(temp,temp_map[prop_row-1][prop_col]);      
                         temp=max(temp,temp_map[prop_row][prop_col-1]);
                         temp=max(temp,temp_map[prop_row+1][prop_col]);
                         temp=max(temp,temp_map[prop_row][prop_col+1]);   
                         new_wgt = (1-amount)*(temp_map[prop_row][prop_col]) + (amount*temp);
                         map[prop_row-1][prop_col-1] = new_wgt;
                    }
               }     
          } else {
               for (prop_row = right_row;prop_row <= left_row; prop_row++) {
                     for (prop_col = min_col;prop_col <= max_col; prop_col++) {
                         temp=temp_map[prop_row][prop_col];          
                         temp=max(temp,temp_map[prop_row-1][prop_col]);      
                         temp=max(temp,temp_map[prop_row][prop_col-1]);
                         temp=max(temp,temp_map[prop_row+1][prop_col]);
                         temp=max(temp,temp_map[prop_row][prop_col+1]);
                         new_wgt = (1-amount)*(temp_map[prop_row][prop_col]) + (amount*temp);
                         map[prop_row-1][prop_col-1] = new_wgt;
                    }
               }
          }
     } // end of main loop
     range[0] = right_row-1; range[1] = left_row-1;	 
     range[2] = min_col-1;   range[3] = max_col-1;
}

void Polargrid::dir_propagate(int ptimes, float amount)
{
     if(prev_i==-1||prev_j==-1)return;
     ptimes = min(ptimes,15);
     int right_row = range[0]+1;
     int left_row = range[1]+1;
     int min_col = range[2]+1;
     int max_col = range[3]+1;
     int temp_right_row = 1;
     int temp_left_row  = NRows-1;
     float temp, new_wgt; 
     int row, col, prop_row, prop_col, zone_left, zone_right;

     static float temp_map[NRows+1][NCols+2];
     memset(temp_map, 0, (NRows+1)*(NCols+2)*sizeof(float));

     for(int count=1;count<=ptimes;count++) {
          zone_left  = left_row + 2;
          if (zone_left > NRows-1) zone_left -= NRows-1;
          zone_right = right_row-2;
          if (zone_right < 1) zone_right += NRows-1;
   
          // Let's copy the area that is two index bigger than present range
          if (zone_right > zone_left) {// overflow case
               for(col=min_col;col<=max_col;col++) {
                    for(row=zone_right;row<=NRows;row++) {
                         if (col==0 || col==NCols+1)
                              temp_map[row][col] = 0.0;
                         if (col>=1 && row>=1 && col<=NCols)
                              temp_map[row][col] = map[row-1][col-1];
                    }
               }
               for(col=min_col; col<=max_col; col++) {
                    for(row = 1; row <= zone_left; row++) {
                         if (col==0 || col==NCols+1)
                              temp_map[row][col] = 0;
                         if (col>=1 && row>=1 && col<=NCols)
                              temp_map[row][col] = map[row-1][col-1];
                    }
               }
               
          } else for(col=min_col;col<=max_col;col++) {
               for(row=zone_right; row<=zone_left; row++) {
                    if (col==0 || col==NCols+1)
                         temp_map[row][col] = 0;
                    if (col>=1 && row>=1 && col<=NCols)
                         temp_map[row][col] = map[row-1][col-1];
               }
          }    
          for(col=min_col; col<=max_col; col++)
               temp_map[0][col] = map[NRows-2][col-1];
          
          right_row = right_row - 1;
          left_row = left_row + 1;
          if (right_row < 1 )      right_row = NRows-1;
          if (left_row  > NRows-1) left_row  = 1;
          
          if (right_row > left_row) {
               for (prop_row = right_row; prop_row <= temp_left_row; prop_row++) {
                    for (prop_col = min_col; prop_col <= max_col; prop_col++) {
                         temp=temp_map[prop_row][prop_col];          
                         temp=max(temp,temp_map[prop_row-1][prop_col]);      
                         temp=max(temp,temp_map[prop_row+1][prop_col]);
                         new_wgt = (1-amount)*(temp_map[prop_row][prop_col]) + (amount*temp);
                         map[prop_row-1][prop_col-1] = new_wgt;
                         if(prop_row==NRows) map[0][prop_col-1] = new_wgt;
                    }
               }
               for (prop_row = temp_right_row; prop_row <= left_row; prop_row++) {
                      for (prop_col = min_col; prop_col <= max_col; prop_col++) {
                         temp=temp_map[prop_row][prop_col];          
                         temp=max(temp,temp_map[prop_row-1][prop_col]);      
                         temp=max(temp,temp_map[prop_row+1][prop_col]);
                         new_wgt = (1-amount)*(temp_map[prop_row][prop_col]) + (amount*temp);
                         map[prop_row-1][prop_col-1] = new_wgt;
                         if(prop_row==1) map[NRows-1][prop_col-1] = new_wgt;
                    }
               }     
          } else {
               for (prop_row = right_row;prop_row <= left_row; prop_row++) {
                     for (prop_col = min_col;prop_col <= max_col; prop_col++) {
                         temp=temp_map[prop_row][prop_col];          
                         temp=max(temp,temp_map[prop_row-1][prop_col]);      
                         temp=max(temp,temp_map[prop_row+1][prop_col]);
                         new_wgt = (1-amount)*(temp_map[prop_row][prop_col]) + (amount*temp);
                         map[prop_row-1][prop_col-1] = new_wgt;
                    }  
               }
          }
     } // end of main loop
     range[0] = right_row-1; range[1] = left_row-1;	 
     range[2] = min_col-1;   range[3] = max_col-1;
}

// **************************
// For viewing and saving: *
// ************************
// *********************************************
//        SHOW_GRID                            *
// *********************************************
void Polargrid::show_grid()
{
     cout<<"Maptype is "<<mytype<<endl;
     for(int j=0;j<NRows;j++){
          for(int i=0;i<NCols;i++){
               if(j == NRows/2) cout<<map[j][i]<<"=";
               else cout<<map[j][i]<<" ";}
          if(j == NRows/2)cout<<">>";
          cout<<endl;}
}
// *********************************************
//        SAVE_MAP                             *
// *********************************************
// Simple savestyle that matlab viewmap-function can read. 
void Polargrid::save_map(char* mapname)
{
/*
     cout << "- Saving Polargrid " << mapname <<" to file.."<< endl;
     fout.open(mapname);
     for(int x=0;x<NRows;x++) {
          for(int y=0;y<NCols;y++) {
               fout<<map[x][y] << " ";}
          fout << endl;}    
     fout.close();   */
}

int Polargrid::cpp_open()
{  
    if(!tried_cpp_open){
        if(Cons2::debug_level() || Cons2::introduction()){
#ifdef CPP_FILE
            cpp_out = new cpp_ostream("maps/new.map");
#else 
            cpp_out = new cpp_ostream("|bin/gnuplot");
#endif
            // if we stop at this assert, something weird has happened, and
            // opening of the file or pipe was not succesful
            assert(cpp_out);
        }
        tried_cpp_open = 1;
    }
    return (int)cpp_out;
}
    
void Polargrid::cpp_close()
{
    if(cpp_out){
        cpp_out->close();
        delete cpp_out;
        cpp_out = NULL;
    }
}

int Polargrid::cpp_send(char* title)
{
     if(!cpp_open()){
          cerr<<"CPP File not open... Make sure debug_level is at least 1!" <<endl;
          return 0;
     }
     char cpp_title[CPP_MAX_NAME_LEN];
     strncpy(cpp_title, title, CPP_MAX_NAME_LEN);
     cpp_title[CPP_MAX_NAME_LEN-1] = '\0';
      if(mytype==Move)
           return my_cpp.send_surface (*cpp_out, cpp_title,
                                NRows, NCols, MAX_SPD, 
                                &(map[0][0]));
      else if(mytype == Kick)
           return my_cpp.send_surface (*cpp_out, cpp_title,
                                NRows, NCols, MAX_IMPULSE, 
                                &(map[0][0]));
      else assert(0);
      return 0;
}

int Polargrid::cpp_send(char* title, char* eps_file)
{
     if(!cpp_open()){
          cerr<<"CPP File not open... Make sure debug_level is at least 1!" <<endl;
          return 0;
     }
     char cpp_title[CPP_MAX_NAME_LEN];
     strncpy(cpp_title, title, CPP_MAX_NAME_LEN);
     cpp_title[CPP_MAX_NAME_LEN-1] = '\0';
     if(mytype==Move)
          return my_cpp.send_surface_to_epsfile  
               (*cpp_out, cpp_title,
                NRows, NCols, MAX_SPD, 
                &(map[0][0]),
                eps_file);
     else if(mytype == Kick)
          return my_cpp.send_surface_to_epsfile  
               (*cpp_out, cpp_title,
                NRows, NCols, MAX_IMPULSE, 
                &(map[0][0]),
                eps_file);
     else assert(0);
     return 0;
}
int Polargrid::cpp_opened = 0;

