// -*- mode:C++ -*-                                     
// File:   Polargrid.h
// Author: Antti Tikanmki 
// Contact: sunday@ee.oulu.fi
// This file declares Polargrid-class
// INFO:
// - Use add_point(...)-methods for adding points to map
// - value -1 is "not valid" value used in index

#ifndef POLARGRID_H
#define POLARGRID_H

#include "tools.h"
#include <string.h>
#include <math.h>
#include <fstream.h>
#include <assert.h>
#include "Cons2.h"
#include "Vector.h"
#include "cpp_comm.h"

typedef enum Maptype{ Not, Move, Kick};

class Polargrid {
  protected:
     int prev_i, prev_j;
     int best_i, best_j;
     float best_wgt;
     int range[4]; // 0 and 1 for directions right, left, and 2 and 3 for distances
     float MAX_IMPULSE;
     float MAX_SPD;
     Maptype mytype;
     static int cpp_opened;
     int tried_cpp_open;
     cpp_comm my_cpp;
     cpp_ostream *cpp_out;

     void fill_gap(int i, int j);
     int cpp_open(); // returns 1 if cpp is open
    
  public:
     void scan_nonzero(); // for debugging
     
     // Size of the map. number of rows must be odd, if not there is no forward direction.
     enum { NRows = 41 }; 
     enum { NCols = 21 }; 
     float map[NRows][NCols];

     inline void reset(){ 
         memset(*map, 0, NCols*NRows*sizeof(float));
         prev_i = prev_j = best_i = best_j = -1;
         best_wgt = 0.0;
         range[0]=range[1]=range[2]=range[3]=0;
     }; 
    
     void cpp_close(); // close cpp if open and terminates gnuplot
     int cpp_send(char* title);
     int cpp_send(char* title, char* eps_file);
     
     Polargrid();
     Polargrid(Maptype newtype);
     ~Polargrid();
     
     void set_type(Maptype newtype) {mytype = newtype; };

// ADDING POINTS:
     void add_point(float dir, float dst, float wgt, int fill = 1);
     void add_point(int dir, int dst, float wgt, int fill = 1);
     void add_mask_point(float dir, float dst, float wgt);
     void fill_faster_speeds(float dir, float dst, float wgt); // fills speeds faster than given with wgt
     void fill_faster_speeds(int row, int col, float wgt);

// FILTERING: 
     void filter_weight_lower_than(float weight_lower_than); // remove points wiht weight bellow limit
     void filter_speeds_faster_than(float curr_max_speed);   // remove points where speed is faster than...

     void Polargrid::use_mask(Polargrid& mask);    // remove points from map using mask map 
     void Polargrid::use_invmask(Polargrid& mask); // -''- points to remove are marked as 1 
     void lift_sector(float sector, float amount); // as it says, lift sector in the map
     void propagate(int ptimes, float pcoeff);
     void dir_propagate(int ptimes, float amount);
 
     void invert(); // inverts map
    
// COMBINATION METHODS
     friend void operator+=(Polargrid&, const Polargrid&);
     friend void operator-=(Polargrid&, const Polargrid&);
     friend void operator<<(Polargrid&, const Polargrid&);
     friend Polargrid operator*(float, const Polargrid&);
     void mav(Polargrid& avoidgrid, float weight_diff); // Maximum of Absolute Values,
     
// GETTING VALUES
     float   get_spdres()     {return Cons2::player_max_spd()/(float)(NCols-1);};
     float   get_dirres()     {return M_PI/(float)(NRows-1); };
     float   get_kickres()    {return MAX_IMPULSE/(float)(NCols-1);};
     float   get_max_speed()  {return Cons2::player_max_spd();};
     float   get_max_impulse(){return MAX_IMPULSE;};
     Maptype get_type()     {return mytype; };
     float   get_best_wgt()   {return best_wgt;};
     float   get_nrows()     {return NRows;};
     float   get_ncols()     {return NCols;};
     float   get_wgt(int row, int col) {return map[row][col]; }

     int mgn2col(float spd);
     int dir2row(float dir);
    
     float row2dir(int row){ 
         assert(row>=0 && row < NRows); 
         return M_PI * ((row/((NRows-1)*0.5)) - 1.0);
     };
     float col2mgn(int col){
          assert(col>=0 && col < NCols); 
          float max_mgn = (mytype==Move) ? MAX_SPD : MAX_IMPULSE;
          return max_mgn * (col/(float)(NCols-1));
     };
     int find_best_action(Vector &best,float CUR_MAX_SPEED);

     void show_grid();
     void save_map(char* mapname);
};
#endif
