// -*- mode:C++ -*-

// file    : Arbiter.h
// includes: class declaration for the arbiter
//

#ifndef _ARBITER
#define _ARBITER

#include "Vector.h"
#include "fieldobject.h"
struct Kick_cmd
{
    int valid;
    
    Vector g_vel;
    Vector imp;
    // Kick_actionmap map;
};

struct Move_cmd
{
    int valid;
    
    float spd;
    float dir;
    // maybe actionmap later...
};

struct Hold_cmd
{
    float dir;
    int valid;
};

struct View_change_cmd
{
    ViewAngle va;
    ViewQuality vq;
    int valid;
};


class Arbiter;

#include "Module.h"
#include <Ma/Ma_me.h>
#include <Ma/Ma_ball.h>
#include <Ma/Ma_teammate.h>
#include <Ma/Ma_opponent.h>
#include <Ma/Ma_goal.h>
#include <Ma/Ma_defloc.h>
#include <Ma/Ma_special_loc.h>
#include <Vs/Vs_referee.h>
#include <Ma/Ma_offside.h>
#include "Sensor.h"
#include "Be_move.h"
#include "Be_kick.h"
//#include "State_machine.h"

class Arbiter: public Module
{
public:
    Arbiter();
    ~Arbiter();
        
    void reset(){;}
    void invalidate_all();
    void Init(EmptySamba *sambapointer);
    int get_time()
        {
            return curr_time;
        };
    
    void read(Move_cmd &to);
    void read(Kick_cmd &to);
    void read(Hold_cmd &to);
    void read(View_change_cmd &to);

    float defloc_speed(){ return 0.5; }
protected:
    int transform();
    
    void read_markers();
    void read_move_behaviors();
    void read_kick_behaviors();
    
    void move(Move_cmd move);
    void kick(Kick_cmd kick);
    void hold_ball(Hold_cmd hold);
    int ball_kickable();
    int opponent_special_situation();
    int gk_catch_ball_valid();
    int catch_ball_valid();
    int must_avoid_opponents();
    float catch_time(Vector &player2ball, Vector &ball_g_vel, float max_spd);
    int closest_to_ball();
    int closest_calculated;
    int was_closest;
                   
    Ma_ball *ma_ball;
    Ma_me *ma_me;
    Ma_teammate *ma_teammate;
    Ma_opponent *ma_opponent;
    Ma_goal *ma_goal;
    Ma_defloc *ma_defloc;
    Ma_special_loc *ma_specloc;
    Ma_offside *ma_offside;
    
    Sensor *sensor;
    //Be_goto_ball *be_goto_ball;
    Be_move *be_move;
    Be_kick *be_kick;
    Vs_referee *vs_referee;

    int is_any_sensor_valid;
    Me_marker me;
    Ball_marker ball;
    Ball_marker_view_change view_change;
    Goal_marker opp_goal, own_goal;
    Referee_vs referee;
    Defloc_marker defloc;
    Special_loc specloc;
    Offside_marker offside;
    
    All_players team, opp_team;
        
    Kick_cmd final_kick;
    Move_cmd final_goto;
    Hold_cmd final_hold;
    View_change_cmd final_view_change;
        
    friend class Lookaround;
    friend class Make_close_goal;
    friend class Make_goal;
    friend class Pass_close;
    friend class Pass;
    friend class Dribble;
    friend class KickFar;
    friend class PanicKick;
    friend class Main_state_machine;
    friend class Catch_ball;
    friend class Turnto_ball;
    friend class Goto_defloc;

    friend class Goal_keeper;
    friend class Gk_hold_ball;
    friend class Gk_goto_defloc;
    friend class Gk_straight_forward;
    friend class Gk_straight_reverse;
    friend class Gk_left_forward;
    friend class Gk_right_forward;
    friend class Gk_left_reverse;
    friend class Gk_right_reverse;
    friend class Gk_catch_ball;
    friend class Gk_special_situation;
    
    friend class Special_situation;
    friend class Goto_specloc;
    friend class Turnto_goal;
    friend class Avoid_offside_sm;
    friend class Move_from_offside;
    friend class Offside_lookaround;
    
    Main_state_machine *main_state_machine;
    
    int last;
    
};
#endif
