/* File    : base_model.cc
 * Includes: functions for the Base model class
 */

#include "decls.h"
#include "Base_model.h"

/* constructor function */
Base_model::Base_model ()
{
  reset ();

  /* if you read parameters from configuration file:  
   *  if (!cfg_get ("xxx", &value)) value = 8000; 
   * xxx = value;
   * fprintf (stderr, "xxx set to %4.1f mm\n", value);
   */
    
  if(Cons2::debug_level_startup()) fprintf (stderr, "base_model: CONSTRUCTOR executed\n");
}


Base_model::~Base_model ()
{
  /* DEBUG */
     if(Cons2::debug_level_exit()) fprintf (stderr, "base_model: DESTRUCTOR executed\n");
}


void Base_model::reset ()
{
  turn_energy = 0;
  drive_energy = 0;
  turn_time = 0;
  drive_time = 0;
  spd_at_last_drive = 0;
  
}


// the current state of the base
void Base_model::state (int time, float *spd, float *aspd, float *turned, float *driven) 
{
     //float prev_turn_energy = turn_energy;
     //float prev_turn_time  = turn_time;
     //float turn_interval = time - prev_turn_time;

     //float prev_drive_energy = drive_energy;
     //float prev_drive_time  = drive_time;
     //float drive_interval = time - prev_drive_time;

//     turn_time = time;
//     drive_time = time;
     
     *turned = *aspd;
     *aspd = 0.0; // rotational speed / 0.1 seconds 
     
     *driven += .1*(*spd);
     *spd *= Cons2::player_decay();
     
     
     // turn_energy = decr_turn_energy (prev_turn_energy,turn_interval);
     // drive_energy = decr_drive_energy (prev_drive_energy,drive_interval);
     // Calculate spd - current translational speed (unit/s)
     //           aspd - current rotational speed (rad/s)
     //           turned - turned amount during turn interval (rad)
     //           driven - driven distance during drive interval (unit)
}


// the impulse required to turn the base to goal_dir
float Base_model::turn_impulse (int time, float curr_spd,float goal_dir)
{
     float impulse = goal_dir * (1.0 + 0.5*curr_spd);

     if(impulse > 180.0) impulse = 180;
     if(impulse < -180.0) impulse = -180.0;
     
  // Calculate the impulse required to turn to goal_dir - when
  // considering the current turn energy - and the resulting total 
  // turn energy. store the resulting energy in turn_energy

  turn_time = time;
  return impulse;
}

float Base_model::turn_angle(float curr_spd, float impulse)
{
     return impulse / (1.0 + 5.0*curr_spd);
}


// the impulse required to change the speed to goal_spd
float Base_model::drive_impulse (int time, float curr_spd, float goal_spd)
{

     float dash_power_rate = Cons2::dash_power_rate();
     float player_decay = Cons2::player_decay();
     
     float impulse;// = (goal_spd-player_decay*curr_spd)/dash_power_rate;

     if(Cons2::debug_level() > 4)cerr << "drive_time: " << drive_time << " time: " << time << endl;
     
//     if(time - drive_time > 5)
     impulse = (goal_spd - player_decay*goal_spd/*curr_spd*/) /
               dash_power_rate;
     
     // limit impulse
     if(impulse > 100.0) impulse = 100.0;
     if(impulse < 0.0) impulse = 0.0;
     
     spd_at_last_drive = player_decay*(curr_spd + impulse*dash_power_rate);
     drive_time = time;
     return impulse;
}

float Base_model::new_speed(float curr_spd, float impulse)
{
     return curr_spd + Cons2::dash_power_rate()*impulse;
}



/* end of file */


