// -*- mode:C++ -*-
// File: Be_kick.h
// Author: Antti Tikanmki sunday@ee.oulu.fi
#ifndef BE_KICK_H
#define BE_KICK_H

class Be_kick;

#include <Ma/Ma_me.h>
#include <Ma/Ma_ball.h>
#include <Ma/Ma_goal.h>
#include <Ma/Ma_opponent.h>
#include <Ma/Ma_teammate.h>

#include "EmptySamba.h"
#include "Module.h"
#include "Sensor.h"
#include <Am/Kickto_map.h>
#include <Am/Kickto_Near_map.h>
#include <Am/Kickto_Goal_map.h>
#include <Am/Kickto_Teammate_map.h>
#include <Am/Kickto_Opp_map.h>
#include <Am/Kickto_Bad_sector_map.h>
#include <Am/Kickto_Far_map.h>
#include "Be_move.h"

enum am_kickname {AM_KGOAL, AM_KNEAR, AM_KTEAMMATE, AM_KOPPONENT,
                  AM_KBADSECTOR, AM_KFAR, MAKE_GOAL, PASS, DRIBBLE, FAR,
                  NUMBER_OF_KICKNAMES};

class Be_kick:public Module
{
public:
    Be_kick();
    ~Be_kick();
    void Init(EmptySamba *sambapointer);
    Kickto_map *read(am_kickname name);
    
protected:
// Marker address
     Ma_me *ma_me;
     Ma_ball *ma_ball;
     Ma_goal *ma_goal;
     Ma_opponent *ma_opponent;
     Ma_teammate *ma_teammate;

// Data structures
     Me_marker me;
     Ball_marker ball;
     Goal_marker goal;
     All_players opponents;
     All_players teammates;     

// Actionmaps
     Kickto_Near_map nearmap;
     Kickto_Goal_map goalmap;
     Kickto_Teammate_map teammatemap[MAX_SEEN_PLAYERS];
     Kickto_Teammate_map all_teammatesmap;
     Kickto_Opp_map opponentmap[MAX_SEEN_PLAYERS];
     Kickto_Opp_map all_opponentsmap;
     Kickto_Bad_sector_map badsectormap;
     Kickto_Far_map farmap;

// Combination maps
     Kickto_map makegoal;
     Kickto_map pass;
     Kickto_map dribble;
     Kickto_map far;

// Flags for calculated maps, readed markers
     int calculated[NUMBER_OF_KICKNAMES];
     int marker_flag[NUMBER_OF_MTYPES];
     Kickto_map *maps[NUMBER_OF_KICKNAMES];

     must_calc_teammate(int player_no);
     must_calc_opponent(int player_no);
     
     int transform();
     void invalidate_all(){
          memset(calculated, 0,NUMBER_OF_KICKNAMES*sizeof(int));
          memset(marker_flag, 0,NUMBER_OF_MTYPES*sizeof(int));
          for( int i = 0; i < NUMBER_OF_KICKNAMES; i++)
              maps[i]->reset();
          far.set_kicks_calculated(0); // this can be any map
     };
#ifdef BE_KICK_AM_SAVE
     ofstream am_save_out;
#endif
};
#endif
