/* file    : Be_shout.cc
 * includes: functions for the shout behavior class
 */

#include "decls.h"
#include "Be_shout.h"
#include "Cons2.h"

Be_shout::Be_shout ()
{
  my_id = BE_SHOUT_ID;
  inhibited = NO;

  strcpy(teamname,"CAT");
  teamname[3] = Cons2::side_char();
  teamname[4] = ' ';
  teamname[5] = 0;
    
  if(Cons2::debug_level_startup()) 
      cerr << "Be_shout: CONSTRUCTOR executed" << endl;
}


Be_shout::~Be_shout ()
{
     if(Cons2::debug_level_exit()) 
         cerr << "Be_shout: DESTRUCTOR executed" << endl;
}
void Be_shout::Init(EmptySamba *sambapointer)
{
     samba  = sambapointer;
     vs_referee = (Vs_referee*)samba->get_pointer( VS_REFEREE_ID );
     ma_ball = (Ma_ball*)samba->get_pointer( MA_BALL_ID );
     ma_opponent = (Ma_opponent*)samba->get_pointer( MA_OPPONENT_ID );
}

FLAG Be_shout::valid_conds()
{
    vs_referee->read(referee);
    if (referee.game_state != after_goal
        && referee.game_state != before_kick_off
        && Cons2::me_unum() == 1 + ( curr_time % 11))
        return YES;
    return NO;
}

void Be_shout::invalidate_all()
{
    shout.valid = 0;
}

void Be_shout::read(Supply_cmd &to)
{
    check_executed();
    to = shout;
}

void Be_shout::put_loc_and_vel(char *message, int &i, 
                               Vector &loc, Vector &vel )
{
    static char temp[100];
    
    // copy location and velocity of ball to message
    sprintf(temp,"%.1f %.1f %.2f %.2f ", 
            loc.get_x(), loc.get_y(),
            vel.get_x(), vel.get_y() );
    strcpy(&(message[i]), temp);
    i += strlen(temp);
}

void Be_shout::put_int(char *message, int &i, int value)
{
    static char temp[30];
    sprintf(temp,"%d ", value);
    strcpy(&(message[i]), temp);
    i += strlen(temp);
}


int Be_shout::transform ()
{
         
     static char message[512];
          
     int i = 0;

     ma_ball->read(ball);
     if( ball.cer < 0.8) return 0;
     
     // copy teamname to message
     strcpy(&(message[i]), teamname /*"CAT_Finland98 "*/);
     i += strlen(teamname);

     // copy unum to message
     put_int(message, i, Cons2::me_unum());

     // copy current time to unum
     put_int(message, i, curr_time);

     // copy check value to message
     put_int(message, i, Cons2::me_unum() * 2 * ( curr_time % 111 ) - 1);
          
     // copy location and velocity of ball to message
     put_loc_and_vel(message, i, ball.g_loc, ball.g_vel);
     
     ma_opponent->read(opponents);
     
     if(opponents.valid){
         for( int k = 0; k < 11; k++){
             if(opponents.player[k].cer < 0.8) continue;
             put_int(message, i, k+1);
             put_loc_and_vel( message, i, opponents.player[k].g_loc,
                              opponents.player[k].g_vel);
             if( i >= 512 ) assert(0);
         }
     }
     // put value 12 to tell that no more players are to be sent
     put_int(message, i, 12);
     
          
     if(Cons2::debug_level()>1) cerr << "Sending at time " << curr_time 
                                     << " Length is " << i << endl
                                     << "Message: " << message << endl;
     

     shout.type = SAY_CMD_ID;
     shout.text = message;
     shout.valid = 1;
     
     return 0;
}

