#include "Cons2.h"
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include <assert.h>
#include <stdlib.h>
#include "cfg.h"
#include "tools.h" 
//INIT_FLAG_COORDS initial global coordinates of all the flags. 
// Flags must be ordered in the same order as in FLAGTYPES.
// Coords are defined here in .cc file because they should not be used 
// directly, since they are only initial values. Especially Goal flags will
// change their initial coordinate value to something else
extern const float INIT_FLAG_COORDS[NFLAGS][2];

//Cons2::Cons2() Initialize constants by reading configuration files
Cons2::Cons2(int doit)
          :Const(0) // prevent superclass to opening files
{
    side = 0;
    if(doit){
        // first read modulenames from a header file to ID_strings array
        int success = read_IDs_from_icomm_h();
        
        // and then fill constants with configuraton values
        if(success) success = read_config_file();
        
        if(!success){
            cerr <<"Tip: change to samba directory and execute program by typing" 
                 << endl << "bin/sambaplayer" << endl
                 << "Sources and configuration files should be "
                 << "located in samba/src/" << endl;
            exit(1);
          }
        
        // fill flag coordinate structure with initial values and
        // change value of initial coordinates of the goal flags
        init_flag_coords();
        theor_max_robot_speed = 
            MAX_DASH_IMP * const_dash_power_rate / const_player_decay;
        const_introduction = 0;
    }
}

// set uniform number and check if this player must debug
void Cons2::set_uniform_number(int unum){ 
    // set uniform number
    const_me_unum = unum;
    // check if this player must print debug information:
    // if debug_plr == 0, no one debugs
#ifdef TEST_SAMBA
    if(!debug_plr){ 
#ifdef VIS
        Visual_execute = 0;
#endif
    }
    // if debug_plr > 0, only player whose unum == debug_plr debugs
    else if(debug_plr > 0){
        if(debug_plr == unum) debug_plr = 1;
        else{
            debug_plr = 0;
#ifdef VIS
            Visual_execute = 0;
#endif
        }
    } 
    // if debug_plr < 0, all players debug
    else if(debug_plr < 0) debug_plr = 1;
#endif

};


// Reads soccer.cfg file from samba/src directory
// fills variables with their values configured in the file
// Returns zero on failure, 1 on success
int Cons2::read_config_file()
{
     
     FILE *cfg_fp;
     
     cfg_fp = cfg_open ("src/soccer.cfg");
     if (cfg_fp != NULL) {
          cfg_read (cfg_fp);
          cfg_close (cfg_fp);
     } else {
          cerr << "Could not read configuration file soccer.cfg!" << endl;
          return 0;
     }
          
     float temp;
#ifdef TEST_SAMBA
     cfg_get ("debug_plr_unum", &temp);
     debug_plr = round(temp);

     cfg_get ("debug_level", &temp);
     d_level = round(temp);

     cfg_get("debug_level_startup",&temp);
     d_level_startup = round(temp);

     cfg_get("debug_level_exit",&temp);
     d_level_exit = round(temp);

#endif //TEST_SAMBA
#ifdef VIS
     cfg_get ("Visual_local", &temp);
     Visual_local = round(temp);
     
     cfg_get ("Visual_opponents", &temp);
     Visual_opponents = round(temp);
     
     cfg_get ("Visual_teammates", &temp);
     Visual_teammates = round(temp);
     if(Cons2::debug_level_startup()) cerr << "Visual drawing " << Visual_teammates
                                  << " teammates " << endl;
     
     cfg_get ("Visual_ball", &temp);
     Visual_ball = round(temp);

     cfg_get ("Visual_execute", &temp);
     Visual_execute = round(temp);

     cfg_get ("Visual_exec_loc", &temp);
     Visual_exec_loc = round(temp);

     cfg_get ("Visual_heading", &temp);
     Visual_heading = round(temp);

     cfg_get ("Visual_exec_heading", &temp);
     Visual_exec_heading = round(temp);

     cfg_get ("Visual_opp_goal", &temp);
     Visual_opp_goal = round(temp);

     cfg_get ("Visual_own_goal", &temp);
     Visual_own_goal = round(temp);

     cfg_get ("Visual_goal_flags", &temp);
     Visual_goal_flags = round(temp);

     cfg_get ("Visual_border_flags", &temp);
     Visual_border_flags = round(temp);

     cfg_get ("Visual_lines", &temp);
     Visual_lines = round(temp);
          
     cfg_get ("Visual_defloc", &temp);
     Visual_defloc = round(temp);

     cfg_get ("Visual_special_loc", &temp);
     Visual_special_loc = round(temp);

     cfg_get ("Visual_pos_calc_objects", &temp);
     Visual_pos_calc_objects = round(temp);
#endif //VIS
     cfg_get("marker_advance", &temp);
     system_delay = round(temp);



     cfg_get("goal_width",&const_goal_width);
     if(const_goal_width != GOAL_WIDTH){
          // this is because i suspect that there is some places where
          // Cons2::goal_width is not used but program is using some
          // constant value (GOAL_WIDTH from icomm.h)
          cerr << "Goal width is wrong " << __FILE__ << " " << __LINE__ << endl;
          return 0;
     }
     cfg_get("goalie_catch_ban_cycle",&temp);
     goalie_catch_ban_cycle = round(temp);
     

     cfg_get("player_decay",&const_player_decay);  
     cfg_get("player_speed_max", &const_player_max_spd );
     cfg_get("player_rand",  &const_player_rand);
     cfg_get("player_size", &const_player_size );
     cfg_get("player_weight", &const_player_weight);
     cfg_get("dash_power_rate",&const_dash_power_rate);
     
     cfg_get("ball_decay",&const_ball_decay);
     cfg_get("ball_speed_max", &const_ball_max_spd);
     cfg_get("ball_rand", &const_ball_rand);
     cfg_get("ball_size",&const_ball_size );
     cfg_get("ball_weight",&const_ball_weight);
     cfg_get("kickable_margin",&const_kick_max_dist);
     // player and ball sizes must have been read before this
     const_kick_max_dist += const_player_size + const_ball_size;
     cfg_get("kick_power_rate",&const_kick_power_rate);
     
     cfg_get("cer_factor",&certainty_factor);

     // global locations for all the players,
     // defx,defy is the most defencive location
     // attx,atty is the most attacking position

     cfg_get("plr1defx",&def_coords[0][0]);
     cfg_get("plr1defy",&def_coords[0][1]);
     cfg_get("plr1attx",&att_coords[0][0]);
     cfg_get("plr1atty",&att_coords[0][1]);
     
     cfg_get("plr2defx",&def_coords[1][0]);
     cfg_get("plr2defy",&def_coords[1][1]);
     cfg_get("plr2attx",&att_coords[1][0]);
     cfg_get("plr2atty",&att_coords[1][1]);

     cfg_get("plr3defx",&def_coords[2][0]);
     cfg_get("plr3defy",&def_coords[2][1]);
     cfg_get("plr3attx",&att_coords[2][0]);
     cfg_get("plr3atty",&att_coords[2][1]);

     cfg_get("plr4defx",&def_coords[3][0]);
     cfg_get("plr4defy",&def_coords[3][1]);
     cfg_get("plr4attx",&att_coords[3][0]);
     cfg_get("plr4atty",&att_coords[3][1]);
     
     cfg_get("plr5defx",&def_coords[4][0]);
     cfg_get("plr5defy",&def_coords[4][1]);
     cfg_get("plr5attx",&att_coords[4][0]);
     cfg_get("plr5atty",&att_coords[4][1]);

     cfg_get("plr6defx",&def_coords[5][0]);
     cfg_get("plr6defy",&def_coords[5][1]);
     cfg_get("plr6attx",&att_coords[5][0]);
     cfg_get("plr6atty",&att_coords[5][1]);
     
     cfg_get("plr7defx",&def_coords[6][0]);
     cfg_get("plr7defy",&def_coords[6][1]);
     cfg_get("plr7attx",&att_coords[6][0]);
     cfg_get("plr7atty",&att_coords[6][1]);
     
     cfg_get("plr8defx",&def_coords[7][0]);
     cfg_get("plr8defy",&def_coords[7][1]);
     cfg_get("plr8attx",&att_coords[7][0]);
     cfg_get("plr8atty",&att_coords[7][1]);
     
     cfg_get("plr9defx",&def_coords[8][0]);
     cfg_get("plr9defy",&def_coords[8][1]);
     cfg_get("plr9attx",&att_coords[8][0]);
     cfg_get("plr9atty",&att_coords[8][1]);
     
     cfg_get("plr10defx",&def_coords[9][0]);
     cfg_get("plr10defy",&def_coords[9][1]);
     cfg_get("plr10attx",&att_coords[9][0]);
     cfg_get("plr10atty",&att_coords[9][1]);
     
     cfg_get("plr11defx",&def_coords[10][0]);
     cfg_get("plr11defy",&def_coords[10][1]);
     cfg_get("plr11attx",&att_coords[10][0]);
     cfg_get("plr11atty",&att_coords[10][1]);
// ACTIONMAP
     cfg_get("time_step", &amap_time_step);
     cfg_get("sidekick_wgt", &amap_sidekick_wgt);
     cfg_get("last_kick_time", &amap_last_kick_time);
     cfg_get("ball_near_speed", &amap_ball_near_speed);
     cfg_get("kick_near_wgt", &amap_kick_near_wgt);
     cfg_get("min_reach_time", &amap_min_reach_time);
     cfg_get("max_reach_time", &amap_max_reach_time);
     cfg_get("max_opp_inv_time", &amap_max_opp_inv_time);
     cfg_get("min_defloc_dst", &amap_min_defloc_dst);
     cfg_get("max_defloc_dst", &amap_max_defloc_dst);
     cfg_get("pass_end_spd", &amap_pass_end_speed);
     cfg_get("project_maps",&temp);amap_project_maps = round(temp);
     cfg_get("max_team_inv_time", &amap_max_team_inv_time);
     cfg_get("estimated_opp_spd", &amap_estimated_opp_spd);
// NEW 170898 by AT:
     cfg_get("opp_reaction_time", &amap_opp_reaction_time);
     cfg_get("far_sector", &amap_far_sector);
     cfg_get("angle_diff_limit", &amap_angle_diff_limit);
    
     return 1;
}



//char Cons2::ID_strings[LAST_ID+1+2][40]; // varan vuoksi 2 liikaa!
/*
// field constants
float Cons2::flag_coords[NFLAGS][2];
float Cons2::goal_width;

// player physics constants
float Cons2::player_decay;
float Cons2::player_max_spd;
float Cons2::player_rand;
float Cons2::player_size;
float Cons2::player_weight;
float Cons2::dash_power_rate;

// ball physics constants
float Cons2::ball_decay;
float Cons2::ball_max_spd;
float Cons2::ball_rand;
float Cons2::ball_size;
float Cons2::ball_weight;
float Cons2::kick_max_dist;
float Cons2::kick_power_rate;
*/


#ifdef TEST_SAMBA
int Cons2::debug_plr;
#endif //TEST_SAMBA
#ifdef VIS
int Cons2::Visual_execute;
int Cons2::Visual_local;
int Cons2::Visual_opponents;
int Cons2::Visual_teammates;
int Cons2::Visual_ball;
int Cons2::Visual_heading;
int Cons2::Visual_exec_loc;
int Cons2::Visual_exec_heading;
int Cons2::Visual_opp_goal;
int Cons2::Visual_own_goal;
int Cons2::Visual_goal_flags;
int Cons2::Visual_border_flags;
int Cons2::Visual_lines;
int Cons2::Visual_defloc;
int Cons2::Visual_special_loc;
int Cons2::Visual_pos_calc_objects;

#endif //VIS
char Cons2::side;
int Cons2::system_delay;
float Cons2::certainty_factor;

int Cons2::goalie_catch_ban_cycle;
float Cons2::def_coords[11][2];
float Cons2::att_coords[11][2];
float Cons2::theor_max_robot_speed;
ViewAngle Cons2::view_angle;
// ACTIONMAP:
float Cons2::amap_time_step;
float Cons2::amap_sidekick_wgt;
float Cons2::amap_last_kick_time;
float Cons2::amap_ball_near_speed;
float Cons2::amap_min_reach_time;
float Cons2::amap_max_reach_time;
float Cons2::amap_max_opp_inv_time;
float Cons2::amap_min_defloc_dst;
float Cons2::amap_max_defloc_dst;
float Cons2::amap_kick_near_wgt;
float Cons2::amap_pass_end_speed;
int Cons2::amap_project_maps;
float Cons2::amap_max_team_inv_time;
float Cons2::amap_estimated_opp_spd;

// NEW 170898 by AT:
float Cons2::amap_opp_reaction_time;
float Cons2::amap_far_sector;
float Cons2::amap_angle_diff_limit;

