#include "Const.h"
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <assert.h>
#include "cfg.h"
#include "tools.h" 
//INIT_FLAG_COORDS initial global coordinates of all the flags. 
// Flags must be ordered in the same order as in FLAGTYPES.
// Coords are defined here in .cc file because they should not be used 
// directly, since they are only initial values. Especially Goal flags will
// change their initial coordinate value to something else
const float INIT_FLAG_COORDS[NFLAGS][2] =
//TL_50, TL_40, TL_30, TL_20, TL_10, T_0, TR_10, TR_20, TR_30, TR_40, TR_50, 
{{-50, 39}, {-40, 39}, {-30, 39}, {-20, 39}, {-10, 39}, {  0, 39}, { 10, 39}, { 20, 39}, { 30, 39}, { 40, 39}, { 50, 39},
//LT, CT, RT, // flags on top line
 { -52.5,  34.0}, {  0.0 ,  34.0}, { 52.5,  34.0},  // flags on top line, left to right
//LT_30, RT_30,
 {-57.5, 30}, { 57.5,30},
//LT_20, PLT, PRT, RT_20, 
 {-57.5, 20}, { -36.0,  20.0}, { 36.0,  20.0}, { 57.5, 20},
//GLT, LT_10, RT_10, GRT,
 {-52.5,  7}, {-57.5, 10}, { 57.5, 10}, { 52.5,  7},
//L_0, PLC, C, PRC, R_0,
 {-57.5,  0}, { -36.0,   0.0},  {  0,  0 }, { 36.0,   0.0}, { 57.5,  0},
//GLB, LB_10, RB_10, GRB,
 {-52.5, -7}, {-57.5,-10}, { 57.5,-10}, { 52.5, -7},
//LB_20, PLB, PRB, RB_20,
 {-57.5,-20}, { -36.0, -20.0}, { 36.0, -20.0 },{ 57.5,-20},
//LB_30, RB_30,
 {-57.5,-30}, { 57.5, -30}, 
//LB, CB, RB,
 { -52.5, -34.0}, {  0.0 , -34.0}, { 52.5, -34.0},  // bottom line
//BL_50, BL_40, BL_30, BL_20, BL_10, B_0, BR_10, BR_20, BR_30, BR_40, BR_50, ERROR
 {-50,-39}, {-40,-39}, {-30,-39}, {-20,-39}, {-10,-39}, {  0,-39}, { 10,-39}, { 20,-39}, { 30,-39}, { 40,-39}, { 50,-39}};


//Const::Const() Initialize constants by reading configuration files
Const::Const(int doit)
{
     if(doit){ // this may prevent this contructor to work 
          // first read modulenames from a header file to ID_strings array
          int success = read_IDs_from_icomm_h();
          
          // and then fill constants with configuraton values
          if(success) success = read_config_file();
          
          if(!success){
               cerr <<"Tip: change to samba directory and execute program by typing" 
                    << endl << "bin/sambaplayer" << endl
                    << "Sources and configuration files should be "
                    << "located in samba/src/" << endl;
               exit(1);
          }
          
          // fill flag coordinate structure with initial values and
          // change value of initial coordinates of the goal flags
          init_flag_coords();
          const_introduction = 0;
     }
}

void Const::init_flag_coords()
{
     for(int i = 0; i < NFLAGS; i++){
          const_flag_coords[i][0] = INIT_FLAG_COORDS[i][0];
          const_flag_coords[i][1] = INIT_FLAG_COORDS[i][1];
     }
     // change y coordinate of the goal flags
     const_flag_coords[GLT][1] = const_goal_width/2.0;
     const_flag_coords[GLB][1] = -const_goal_width/2.0;
     const_flag_coords[GRT][1] = const_goal_width/2.0;
     const_flag_coords[GRB][1] = -const_goal_width/2.0;
}

void Const::flag_coords(FLAGTYPES flag, float &x, float &y)
{
     int i = (int)flag;
     if(i >=0 && i < (FLAGTYPES)ERROR){
          x = const_flag_coords[i][0];
          y = const_flag_coords[i][1];
          return;
     }
     
     cerr << "Error in " <<__FILE__<<" "<<__LINE__<<endl;
     if(i<0) cerr << "Flag number was below zero: " << i << endl;
     else cerr << "Flag number was too big: " << i << endl;
     assert(0); // stop execution with core
    
}

     
          

// This function read ID names of modules, signals etc. to the ID_strings array.
// The array is used by modulename() function.
// Returns zero if failure, 1 on success
int Const::read_IDs_from_icomm_h()
{
     FILE *fp;
     int i;

     // Open icomm.h
     if ((fp = fopen ("src/icomm.h", "r")) == NULL){
          cerr << "can't open icomm.h " << __FILE__ << " "
               << __LINE__ << endl;
          return 0;
     }
     i = 0;
     while(i++ < 10000){
          if(!fscanf(fp,"%s",ID_strings[0])){
               cerr << "no string..." << endl;
               continue;
          }
          if(!strcmp(ID_strings[0],"HERE_THEY_ARE")) break;
     }
     
     i = 0;
     
     char temp[120];
     unsigned int p;
     int length;
     
     while( i <= LAST_ID){
          while(!fscanf(fp, " %[^\n]",temp))
               cerr << "!Why here??" << ID_strings[i] << " " << __FILE__
                    << " " << __LINE__ <<endl;
          if(!strncmp(temp ,"//",2))
               continue;
          p=0;
          while(p < strlen(temp) && sscanf(&(temp[p]), " %[^,], %n", ID_strings[i],&length)){
//               cerr << ID_strings[i] << endl;
               i++;
               p+=length;
          }
     }
     fclose(fp);
//   print all Id names on screen:     
//     for(i = 0; i<=LAST_ID; i++) cerr << ID_strings[i] << " " << i << endl;
     return 1;
     
}


// This function returns name of the ID, ie name of the module or signal.
char *Const::modulename(ID i)
{
     return ID_strings[i];
}

// Reads soccer.sfg file from samba/src directory
// fills variables with their values configured in the file
// Returns zero on failure, 1 on success
int Const::read_config_file()
{
     
     FILE *cfg_fp;
     
     cfg_fp = cfg_open ("src/soccer.cfg");
     if (cfg_fp != NULL) {
          cfg_read (cfg_fp);
          cfg_close (cfg_fp);
     } else {
          cerr << "Could not read configuration file soccer.cfg!" << endl;
          return 0;
     }
          
     float temp;
     
     cfg_get ("debug_level", &temp);
     d_level = round(temp);

     cfg_get("debug_level_startup",&temp);
     d_level_startup = round(temp);
     
     cfg_get("debug_level_exit",&temp);
     d_level_exit = round(temp);
  
     cfg_get("goal_width",&const_goal_width);
     if(const_goal_width != GOAL_WIDTH){
          // this is because I suspect that there is some places where
          // const::goal_width is not used but program is using some
          // constant value (GOAL_WIDTH from icomm.h)
          cerr << "Goal width is wrong " << __FILE__ << " " << __LINE__ << endl;
          return 0;
     }

     cfg_get("player_decay",&const_player_decay);  
     cfg_get("player_speed_max", &const_player_max_spd );
     cfg_get("player_rand",  &const_player_rand);
     cfg_get("player_size", &const_player_size );
     cfg_get("player_weight", &const_player_weight);
     cfg_get("dash_power_rate",&const_dash_power_rate);
     
     cfg_get("ball_decay",&const_ball_decay);
     cfg_get("ball_speed_max", &const_ball_max_spd);
     cfg_get("ball_rand", &const_ball_rand);
     cfg_get("ball_size",&const_ball_size );
     cfg_get("ball_weight",&const_ball_weight);
     cfg_get("kickable_margin",&const_kick_max_dist);
     // player and ball sizes must have been read before this
     const_kick_max_dist += const_player_size + const_ball_size;
     cfg_get("kick_power_rate",&const_kick_power_rate);
     
     return 1;
}

char Const::ID_strings[LAST_ID+1+2][40]; // varan vuoksi 2 liikaa!

int Const::d_level;
int Const::d_level_startup;
int Const::d_level_exit;

int Const::const_introduction;

// field constants
float Const::const_flag_coords[NFLAGS][2];
float Const::const_goal_width;

// player physics constants
float Const::const_player_decay;
float Const::const_player_max_spd;
float Const::const_player_rand;
float Const::const_player_size;
float Const::const_player_weight;
float Const::const_dash_power_rate;

// ball physics constants
float Const::const_ball_decay;
float Const::const_ball_max_spd;
float Const::const_ball_rand;
float Const::const_ball_size;
float Const::const_ball_weight;
float Const::const_kick_max_dist;
float Const::const_kick_power_rate;
int Const::const_me_unum;

// end of file
