// -*- mode:C++ -*-                                     
// File:   Ma_ball.h                             
// Author: Jussi Pajala 1.5.1997,  jussip@tks.oulu.fi   
//
// this file declares class of the ball marker

#ifndef MA_BALL_H
#define MA_BALL_H

struct Ball_marker
{
    Vector g_loc;     // global location
    Vector g_vel;     // global velocity
         
    Vector mix_vel; // global speed and local direction:
                      // this is needed in actionmap calculations!
    Vector loc;       // local location
    Vector vel;       // local velocity
    
    float cer;        // certainty

    int valid;
};

struct Ball_marker_view_change
{
    int angle;
    int quality;
    int valid;
};




class Ma_ball;

#include "Module.h"
//#include "Ab_ball_filter.h"
#include <Ma/Ball_state.h>
#include <Ma/Ma_me.h>
#include "Sensor.h"
#include <Vs/Vs_referee.h>
#include <Vs/Vs_team_audio.h>
#include "Ac_kick.h"
//#include "Ac_supply.h"
#include "Arbiter.h"
// These two must be declared here because we have an declaration loop
// Ma_ball->Ac_kick->Ma_ball
class Ac_kick;
struct Kick_actuator;
// And these two because we have an extra loop during testing
class Ma_me;

class Ma_ball:public Module
{
public:
    Ma_ball();
    ~Ma_ball();
    void Init(EmptySamba *sambapointer);
    
    virtual void reset(){ ;};  // do nothing in reset!!
    
    //void read_sens(Ball_marker &to);//ball state computed from sensor data
    //void read_exec(Ball_marker &to);//ball state at the next command execution
    void read(Ball_marker &to);
    void read_vq_change(View_change_cmd &to);
protected:
    int       transform();      // called by Module::execute()
    
    void      compute_curr_pos();
    void      use_team_audio();
    void      compute_with_delay_reduction();
    void      compute_with_old_routine();
    void      update_ball_state();
    void      check_game_state();
    void      calculate_exec_and_sense_state();
    void      calculate_certainty();
    
    void      read_inputs();
    void      invalidate_all();//invalidates all data produced by this module
    
    Ma_me         *ma_me;        //me_msig_in
    Sensor        *sensor;       //ball_ssig_in, sensor_ssig_in
    Vs_referee    *vs_referee;   //referee_vssig_in
    Vs_team_audio *vs_team_audio;//team_audio_in
    Ac_kick       *ac_kick;      //kicks_in
    
    Vector g_loc;
    Vector g_vel;
    int cer_calculated;
    int g_loc_calculated;
    int g_vel_calculated;
    
    Ball_marker sense; // ball computed from sensor data
    Ball_marker exec;  // ball state at the next command execution moment
    
    Referee_vs referee;
 
    Kick_actuator kicks;
    View_change_cmd vq_change;
    
//    Ab_ball_filter *filt_g_x;
//     Ab_ball_filter *filt_g_y;
    
    Team_audio_vs team_audio;
    
    Ball_sensor ball_sensor;
    
    Me_marker me_sens, me_exec;
     
    Ball_state *ball_state; // estimates ball state at execution moment
};

#endif
