// -*- mode:C++ -*-                                     
// File:   Ma_border.h                             
// Author: Jussi Pajala 1.5.1997,  jussip@tks.oulu.fi   
//
// this file declares class of the border marker


#ifndef MA_BORDER_H
#define MA_BORDER_H
#include "Vector.h"

struct Line_data
{
     Vector loc; //local location
     float llen; //length to the left( from the point that is nearest to the 
                 // robot and robot looking towards that point )
     float rlen; //length to the right
     float angle;//angle to the line
     CERTAINTY cer; 
     Vector start_g_loc; // global starting of the line
     Vector end_g_loc;   // global ending 
};

struct Border_marker
{
     Line_data T,B,Lt,Lb,Rt,Rb,C;
     // Top, Bottom, LeftTop, LeftBottom, RightTop, RightBottom, Center
     // Right ang Left lines are cutted off so that goal is located between
     // {Rt,Lt} and {Rb,Lb} lines, respectively.
};

#include "Module.h"


class Ma_border:public Module
{
public:
     Ma_border();
     ~Ma_border();
     void Init(EmptySamba *sambapointer);
     virtual void reset() {;};
     
     void read(Border_marker &to);
protected:
     FLAG      valid_conds();  // used in Module::execute()
     int       transform();      // called by Module::execute()

     void invalidate_all(); // invalidates all data produced by this module
     
// pointers to incoming buffers, connected when initialized
     Sensor  *sensor;       // *sensor_ssig_in;
     Ma_me   *ma_me;        // *me_msig_in;
     
     Me_marker me_sens;
     Me_marker me_exec;
     
     Vector TOP_LEFT;
     Vector TOP_RIGHT;
     Vector RIGHT_GOAL_TOP;
     Vector RIGHT_GOAL_BOTTOM;
     Vector BOTTOM_RIGHT;
     Vector BOTTOM_LEFT;
     Vector LEFT_GOAL_BOTTOM;
     Vector LEFT_GOAL_TOP;
     
     Border_marker lines;
};

#endif
