// file    : ma_goal.cc   

#include "decls.h"
#include <Ma/Ma_goal.h>

Ma_goal::Ma_goal()
{
     my_id = MA_GOAL_ID;
     inhibited = NO;
     
     OWN_GOAL_G_LOC.left.  set_xy(MIN_X,-Cons2::goal_width()/2.0);
     OWN_GOAL_G_LOC.right. set_xy(MIN_X, Cons2::goal_width()/2.0);
     OWN_GOAL_G_LOC.center.set_xy(MIN_X, 0.0);
     
     OPP_GOAL_G_LOC.left.  set_xy(MAX_X, Cons2::goal_width()/2.0);
     OPP_GOAL_G_LOC.right. set_xy(MAX_X,-Cons2::goal_width()/2.0);
     OPP_GOAL_G_LOC.center.set_xy(MAX_X, 0.0);
     
     if(Cons2::debug_level_startup()) 
          cerr << "Ma_goal: CONSTRUCTOR executed" << endl;
}

Ma_goal::~Ma_goal()
{
     if(Cons2::debug_level_exit()) 
          cerr << "Ma_goal: DESTRUCTOR executed" << endl;
}

void Ma_goal::Init(EmptySamba *sambapointer)
{
     samba = sambapointer;
     sensor = (Sensor*)samba->get_pointer( SENSOR_ID );
     ma_me = (Ma_me*)samba->get_pointer( MA_ME_ID );
}

void Ma_goal::read_opp_goal(Goal_marker &to)
{
     check_executed();
     to = exec_opp_goal;
}

void Ma_goal::read_own_goal(Goal_marker &to)
{
     check_executed();
     to = exec_own_goal;
}


void Ma_goal::calc_goal_data(Goal_marker &exec, Goal_marker& sens, 
                             Goal_sensor goal_sensor, Goal_locs G_LOC)
{
    static Vector temp_vect;
    
    // get opp_goal's new location or approximate it,
    if( goal_sensor.valid ){
        sens.loc.center = goal_sensor.pos;
        
        // calculate sensed global position ( may be a little different
        // to real position )
        sens.g_loc.center = loc2g_loc( sens.loc.center, me_sens.g_loc, 
                                       me_sens.heading );
        // calculate left and right poles, too
        temp_vect = sens.g_loc.center - G_LOC.center;
        sens.g_loc.left  = G_LOC.left  + temp_vect;
        sens.g_loc.right = G_LOC.right + temp_vect;
        
        // with this assert we can check if there is really terrible mistake
        if(temp_vect.get_len() >= 15.0) {
#ifndef NOPRINTS 
            cerr << "goal sensed location is: (" << sens.g_loc.center.get_x()
                 << "," << sens.g_loc.center.get_y() << ")" << endl
                 << "And should be: (" << G_LOC.center.get_x()
                 << "," << G_LOC.center.get_y() << ")" << endl
                 << "me exec g_loc: (" << me_exec.g_loc.get_x() << ","
                 << me_exec.g_loc.get_y() << ")" << "me dir "
                 << me_exec.heading << endl
                 << "me sense g_loc: (" << me_sens.g_loc.get_x() << ","
                 << me_sens.g_loc.get_y() << ")" << "me dir "
                 << me_sens.heading << endl;
            
            //assert ( temp_vect.get_len() < 15.0 );
#endif
        }
        
        // then calculate local locations of the poles
        sens.loc.left = g_loc2loc( sens.g_loc.left, me_sens.g_loc, 
                                   me_sens.heading );
        sens.loc.right = g_loc2loc( sens.g_loc.right, me_sens.g_loc, 
                                    me_sens.heading );
        sens.cer = 1.0;
    } else {
        if( sensor->is_any_valid() ){
            sens.g_loc = G_LOC;
            sens.cer   = me_sens.g_cer;
        } else {
            //sens.g_loc = sens.g_loc // using old g_loc !
            sens.cer *= Cons2::cer_factor();
        }
        // because not seen, local locations must be calculated
        sens.loc.left = g_loc2loc(sens.g_loc.left, me_sens.g_loc, 
                                  me_sens.heading);
        sens.loc.center = g_loc2loc(sens.g_loc.center, me_sens.g_loc, 
                                    me_sens.heading);
        sens.loc.right = g_loc2loc(sens.g_loc.right, me_sens.g_loc, 
                                   me_sens.heading);
    }
    
    if(!Cons2::delay()) {
        cerr << "Ball marker doesn't support no-delay-mode" << endl
             << "Is there anything to be separately supported" << endl
             << "is another thing..." << endl
             << __FILE__ << " " << __LINE__ << endl;
//        exit(1);
    }
    // global location must be same since goal is not moving
    exec.g_loc = sens.g_loc;
    // local locations can be calculated with me marker's estimate of our
    // global location at execution moment
    exec.loc.left = g_loc2loc(exec.g_loc.left, me_exec.g_loc, 
                              me_exec.heading );
    exec.loc.center = g_loc2loc(exec.g_loc.center, me_exec.g_loc, 
                                me_exec.heading );
    exec.loc.right = g_loc2loc(exec.g_loc.right, me_exec.g_loc, 
                               me_exec.heading );

    exec.cer = me_exec.g_cer;
    exec.valid = sens.valid = 1;
    // and that's all...
    return;
}

void Ma_goal::invalidate_all()
{
    exec_opp_goal.valid = 0;
    sens_opp_goal.valid = 0;
    exec_own_goal.valid = 0;
    sens_own_goal.valid = 0;
}


int Ma_goal::transform()
{
    ma_me->read_sens(me_sens);
    ma_me->read_exec(me_exec);
    sensor->read_goal_sensors(opp_goal_sensor, own_goal_sensor);
    
    calc_goal_data( exec_opp_goal, sens_opp_goal, opp_goal_sensor, 
                    OPP_GOAL_G_LOC );
    calc_goal_data( exec_own_goal, sens_own_goal, own_goal_sensor, 
                    OWN_GOAL_G_LOC );
    
    return 0;
}
