// file    : Ma_me.cc   

//#include "decls.h"
#include <Ma/Ma_me.h>

#include <Am/Actionmap.h>
#include <stdlib.h>
#ifndef RAD2DEG
#define RAD2DEG (180.0 / (float)M_PI) 
#define DEG2RAD ((float)M_PI / 180.0)
#endif

class Pos_calculator
{
protected:
    float heading;
    float sum_x;
    float sum_y;
    int n;
public:
    Pos_calculator();
    void reset();
    void set_heading(float head);
    void new_object(Vector pos, float gx,float gy);
    Vector get_position();
};

Pos_calculator::Pos_calculator()
{
    reset();
}
void Pos_calculator::reset()
{
    sum_x = 0;
    sum_y = 0;
    heading = 0;
    n = 0;
}
void Pos_calculator::set_heading(float head)
{
    heading = head;
}

void Pos_calculator::new_object(Vector pos, float gx, float gy)
{
    sum_x += gx - pos.get_len() * cos(heading + pos.get_dir());
    sum_y += gy - pos.get_len() * sin(heading + pos.get_dir());
    n++;
}

Vector Pos_calculator::get_position()
{
    Vector temp;
    if(n == 0){
        temp.set_xy(10000.0,0.0);
    } else {
        temp.set_xy(sum_x/(float)n,sum_y/(float)n);
    }
    
    return temp;
}

Pos_calculator pos_calc;

//#define MA_ME_FILTER
     

Ma_me::Ma_me()
{
     my_id = MA_ME_ID;
     inhibited = NO;

     sense.g_loc.set_xy(0.0, 0.0);
     sense.g_vel.set_xy(0.0, 0.0);
     sense.heading   = 0.0;
     sense.d_heading = 0.0;
     sense.short_stamina   = 0;
     sense.long_stamina = 0;
     exec.g_cer = sense.g_cer     = 0.0;
     exec.heading = exec.d_heading = 0.0;
     g_cer = head_cer = 0.0;
     
     robot_state = new Robot_state(Cons2::delay(),
                                   Cons2::player_decay());
     moves = new Base_actuator;
     moves->target_speed = 0;
     moves->target_d_angle = 0;
     moves->action = AC_BASE_NONE;

     curr_view_angle = M_PI/2.0;
     
#ifdef TEST_SAMBA
     if(Cons2::debug_level_startup()) 
          cerr << "Ma_me: CONSTRUCTOR executed" << endl;
     
#endif
}

Ma_me::~Ma_me()
{
#ifdef TEST_SAMBA
     if(Cons2::debug_level_exit()) 
          cerr << "Ma_me: DESTRUCTOR executed" << endl;
#endif
}

// initializes pointers to modules whose data is to be read
void Ma_me::Init(EmptySamba *sambapointer)
{
     samba = sambapointer;
     sensor = (Sensor*)samba->get_pointer( SENSOR_ID );
     ac_base = (Ac_base*)samba->get_pointer( AC_BASE_ID );
     ac_kick = (Ac_kick*)samba->get_pointer( AC_KICK_ID );
     vs_referee = (Vs_referee*)samba->get_pointer( VS_REFEREE_ID );
}

// by this function, player unum can be told to me marker
// maybe this is not needed anymore?
void Ma_me::set_unum(int number)
{
     static int already_called = 0;
     if(already_called) {
          cerr << __FILE__ << " " << __LINE__ << endl;
          exit(1);
     }
     already_called = 1;
     unum = number;
}

void print_obj_name(int num)
{
    switch(num){
    case 1000: cerr << " own goal ";
        break;
    case 1001: cerr << " opp goal ";
        break;
    default: cerr << " flag: " << num << " ";
    }
}


// finds 0-2 closest objects on the field. (flags and goals)
void Ma_me::get_closest_objects()
{
     // when this function is called, flags and goal must already
     // been read!! 
     int i;

     close_1_dist = close_2_dist = 1000.0;
     close_2_x = 80;
     close_1_x = 80;
     int close_1_flag = -1, close_2_flag = -1;
     
     for(i = 0; i < NFLAGS; i++){
          if( flag[i].valid ){
               if( flag[i].pos.get_len() < close_1_dist ){
                    close_2_dist = close_1_dist;
                    close_2_dir = close_1_dir;
                    close_2_x = close_1_x;
                    close_2_y = close_1_y;
                    close_2_flag = close_1_flag;
                    close_1_dist = flag[i].pos.get_len();
                    close_1_dir = flag[i].pos.get_dir();
                    close_1_flag = i;
                    Cons2::flag_coords(FLAGTYPES(i),close_1_x, close_1_y );
               }
               else if( flag[i].pos.get_len() < close_2_dist ){
                    close_2_dist = flag[i].pos.get_len();
                    close_2_dir = flag[i].pos.get_dir();
                    Cons2::flag_coords(FLAGTYPES(i),close_2_x, close_2_y );
                    close_2_flag = i;
               }
          }
     }

     

     if( own_goal.valid ){
          if( own_goal.pos.get_len() < close_1_dist ){
               close_2_dist = close_1_dist;
               close_2_dir = close_1_dir;
               close_2_x = close_1_x;
               close_2_y = close_1_y;
               close_2_flag = close_1_flag;
               close_1_dist = own_goal.pos.get_len();
               close_1_dir = own_goal.pos.get_dir();
               close_1_x = MIN_X;
               close_1_y = 0;
               close_1_flag = 1000;
          }
          else if( own_goal.pos.get_len() < close_2_dist ){
               close_2_dist = own_goal.pos.get_len();
               close_2_dir = own_goal.pos.get_dir();
               close_2_x = MIN_X;
               close_2_y = 0;
               close_2_flag = 1000;
          }
     }
     if( opp_goal.valid ){
          if( opp_goal.pos.get_len() < close_1_dist ){
               close_2_dist = close_1_dist;
               close_2_dir = close_1_dir;
               close_2_x = close_1_x;
               close_2_y = close_1_y;
               close_2_flag = close_1_flag;
               close_1_dist = opp_goal.pos.get_len();
               close_1_dir = opp_goal.pos.get_dir();
               close_1_x = MAX_X;
               close_1_y = 0;
               close_1_flag = 1001;
          }
          else if( opp_goal.pos.get_len() < close_2_dist ){
               close_2_dist = opp_goal.pos.get_len();
               close_2_dir = opp_goal.pos.get_dir();
               close_2_x = MAX_X;
               close_2_y = 0;
               close_2_flag = 1001;
          }
     }
/*     cerr << "close 1 object:";
     print_obj_name(close_1_flag);
     cerr << "close 2 object:";
     print_obj_name(close_2_flag);*/
}
// uses closest objects and heading information to calculate global position
int Ma_me::compute_position()
{
    int i;
    
    pos_calc.reset();
    pos_calc.set_heading(heading);
        
    for(i = 0; i < NFLAGS; i++)
        if( flag[i].valid ){
            float x,y;
            Cons2::flag_coords(FLAGTYPES(i),x, y );
            pos_calc.new_object(flag[i].pos,x,y);
        }
    if( own_goal.valid ) pos_calc.new_object(own_goal.pos,-MAX_X,0.0);
    
    if( opp_goal.valid ) pos_calc.new_object(opp_goal.pos,MAX_X,0.0);
    Vector temp;
    
    temp = pos_calc.get_position();
    if(temp.get_x() > 1000.0){
        g_cer *= Cons2::cer_factor();
        return 0;
    } else {
        g_loc = temp;
//        g_loc.err_print_xy();
//        cerr << endl;
        
        g_cer = 1;
    }
    
/*     float x1,x2,y1,y2;
 
     if( close_1_dist < 1000.0 ){
               
          x1 = close_1_x - close_1_dist * cos(heading + close_1_dir);
          y1 = close_1_y - close_1_dist * sin(heading + close_1_dir);
          
          if(close_1_dist > 10.0 && close_2_dist < 1000.0) {
               x2 = close_2_x - close_2_dist * cos(heading + close_2_dir);
               y2 = close_2_y - close_2_dist * sin(heading + close_2_dir);
               g_loc.set_xy( (x1+x2)/2.0, (y1+y2)/2.0);
               g_cer = 0.7 + 0.3*20.0/(close_1_dist + close_2_dist);
               if(g_cer > 1.0) g_cer = 1.0;
               if(g_cer < 0.7) g_cer = 0.7;
          } else {
               g_loc.set_xy( x1,y1 );
               if(close_1_dist > 10.0 ){
                    g_cer = 0.7 + 0.3*10.0/close_1_dist;
               } else g_cer = 1.0;
               if(g_cer > 1.0) g_cer = 1.0;
               if(g_cer < 0.7) g_cer = 0.7;
          }
     } else {
          g_cer *= Cons2::cer_factor();
          return 0; // return 0 to acknowledge that position not calculated
     }
*/
    
    
    return 1;
}

// calculates heading basing on the line given in input 
void Ma_me::calculate_heading(LINETYPES t, Line_sensor &thisline)
{
     float dir;
     switch(t){
     case TOP:    dir = 0.0; break;
     case LEFT:   dir = 0.5 * M_PI; break;
     case BOTTOM: dir = M_PI; break;
     case RIGHT:  dir = 1.5 * M_PI;break;
     case CENTER: cerr << "Cannot calculate direction from centerline" << endl;
     } 
     if(thisline.dir > 0.0) dir += M_PI;
     heading = dir - thisline.dir;
     if(fabs(heading) >= 10.0)
         cerr << __FILE__ << " " << __LINE__ << " heading > 10!" << endl;
     normalize_angle(heading);
}

void max_speed_for_stamina(int min_s,int max_s,int curr_s, // stamina limits and curr stamina
                           float min_spd,float max_spd, // maximum and minimum speed for stamina interval
                           float old_ms,float &new_ms){ // last max_speed and new max_speed
    // if stamina not in this interval
    if(!(curr_s > min_s && curr_s <= max_s)) return;
    if(old_ms > max_spd) new_ms = max_spd;
    else if(old_ms < min_spd) new_ms = min_spd;
    return;
}



// changes max_curr_speed variable of the actionmap class
void Ma_me::limit_actionmap_max_speed()
{
    static float old_ms = 0.0, new_ms = 0.0; // old and new max speed
    
    // When short stamina decreases under 0.3*max_stamina (== 600),
    // effort and stamina recovery goes rapidly down, that is why it
    // is prevented

    max_speed_for_stamina(1600, 2000, sense.short_stamina,
                          1.0, 1.0,
                          old_ms, new_ms);
    max_speed_for_stamina(1400, 1600, sense.short_stamina,
                          0.99, 1.0,
//                          0.8, 1.0,
                          old_ms, new_ms);
    max_speed_for_stamina(1000, 1400, sense.short_stamina,
                          0.8, 0.99,
//                          0.5, 0.8,
                          old_ms, new_ms);
    max_speed_for_stamina(600, 1000, sense.short_stamina,
                          0.3, 0.8,
//                          0.2, 0.5,
                          old_ms, new_ms);
/*    max_speed_for_stamina(1600, 2000, sense.short_stamina,
                          1.4, 1.4,
                          old_ms, new_ms);
    max_speed_for_stamina(400, 1600, sense.short_stamina,
                          0.3, 1.4,
                          old_ms, new_ms);
*/
    old_ms = new_ms;
    Actionmap::set_max_speed(new_ms);
}

// computes global position and heading on the field
// if heading calculated, heading_calculated is set to 1
// if position calculated g_pos_calculated is set to 1
void Ma_me::compute_curr_position()
{
    // fing nearest object whose position is well known
    get_closest_objects();
    
    // compute robot heading using line data. The first thing to do 
    // is to find the furthest valid line 
    float furthest = -10000.0; // distance to furthest line
    int fl = -1; // index to furthest line
    int i;
    
    for( i = 0; i < NLINES ; i++ )
        if(line[i].valid && line[i].pos.get_len() > furthest){
            furthest = line[i].pos.get_len();
            fl = i;
        }
    // if line found calculate heading using line data
    if(fl >= 0){ 
        calculate_heading((LINETYPES)fl,line[fl]);
        heading_calculated = 1;
    }
    else if(close_1_dist < 1000.0 && close_2_dist < 1000.0) {
        // first make global and relative vectors of two nearest objects
        Vector g_one(close_1_x,close_1_y),r_one;
        Vector g_two(close_2_x,close_2_y),r_two;
        
        r_one.set_dir_len(close_1_dir, close_1_dist);
        r_two.set_dir_len(close_2_dir, close_2_dist);
        
        g_one = g_two - g_one;
        r_one = r_two - r_one;
        
        heading = g_one.get_dir() - r_one.get_dir();
        heading_calculated = 1;
    } else {
        // if no valid lines, use robot_state object to approximate current 
        // heading
        static Vector dummy_g_loc, dummy_g_vel;
        robot_state->get_curr_params( dummy_g_vel, 
                                      dummy_g_loc, heading);
        heading_calculated = 0;
    }
    // use closest objects and heading information to calculate
    // current position on the field
    if(compute_position()) g_pos_calculated = 1;
}

// updates robot_state object with new computed data
void Ma_me::update_robot_state()
{
    robot_state->set_curr_time(curr_time,referee.time_running);
    if(g_pos_calculated) robot_state->set_curr_g_pos(g_loc);
    if(heading_calculated) robot_state->set_curr_heading(heading);
    if(speed_calculated) robot_state->set_curr_speed(body.speed);
}

// updates data structures that are used by other modules (sense and exec)
void Ma_me::update_sense_and_exec()
{
     float delta = exec.heading;
     robot_state->get_exec_params(exec.g_vel, exec.g_loc, exec.heading);
     exec.d_heading = exec.heading - delta;

     delta = sense.heading;
     robot_state->get_curr_params(sense.g_vel, sense.g_loc, 
                                  sense.heading);
     sense.d_heading = sense.heading - delta;

     if(heading_calculated && fabs(heading-sense.heading)>0.01){
         cerr << "HERE: calculated heading: " << heading << " Got heading " << sense.heading << endl;
     }

     
     // next three lines of code added 18.3.1998 by jp
     // there was problems in position calculation. sometimes robot thought
     // that it knows its place very well but in fact it was way out of its
     // route because heading was not calculated for a long time. This could
     // happen when the robot was out of the field facing out of the field
     // and thought that it was on the field.
     if(heading_calculated) head_cer = 1.0;
     else head_cer *= Cons2::cer_factor();
     if(head_cer < 0.1) g_cer = 0.0;
     
     sense.g_cer = g_cer;
     exec.g_cer = g_cer;

     exec.heading_cer /= Cons2::cer_factor();
     if(exec.heading_cer > 1.0) exec.heading_cer = 1.0;
     sense.heading_cer = exec.heading_cer;
     

     sense.unum = exec.unum = Cons2::me_unum();
     if(exec.catched){ 
         exec.catched++;
         if(exec.catched > Cons2::goalie_catch_interval()){
             exec.catched = 0;
         }
     }
     
             
     sense.valid = exec.valid = 1;
}

void Ma_me::read_sens(Me_marker &to)
{
    check_executed();
    to = sense;
}

void Ma_me::read_exec(Me_marker &to)
{
    check_executed();
    to = exec;
}

// this function returns current view angle in radians
// mainly used by other modules
float Ma_me::read_view_angle()
{
    check_executed();
    
    if(body.valid)
        switch(body.view_angle){
        case VA_Narrow: curr_view_angle =  M_PI/4.0;
        case VA_Normal: curr_view_angle = M_PI/2.0;
        case VA_Wide: curr_view_angle = M_PI;
        }
    
    return curr_view_angle;
}

void Ma_me::invalidate_all()
{
    sense.valid = 0;
    exec.valid = 0;
}

void Ma_me::read_inputs()
{
    sensor->read_sense_body(body);
    if( body.valid ) speed_calculated = 1;
    if( sensor->is_any_valid() ){
        sensor->read_flag_sensors(flag);
        sensor->read_goal_sensors(opp_goal, own_goal);
        sensor->read_line_sensors(line);
    }
    ac_base->no_trigger_read(*moves);
    if(ac_kick->catched()) exec.catched++;
    vs_referee->read(referee);
}

void Ma_me::update_stamina_model()
{
    if(body.valid){
        sense.short_stamina = (int)body.short_stamina;
        sense.long_stamina = (int)body.long_stamina;
        exec.short_stamina = (int)sense.short_stamina - (100 - 20)*Cons2::delay();
        exec.long_stamina =  (int)sense.long_stamina;
    }
}


int Ma_me::transform()
{
    if( !Cons2::delay() ){
//        cerr << "No-delay-mode not supported by Ma_me!!!" << endl;
    }

    g_pos_calculated = speed_calculated = heading_calculated = 0;
    
    read_inputs();
    update_stamina_model();
    limit_actionmap_max_speed();
    
    if(moves->action == AC_BASE_DRIVE){
        robot_state->set_target_speed( moves->target_speed );
    }
    else if(moves->action == AC_BASE_TURN){
        robot_state->set_target_d_head( moves->target_d_angle );
        sense.heading_cer = exec.heading_cer = 0.5;
    }
    
    if(sensor->is_any_valid()) 
        compute_curr_position();
    update_robot_state();
    update_sense_and_exec();

#ifdef TEST_SAMBA
// write to debugfile     
     if(Cons2::debug_level() > 3)
         cerr << "  Ma_me: My location: " << sense.g_loc.get_x() << ", " << sense.g_loc.get_y() << endl;
#endif
     return 0;
}
