// -*- mode:C++ -*-                                     
// File:   Ma_me.h                             
// Author: Jussi Pajala 1.5.1997,  jussip@tks.oulu.fi   
//
// this file declares class of the me marker


#ifndef MA_ME_H
#define MA_ME_H

#include "Vector.h"

struct Me_marker
{
    Vector g_loc;     // global location
    Vector g_d_loc;   // global delta location
    Vector g_vel;     // global velocity
    Vector g_d_vel;   // location difference to old location
    float heading;    // robot's heading
    float d_heading;  // difference of heading to old heading
    
    int short_stamina;  // robot's current stamina
    int long_stamina;

    int catched;      // if player has tried to catch the ball
    // when dropped to zero catching is allowed again
    
    int unum;         // uniform number (NECESSARY ???)
    float g_cer;      // certainty of variables
    float heading_cer;
    int valid;
    
};

class Ma_me;


#include "Module.h"
#include "Ab_filter.h"
#include "Robot_state.h"

#include "Sensor.h"
#include "Ac_base.h"
#include "Ac_kick.h"
#include <Vs/Vs_referee.h>

const float NORM_ALPHA = 0.8;

class Ma_me:public Module
{
public:
    Ma_me();
    ~Ma_me();
    virtual void reset() {;};
    void Init(EmptySamba *sambapointer);
    
    void set_unum(int number);  // called once from soccer constructor
    
    void read_sens(Me_marker &to);// robot state computed from sensors
    void read_exec(Me_marker &to);// estimated robot state when next
                                  // command is executed
    void read_pos_calc_objects(float &x1, float &y1, float &x2, float &y2){
        x1 = close_1_x; y1 = close_1_y;
        x2 = close_2_x; y2 = close_2_y;
    }
    float read_view_angle(); // returns total view angle in radians     
     
protected:
    int  transform();      // called by Module::execute()
    void get_closest_objects();
    int  compute_position();
    void advance_position();
    void calculate_heading(LINETYPES t, Line_sensor &thisline);
    void compute_position_and_advance();
    void get_position_from_interface();
    void invalidate_all(); // invalidates all data produced by this module
    
    void read_inputs(); // read inputs from sensor and ac_base outputs
    void limit_actionmap_max_speed();
    void compute_curr_position();
    void update_stamina_model();
    void update_robot_state();
    void update_sense_and_exec();
    
// pointers to modules whose data this module needs
    Sensor *sensor;   // *sensor_ssig_in *flags_in *opp_goal_in *own_goal_in;
    Ac_base *ac_base; // *impulses_in;
    Ac_kick *ac_kick;
    
    Vs_referee *vs_referee;
        
    Goal_sensor opp_goal;
    Goal_sensor own_goal;
    Flag_sensor flag[NFLAGS];  
    Line_sensor line[NLINES];
    Sense_body  body;
    Base_actuator *moves;
    float curr_view_angle; // angle in radians
    
    Referee_vs referee;
    
    float heading;
    Vector g_loc;
    float g_cer;
    float head_cer;
    
    int unum;
    Me_marker sense; // robot state computed from sensors
    Me_marker exec;  // estimated robot state when next command is executed 
    
    float close_1_dist, close_2_dist, close_1_dir, close_2_dir;
    float close_1_x, close_2_x, close_1_y, close_2_y;
    
    int g_pos_calculated, speed_calculated, heading_calculated;
    
    Robot_state *robot_state;
};
#endif

