// file    : Ma_teammate.cc   

#include "decls.h"
//#include "Soccer.h"
#include <Ma/Ma_opponent.h>
#include <Ma/Ma_teammate.h>
#include <assert.h>

const float FILTER_MIN_DIST = 10.0; // minimum distance where filter can be used

Ma_teammate::Ma_teammate(int DO_NOTHING )
{
    if( DO_NOTHING ) return;
  
    my_id = MA_TEAMMATE_ID;
    inhibited = NO;
     
    int i;
    
    for(i = 0; i < 12; i++){
        use_filter[i] = 1;
        marker_team[i].cer = 0.0;
        marker_team[i].norm_dist = 10000.0;
    }
    for(i = 0; i < 11;i++){
        all.player[i].cer = 0.0;
        all.order[i] = i;
        all.player[i].unum = i+1;
    }
    
    
    marker_side = Side_L;   // this marker is for teammates
    if(Cons2::debug_level_startup()) cerr << "Ma_teammate: CONSTRUCTOR executed" << endl;
}


Ma_teammate::~Ma_teammate()
{
    if(Cons2::debug_level_exit()) cerr << "Ma_teammate: DESTRUCTOR executed" << endl;
}

void Ma_teammate::Init(EmptySamba *sambapointer)
{
    samba = sambapointer;
    sensor = (Sensor*)samba->get_pointer( SENSOR_ID );
    ma_me = (Ma_me*)samba->get_pointer( MA_ME_ID );
    vs_team_audio = (Vs_team_audio*)samba->get_pointer( VS_TEAM_AUDIO_ID );
    ma_opponent = (Ma_opponent*)samba->get_pointer( MA_OPPONENT_ID );
}


// Function read_players_from_sensors()
//
// reads sensor data and fills plrs array with those players who might
// belong to marker_side team
// for each players following parameters are calculated:
//    loc
//    vel
//    g_loc
//
void Ma_teammate::read_players_from_sensors()
{
//    int unum;
//    Side side;
    int i,k;
    
    All_players opp; // N_MA_OPP > N_MA_TEAM
    
    if(marker_side == Side_L){ 
        // if teammate marker read opponent marker
        ma_opponent->read(opp);
        
        if(!opp.valid){
            cerr << __FILE__ << " " << __LINE__ << endl
                 << "   Opponent marker not valid!" << endl;
            exit(1);
        }
    }   

    vs_team_audio->read(audio);
    sensor->read_player_sensor(player_sensor);
    
    plr_count = 0;
    
    for( i = 0; i < MAX_SEEN_PLAYERS; i++){
        
        if(plr_count >= MAX_SEEN_PLAYERS){
            cerr << __FILE__ << " " << __LINE__ << endl;
            exit(1);
        }
        if(!player_sensor[i].valid) continue;
        // if some player is nearby and its side is not known, we think it is
        // an opponent
        if(player_sensor[i].side == Side_none &&
          player_sensor[i].pos.get_len() < 4.0) player_sensor[i].side = Side_R;
        // if side is not marker side and side is not none, it is other
        // markers side
        if(player_sensor[i].side != marker_side && 
           player_sensor[i].side != Side_none) continue;
        //if(unum > 11) continue; // if empty element, continue ( when empty, unum is 999 )
        
        
        // if side not received from sensors and marker is for team players
        if( player_sensor[i].side == Side_none && marker_side == Side_L) {
            int marked_as_opponent = 0;
            for( k = 0; k < NOPPS; k++ ) 
                if( opp.player[k].loc == player_sensor[i].pos )
                    marked_as_opponent = 1;
            if(marked_as_opponent) continue;
        }
        
//        if(marker_side == Side_L && player_sensor[i].side == Side_none)
//            cerr << "WHY WE ARE HERE???" << endl;
        
    

    
        Sensor_side[plr_count] = player_sensor[i].side; // this is Side_L or Side_none for team marker and
        // Side_R or Side_none for opponent marker
// 1.1
        if( !(plr_count < MAX_SEEN_PLAYERS) )
            cerr << "plr_count " << plr_count << endl;
        assert( plr_count <  MAX_SEEN_PLAYERS );
        plrs[plr_count].unum = player_sensor[i].unum;
          
        // if it is team-mate, or side is unknown calculate
        // copy location and velocity from sensors
        plrs[plr_count].loc = player_sensor[i].pos;
        plrs[plr_count].vel = player_sensor[i].vel; 
// 1.2
        plrs[plr_count].g_loc = loc2g_loc(plrs[plr_count].loc,me_sense.g_loc,
                                          me_sense.heading);
        plr_count++; // increase team-mate counter
        // plr_count may become too big here, but it is checked below when this loop is over
    }
    // now check if plr_count is too big
    // (it may become too big if all players are seen, it is not a bug,
    // it is a property :)
    if(plr_count >= MAX_SEEN_PLAYERS ) plr_count = MAX_SEEN_PLAYERS-1;
    
}

// Function find_plrs_with_unum()     
//
// copies players whose unum is known from plrs[] to marker_team[]
// and marks identified[unum] = 1
void Ma_teammate::find_plrs_with_unum()
{
    int i;
    
    for (i = 0; i < 12; i++) identified[i] = 0;

    for( i = 0; i < plr_count; i++){
        assert( plr_count <  MAX_SEEN_PLAYERS );
        int unum = plrs[i].unum;
        if( unum != 0){
            assert( unum < 12 );
            if(marker_team[unum].cer < 0.001){ // if certainty was very low, reset filters!
                filt_g_x[unum].reset();
                filt_g_y[unum].reset();
            }
            marker_team[unum] = plrs[i];
            marker_team[unum].cer = 1.0;
            identified[unum] = 1;
            if(Cons2::debug_level() > 3){
                cerr << "PLAYER IDENTIFIED, UNUM: " << unum;
                if(unum > 9) {
                    cerr << "certainty is: " << marker_team[unum].cer;
                }
                cerr << endl;
            }
        }
    }
}

// Function predict_and_identify()
//
// only for those players whose unum is not seen:
// uses filters to predict current location and finds the closest
// match to predicted location from plrs[] array 
// If good enough match is not found, use predicted location and
// reduce certainty
//
void Ma_teammate::predict_and_identify()
{
    float min_dist,dist;
    int min_index, i,j;
    float x,y,vx,vy;
    
    for(i = 1; i < 12; i++)
        if(!identified[i]){
            min_dist = 10.0; // don't accept bigger errors than 10.0 meters
            min_index = 100;
            // get predicted location
            filt_g_x[i].predict(curr_time,0,x,vx);
            filt_g_y[i].predict(curr_time,0,y,vy);
            // i is index of the unidentified player in team array
            // j is index to plrs array when seeking for closest player for unidentified player
            for(j = 0; j < plr_count; j++){
                // if unum is not zero, player is already identified => continue,
                // else find closest match to filter prediction
                assert( j < MAX_SEEN_PLAYERS);
                if(plrs[j].unum) continue;
                
                // calculate distance between predicted and seen position (prediction error)
                dist = sqr(x-plrs[j].g_loc.get_x()) + sqr(y-plrs[j].g_loc.get_y());
                if(marker_side == Side_L && Sensor_side[j] != marker_side)
                    continue;
                // if players side is unknown, double prediction error
                if(Sensor_side[j] == Side_none) dist *= 2.0;
                // if calculated prediction error (dist) is smaller than previously calculated,
                // save this distance and index
                if(dist < min_dist){
                    min_dist = dist;
                    min_index = j;
                }
            }
            if(min_index >= plr_count ){
// 2.3a
                //ei lydy sopivaa - keksi jotain...;
                
                use_filter[i] = 1; // must use filters if there is no sensorinfo
                
                // check if audio can be used to find players
                int audio_useful = 0;
                
                if(marker_side == Side_L){
                    audio_useful = 0;
                    //gpos2coords(audio.team_g_loc[i], audio_x, audio_y);
                } else {
                    audio_useful = audio.opp_valid[i-1];
                }
                
                assert( i < 12);
                if( audio_useful && marker_team[i].cer < 0.5 ){
                    filt_g_x[i].reset();
                    filt_g_y[i].reset();

                    if(marker_side == Side_L){
                        ;// use team plyer audio information
                    } else {
                        marker_team[i].g_loc = audio.opp_g_loc[i-1];
                        marker_team[i].g_vel = audio.opp_g_vel[i-1];
                        
                        filt_g_x[i].smooth(curr_time,
                                           marker_team[i].g_loc.get_x(),x,vx);
                        filt_g_y[i].smooth(curr_time,
                                           marker_team[i].g_loc.get_y(),y,vy);
                    }            
                }
                
                // use predicted location and velocity
                marker_team[i].g_loc.set_xy(x,y);
                marker_team[i].g_vel.set_xy(vx,vy);
                
                // calculate player's local location
                marker_team[i].loc = marker_team[i].g_loc - me_sense.g_loc;
                marker_team[i].loc.turn(-me_sense.heading);
                
                // calculate player's local velocity
                marker_team[i].vel = marker_team[i].g_vel - me_sense.g_vel;
                marker_team[i].vel.turn(-me_sense.heading);
                
                marker_team[i].cer *= Cons2::cer_factor(); // degrease certainty
                //if(my_id == MA_TEAMMATE_ID )
                //    cerr << "Filter used for player" << i << endl;
                
            } else {
// 2.3b
                assert( min_index < MAX_SEEN_PLAYERS);
                plrs[min_index].unum = i; // put unum to plrs array to ensure that the same player
                // is not tought to be two different players 
                assert( i < 12);
                marker_team[i] = plrs[min_index];
                identified[i] = 1;       // mark player as identified
                if( Sensor_side[min_index] == marker_side){
                    marker_team[i].cer = 0.5;
                }
                
                else{
                    marker_team[i].cer = 0.25;
                }
                
            }
        }
}



void Ma_teammate::quess_unidentified()
{
    int i,j;
    
    for(i = 0; i < plr_count; i++){
        if(plrs[i].unum) continue; // if unum is known player it is already identified!
        if(Sensor_side[i] == Side_none) continue;
        for( j = 1; j < 12; j++){
            if(identified[j]) continue;
            if(marker_team[i].cer > 0.02) continue;
            
            plrs[i].unum = j;
            marker_team[j] = plrs[i];
            marker_team[j].cer = 0.3;
            // reset filters because player number has been quessed
            filt_g_x[j].reset();
            filt_g_y[j].reset();

            identified[j] = 1;
            break;
        }
    }
}

void Ma_teammate::filter_and_compute_positions()
{
    float x,y,vx,vy;
    int i;
    
    
    for(i = 1; i < 12; i++)
        if( identified[i] ){
            // get predicted location
            filt_g_x[i].predict(curr_time,0,x,vx);
            filt_g_y[i].predict(curr_time,0,y,vy);
            // if filter error is too big then reset filters
            if(sqr(marker_team[i].g_loc.get_x()-x)+
               sqr(marker_team[i].g_loc.get_y()-y) > 10){
                filt_g_x[i].reset();
                filt_g_y[i].reset();
            }
            
            
            // filter global location, filtered location and
            // velocity is stored to (x,y) and (vx,vy)
            // filtering is done even if filters are not used. So when player goes
            // so far that filters can be used, they are ready
            filt_g_x[i].smooth(curr_time,marker_team[i].g_loc.get_x(),x,vx);
            filt_g_y[i].smooth(curr_time,marker_team[i].g_loc.get_y(),y,vy);
            
            if(marker_team[i].loc.get_len() > FILTER_MIN_DIST){
                // use filter outcome as teammate global location if teammate is measured
                // to be further than FILTER_MIN_DIST
                use_filter[i] = 1;
                marker_team[i].g_loc.set_xy(x, y);
            }
            else {
                // else use global location as calculated above, stop using filters!
                use_filter[i] = 0;
            }
            
            // next calculate global velocity of the teammate
            // if measured local velocity vector is not zero or
            // if filter is not in use then use use the measured velocity value
            // else use velocity vector of the filter outcome
            if(marker_team[i].vel.get_len() != 0.0 || !use_filter){
                marker_team[i].g_vel = loc2g_loc(marker_team[i].vel, 
                                                 me_sense.g_vel,
                                                 me_sense.heading);
            } else {
                // global velocity
                marker_team[i].g_vel.set_xy(vx,vy);
                // calculate local velocity from filtered global
                // velocity
                marker_team[i].vel = loc2g_loc(marker_team[i].g_vel, 
                                               me_sense.g_vel,
                                               me_sense.heading);
            }
        }
}

void Ma_teammate::approximate_position()
{
    float x,y,vx,vy;
    int i;//,j;
     
    for( i = 1; i< 12; i++) {
        if( marker_side == Side_L && i == Cons2::me_unum() ) continue;
        
        // no sensorinfo for the teammates
        // try to approximate teammate's location
        
        if(use_filter[i]){ // teammate is so far away that filter must be used
// 4.1a               
            // predict new global location of the teammate
            filt_g_x[i].predict(curr_time,0,x,vx);
            filt_g_y[i].predict(curr_time,0,y,vy);
            
            if(Cons2::debug_level() > 6) cerr << "Filter predict: (" << x << "," << y << ") ("
                                     << vx << "," << vy << ")" << endl;
            marker_team[i].g_loc.set_xy(x,y);
            marker_team[i].g_vel.set_xy(vx,vy);
        } else { // teammate was so near that filter is not reliable
// 4.1b            // move teammate by amount of its speed
            marker_team[i].g_loc = marker_team[i].g_loc + marker_team[i].g_vel;
        }
// 4.2          
        // calculate ball's local location
        marker_team[i].loc = marker_team[i].g_loc - me_sense.g_loc;    
        marker_team[i].loc.turn(-me_sense.heading);// turn ball_loc by amount of robot's global angle
        
        // calculate ball's local velocity
        marker_team[i].vel = marker_team[i].g_vel - me_sense.g_vel;
        marker_team[i].vel.turn(-me_sense.heading);// turn ball_vel by amount of robot's global angle
        
        // reduce certainties
        marker_team[i].cer   *= Cons2::cer_factor();
    }
}


void Ma_teammate::identify_players()
{
// 2.1
    find_plrs_with_unum();
    // fill 'my' place in array, no teammate can have the same number with this robot
    if(marker_side == Side_L) identified[ Cons2::me_unum() ] = 1;
// 2.2
    predict_and_identify();
// 2.4
    // fill empty places in team array with those players in plrs array who are known
    // to be our side but who are not identified
    quess_unidentified();
}


void Ma_teammate::sort_output_array()
{
    int i;
    
    int smooth[12];
    for( i = 1; i < 12; i++){
        smooth[i] = 0;
    }
    //for (i = 0; i < 11; i++) cerr << "Order[" << i << "] = " << all.order[i] << endl;
      
    // mark 5 closest player to be smoothed
    // so we can keep same players to be important for a longer period
    for(i=0; i < 5; i++) smooth[1 + all.order[i]] = 1;
    for(i = 1; i < 12; i++){
        if(marker_side == Side_L && i == Cons2::me_unum()) continue;  // if this is own team marker, don't accept 'me'
        if(marker_team[i].cer == 0.0){
            marker_team[i].norm_dist = 9999999;
            continue;
        }
        marker_team[i].norm_dist 
            = (1.0 / ( marker_team[i].cer > 0.00001 ? 
                       marker_team[i].cer : 0.00001 ))
            * (fabs(marker_team[i].loc.get_dir()) + 1.0)
            * marker_team[i].loc.get_len();
        
        if(smooth[i]) marker_team[i].norm_dist *= 0.5;
    }
    
    for(i = 0; i < 11;i++) all.order[i] = -1;
    int k,n_closer;
    
    for(i = 0; i < 11; i++){
        n_closer = 0;
        for(k = 0; k < 11; k++){
            if(k == i) continue;
            if(marker_team[k+1].norm_dist < marker_team[i+1].norm_dist)
                n_closer++;
        }
        // if there already is some player with same norm_dist, move this
        // new robot one step further
        while(all.order[n_closer] >= 0) n_closer++;
        all.order[n_closer] = i;
    }
    for (i = 0; i < 11;i++) if(all.order[i] < 0) {
        cerr << "Order < 0!" << endl;
        for (i = 0; i < 11; i++) cerr << "Order[" << i << "] = " << all.order[i] << endl;
        exit(1);
    }
}

void Ma_teammate::advance_positions()
{
    // when we come here players are in marker_team array. Now their
    // state must be advanced to the state they are at execution time
    // and moved to 'all' structure 
    int i;
    static Vector temp;
     
    for(i = 0; i < 11; i++){
        all.player[i].g_loc = marker_team[i+1].g_loc 
            + Cons2::delay()*marker_team[i+1].g_vel;
        all.player[i].g_vel = marker_team[i+1].g_vel;
        all.player[i].loc = g_loc2loc( all.player[i].g_loc, me_exec.g_loc,
                                       me_exec.heading );
        all.player[i].vel = g_loc2loc( all.player[i].g_vel, me_exec.g_vel,
                                       me_exec.heading );
        all.player[i].mix_vel= vel2mix_vel( all.player[i].g_vel,
                                            me_exec.heading );
        all.player[i].cer = marker_team[i+1].cer;
        all.player[i].norm_dist = marker_team[i+1].norm_dist;
    }
    
    all.valid = 1;
}



void Ma_teammate::trace()
{
/*    if(Cons2::debug_file_ma_teammate()){
        // write position of all the teammates to file
        if(Cons2::debug_level()>5) cerr << "writing to trc file" << endl;
        // write to debugfile     
        trc((int)my_id, 1, curr_time,
            marker_team[1].g_loc.get_x(), marker_team[1].g_loc.get_y(),
            marker_team[2].g_loc.get_x(), marker_team[2].g_loc.get_y(),
            marker_team[3].g_loc.get_x(), marker_team[3].g_loc.get_y(),
            marker_team[4].g_loc.get_x(), marker_team[4].g_loc.get_y(),
            marker_team[5].g_loc.get_x(), marker_team[5].g_loc.get_y(),
            marker_team[6].g_loc.get_x(), marker_team[6].g_loc.get_y(),
            marker_team[7].g_loc.get_x(), marker_team[7].g_loc.get_y(),
            marker_team[8].g_loc.get_x(), marker_team[8].g_loc.get_y(),
            marker_team[9].g_loc.get_x(), marker_team[9].g_loc.get_y(),
            marker_team[10].g_loc.get_x(), marker_team[10].g_loc.get_y(),
            marker_team[11].g_loc.get_x(), marker_team[11].g_loc.get_y(),
            
            marker_team[1].cer,
            marker_team[2].cer,
            marker_team[3].cer,
            marker_team[4].cer,
            marker_team[5].cer,
            marker_team[6].cer,
            marker_team[7].cer,
            marker_team[8].cer,
            marker_team[9].cer,
            marker_team[10].cer,
            marker_team[11].cer );
        
        
            }*/
}


void Ma_teammate::read_player(int unum, Player_data &to)
{
    check_executed();
    to = all.player[unum-1];
}

void Ma_teammate::read(All_players &to)
{
    check_executed();
    to = all;
}

void Ma_teammate::invalidate_all()
{
    all.valid = 0;
}


int Ma_teammate::transform()
{
    int i;
    
    ma_me->read_exec(me_exec);
    ma_me->read_sens(me_sense);
    
    Side tempside;
    if(marker_side == Side_L) tempside = Side_R;
    else tempside = Side_L;
    
    // wrong side, if no other information!
    for( i = 0; i < MAX_SEEN_PLAYERS; i++) Sensor_side[i] = tempside;    
// 1
    // THIS FUNCTION IS DESCRIBED IN FILE Ma_teammate.txt
    // get teammate's new location or approximate it,
    if(sensor->is_any_valid()){
        read_players_from_sensors();
// 2
        // when we come here plrs array is filled with players who may be team-mates
        // we must try to identify players
        
        // first put players whose unum is known to the array and then
        // identify players whose unum is unknown by comparing predicted
        // location and measured location
        identify_players();
        
        // at this stage new sensordata is in marker_team[] array and positions
        // of those players who are not at sight are predicted with filters
        
// 3
        // now use filters to smooth positions and compute global velocity
        filter_and_compute_positions();
    } else {
        
// 4
        // if we come here, there is not new sensorinfo available
        // therefore update old information in marker_team array by
        // predicting new position with help of filters or old
        // velocity vector
        approximate_position();
    }
    
// 5
    // calculate normalized distance to each team-mate and sort output array so that
    // the closest team-mates comes the first and so on
    sort_output_array();
    
    // advance positions by amount of execution delay
    advance_positions();
    return 0;
}


