// file    : Samba.cc
// author  : Jussi Pajala
//
// v.0.5       6.1997  the first version
// v.0.8    5. 8.1997  lot's of enchangements, lot's of new modules
//                     execution engation!, execution timing, ...
// v.1.5   20. 1.1998  renamed from Soccer to samba
//                     totally new signalling system
//                     only a few modules included
// v.1.51   6. 2.1998  everything cleaned out, ready for new
//                     and development 
#include "decls.h"
#include "Samba.h"

//#define SOCCER_TIMERS 
#ifdef SOCCER_TIMERS
#include "fstream.h"
ofstream times_out;
#endif

extern Interface interface;

// Samba's constructor function
Samba::Samba ()
{
#ifndef NOPRINTS
     cout << "Samba  v2.0a0.0 (compiled " << __DATE__ << " " << __TIME__ 
          << ")" << endl << endl;
#endif
     if(Cons2::debug_level_startup()) cout << "Initializing..." << endl;
     my_state = INITIALIZING_SOCCER;
     
     // construct a FILO list (first-in-last-out)
     // of modules
     // Modules should be added to this list in reverse execution order
#ifdef VIS
     modulelist.add_module(&visual);
#endif
     modulelist.add_module(&ac_supply);
     modulelist.add_module(&ac_kick);
     modulelist.add_module(&ac_base);
     
     modulelist.add_module(&arbiter);
     
/*     modulelist.add_module(&ar_meta);
     modulelist.add_module(&ar_move);
     modulelist.add_module(&ar_kick);
     
     modulelist.add_module(&be_lookaround);
     modulelist.add_module(&be_turnto_ball);
     modulelist.add_module(&be_turnto_goal);*/ 

     modulelist.add_module(&be_move);
     modulelist.add_module(&be_kick);
     
     modulelist.add_module(&be_jumpto_defloc);
     modulelist.add_module(&be_shout);

     
     modulelist.add_module(&ma_goal);
     modulelist.add_module(&ma_ball);
     modulelist.add_module(&ma_me);// me marker is needed in all markers
     modulelist.add_module(&ma_opponent);
     modulelist.add_module(&ma_teammate);
     modulelist.add_module(&ma_defloc);
     modulelist.add_module(&ma_specloc);
// AT
     modulelist.add_module(&ma_offside);
   
     modulelist.add_module(&vs_referee);       // these virtual sensors are needed 
     modulelist.add_module(&vs_team_audio);
     modulelist.add_module(&sensor);           // sensors first
     
     // go through all the modules in the list 
     // and call every module's init method
     ModuleList *temp = &modulelist; // temporary pointer to a modulelist node
     connections = 0;
     while(temp != NULL){ 
          // check if pointer to a module of the node is null
          // if it is, somethong weird has happened; exit
          if(temp->GetModule() == NULL){
               printf("Module is NULL!!! %s, line: %d\n",__FILE__,__LINE__);
               exit(1);
          }   
          if(Cons2::debug_level_startup())
               cerr << Const::modulename(temp->GetModule()->GetId()) 
                    << ".init()" << endl;
 
          // Initialize the module and tell Samba's pointer
          temp->GetModule()->Init(this);
          temp->GetModule()->reset_executed();
          temp->GetModule()->reset_inhibited();
          // move temp to the next module of the list
          temp = temp->GetNext();
     }
#ifndef NOPRINTS
     cout <<"Total number of connections between modules: "<< connections << endl;
#endif
     // tell players number to me marker
//     ma_me.set_unum( interface.get_uniform_number() );
     
     my_state = SENSORS_ON;    // sensors active, all the rest inhibited 
     if(Cons2::debug_level_startup()) cerr << "...initialized" << endl;

     if( !Cons2::delay() )
         cout << "Delay is zero! Are you sure it works better than nonzero?" 
              << endl;
#ifdef SOCCER_TIMERS
     times_out.open("times.dat");
#endif
     if(Cons2::me_unum() != 1){
         interface.command_change_view(VA_Normal/*Wide*/, VQ_High);
         Cons2::set_visual_angle(VA_Normal/*Wide*/);
     } else {
         // goalkeeper needs wide visual sector
         interface.command_change_view(VA_Wide, VQ_High);
         Cons2::set_visual_angle(VA_Wide);
     }
     
     last_sensor_information = 0;
     if(Cons2::debug_level_startup()) cerr << "Samba: CONSTRUCTOR executed" << endl;
}


// Samba's destructor function
Samba::~Samba ()
{
     if(Cons2::debug_level_exit())
          cout << "Cleaning Samba..." << endl;
     
     /* cleanup devices (robot, sensors, displays,...)
      * and low level processes.
      */
#ifdef SOCCER_TIMERS
     times_out.close();
#endif    
     if(Cons2::debug_level_exit())
          cout << "...cleaned" << endl;
     
     if(Cons2::debug_level_exit())
          cerr<< "Samba: DESTRUCTOR executed" << endl;
}


void Samba::inhibit_modules(SYSTEM_STATE state)
{
    
    if (state == RESET_SOCCER) {
        reset();
        cout << "All modules resetted" << endl;
        // prevent reset occurring many times
        interface.change_control_system_state(ALL_ON);
        my_state = ALL_ON;
    }
    if( state == PAUSE_SOCCER ){
        ModuleList *temp = &modulelist; // temporary pointer to a modulelist node
        while(temp != NULL){ 
            temp->GetModule()->set_inhibited();
            temp = temp->GetNext();// move temp to the next module of the list
        }
        //sensor.reset_inhibited(); // sensors cannot be inhibited!!
        ma_me.reset_inhibited();
        ma_ball.reset_inhibited();
        ma_goal.reset_inhibited();
        vs_referee.reset_inhibited();
        vs_team_audio.reset_inhibited();
        ma_teammate.reset_inhibited();
        ma_opponent.reset_inhibited();
        ma_defloc.reset_inhibited();
        ma_specloc.reset_inhibited();
// AT
        ma_offside.reset_inhibited();

#ifdef VIS
        visual.reset_inhibited();
#endif
    }
    
    
    if (state == SENSORS_ON || state == QUIT_SOCCER) {
        // only sensor and marker modules are on
    }
    
    else if (state == LOOKAROUND_ON) {
        // inhibit some modules...
    }
    
    else if (state == BALL_BEHAVIORS_ON) {
        // inhibit some modules...
        ac_base.set_inhibited();
    }
    
    else if (state == GOAL_BEHAVIORS_ON) {
          // inhibit some modules...
    }
    
    else if (state == ALL_ON) {
          // here we shouldn't inhibit any modules?
    }
}


void Samba::check_sensor_timeout()
{
    if(interface.is_any_valid()){
        last_sensor_information = curr_time;
        return;
    }
        
    if(my_state != QUIT_SOCCER && last_sensor_information &&
       (curr_time - last_sensor_information) > 50){
        cerr << "Player " << Cons2::me_unum() 
             << " Exiting: Sensor timeout!" << endl;
        my_state = QUIT_SOCCER;
    }
}

        

// one cycle of the control system: execute all modules
void Samba::tick (){
     if(Cons2::debug_level() > 4)
          cerr << "Samba.tick!" << endl;

      curr_time = interface.get_time();
     // read new state from interface if user has changed it
     if( my_state != QUIT_SOCCER )
          my_state = interface.get_user_state();
     
     check_sensor_timeout();
     
     // inhibit modules based on the system state
     inhibit_modules(my_state);
     
     if(Cons2::debug_level() > 2) 
         cerr << "Player " << Cons2::me_unum() 
              << " Samba my_state: " << my_state  << endl;
     
     // EXECUTION PHASE
     // the modules are divided into following sets: sensors, markers, behaviors, arbiters and
     // actuators. scheduler follows the signal path, i.e. it executes the sets in the
     // order above.

     // sensor module must always be executed, because
     // it releases the lock of sensor data
     // if lock is not released, interface module gets jammed
     sensor.execute();
     
     // execute actuators
     ac_base.execute();
     ac_kick.execute();
     ac_supply.execute();
     
     // me and ball markers must always be executed after actuators
     // This is because these markers receive signals from the previous 
     // control cycle
     // Usually these modules are executed because some module needs
     // information from me and ball markers, but to ensure execution they 
     // must be called excplicitly here
     ma_me.execute();
     ma_ball.execute();
     
#ifdef VIS
     if(Cons2::Visual_execute)
          visual.execute();
#endif // VIS    
     
     if( vs_referee.game_is_over() )
         my_state = QUIT_SOCCER;
     
     
     ModuleList *temp = &modulelist; // temporary pointer to a modulelist node
     while(temp != NULL){ 
          temp->GetModule()->reset_executed();
          temp->GetModule()->reset_inhibited();
          temp = temp->GetNext();// move temp to the next module of the list
     }

     prev_time = curr_time;
}

Module* Samba::get_pointer( ID module_id )
{
     ModuleList *temp = &modulelist; // temporary pointer to a modulelist node
     // go through all the modules in the list 
     // and compare their ids to requested id
     while(temp != NULL){ 
          if(module_id == temp->GetModule()->GetId()){
               connections++;
               return temp->GetModule();
          }
          
          temp = temp->GetNext();// move temp to the next module of the list
     }
     cerr << "Serious error emerged in " << __FILE__
          << " line: " <<__LINE__<< endl;
     cerr << "Requested module id was not found: " 
          << Cons2::modulename(module_id) << endl;
     
     exit(1);
     //return NULL;
}

//Samba::reset resets everything, 
// including all the modules of the control system
void Samba::reset ()
{
     ModuleList *temp = &modulelist; // temporary pointer to a modulelist node
     // go through all the modules in the list 
     // and call every module's reset method
     while(temp != NULL){ 
          temp->GetModule()->reset();// Reset the module 
          temp = temp->GetNext();// move temp to the next module of the list
     }
}

/* end of file */



