#include "State_machine_gk.h"



Goal_keeper::Goal_keeper(Arbiter *arbiterpointer)
        :State_machine(arbiterpointer){
    strcpy(name, "Goal_keeper_state_machine");

    

    state_list->add_state((Prim_state*)new Turnto_ball  (arbiter)); // the least important
    state_list->add_state((Prim_state*)new Gk_special_situation(arbiter));
    state_list->add_state((Prim_state*)new Gk_goto_defloc(arbiter));
    state_list->add_state((Prim_state*)new Gk_catch_ball    (arbiter));
    state_list->add_state((Prim_state*)new Pass(arbiter) );
      state_list->add_state((Prim_state*)new KickFar(arbiter) );
    state_list->add_state((Prim_state*)new Gk_hold_ball(arbiter));
    state_list->add_state((Prim_state*)new Lookaround    (arbiter)); //the most important state
        
// in the beginning current state is special situation state ie. the state in
// the first node of the State_list
        curr_state = state_list->get_state();
}
   
Gk_goto_defloc::Gk_goto_defloc(Arbiter *arbiterpointer/*, Gk_catch_ball *gk_catch_ball_pointer*/)
        :State_machine(arbiterpointer){
    strcpy(name, "Goal_keeper_defloc");
    
//    state_list->add_state((Prim_state*)new Gk_avoid_players(arbiter, this) );     // the least important
    state_list->add_state((Prim_state*)new Gk_right_forward(arbiter, this) );
    state_list->add_state((Prim_state*)new Gk_left_forward(arbiter, this) );
    state_list->add_state((Prim_state*)new Gk_right_reverse(arbiter, this) );
    state_list->add_state((Prim_state*)new Gk_left_reverse(arbiter, this) );
    state_list->add_state((Prim_state*)new Gk_straight_forward(arbiter, this));
    state_list->add_state((Prim_state*)new Gk_straight_reverse(arbiter, this));
   
    curr_state = state_list->get_state();
}
   
int Gk_goto_defloc::valid_conds()
{
    if(arbiter->referee.game_state != play_on) return 0;
    
    // if ball certainty is too low can't go anywhere
    if( arbiter->ball.cer < 0.5 ) return 0;
    
    // if defloc close enough, don't go to there any more...
    if( arbiter->defloc.loc.get_len() < 2.0 ) return 0;

    // if ball can be kicked, don't move anywhere!
    if( arbiter->ball_kickable() ) return 0;
    
    // if ball must be catched, don't go to defloc
    if(arbiter->gk_catch_ball_valid()) return 0;
    
    // then calculate some angles to help defloc's states
    Vector temp;
    
    temp = arbiter->ball.g_loc - arbiter->me.g_loc;
    g_angle_me_to_ball = temp.get_dir();
    normalize_angle(g_angle_me_to_ball);

    float MAX_BALL_ANGLE = Cons2::visual_angle()*0.5 - 0.26;
    
    g_angle_looking_left = g_angle_me_to_ball + MAX_BALL_ANGLE;
    g_angle_looking_right = g_angle_me_to_ball - MAX_BALL_ANGLE;

    temp = arbiter->me.g_loc - arbiter->defloc.g_loc ;
    g_angle_defloc_to_me = temp.get_dir();
    normalize_angle(g_angle_defloc_to_me);
    
    return 1;
}


Gk_straight_forward::Gk_straight_forward(Arbiter *arbiterpointer,
                                         Gk_goto_defloc *gk_gd)
        :Prim_state(arbiterpointer){
    strcpy(name,"Gk_straight_forward");
    defloc = gk_gd;
}

int Gk_straight_forward::valid_conds()
{
    float temp = defloc->g_angle_defloc_to_me + M_PI;
    normalize_angle(temp);
    
    temp = temp - defloc->g_angle_me_to_ball;
    normalize_angle(temp);

    float MAX_BALL_ANGLE = Cons2::visual_angle()*0.5 - 0.26;
    if( fabs( temp ) < MAX_BALL_ANGLE ) return 1;
    
    return 0;
}

int Gk_straight_forward::execute()
{
    if(!valid_conds()) return 1;
    Move_cmd temp;
    temp.dir = arbiter->defloc.loc.get_dir();
    temp.spd = arbiter->defloc_speed();
    temp.valid = 1;
    arbiter->move(temp);
    return 0;
}

Gk_straight_reverse::Gk_straight_reverse(Arbiter *arbiterpointer,
                                         Gk_goto_defloc *gk_gd)
        :Prim_state(arbiterpointer){
    strcpy(name,"Gk_straight_reverse");
    defloc = gk_gd;
}

int Gk_straight_reverse::valid_conds()
{
    float temp = defloc->g_angle_defloc_to_me - defloc->g_angle_me_to_ball;
    normalize_angle(temp);
    

    float MAX_BALL_ANGLE = Cons2::visual_angle()*0.5 - 0.26;
    if( fabs( temp ) < MAX_BALL_ANGLE ) return 1;
    
    return 0;
}
int Gk_straight_reverse::execute()
{
    if(!valid_conds()) return 1;
    static Move_cmd temp;
    // when reversing not facing to defloc
    temp.dir = arbiter->defloc.loc.get_dir() + M_PI;
    normalize_angle( temp.dir );
    // when reversing, speed is negative
    temp.spd = -arbiter->defloc_speed();
    temp.valid = 1;
    arbiter->move(temp);
    return 0;
}


Gk_right_forward::Gk_right_forward(Arbiter *arbiterpointer,
                                         Gk_goto_defloc *gk_gd)
        :Prim_state(arbiterpointer){
    strcpy(name,"Gk_right_forward");
    defloc = gk_gd;
}

int Gk_right_forward::valid_conds()
{
    float temp = arbiter->defloc.loc.get_dir();
    if(  temp < 0.0 && temp > -0.5*M_PI ) return 1;
    return 0;
}
int Gk_right_forward::execute()
{
    static Move_cmd temp;
     if(!valid_conds()) return 1;
   if(arbiter->is_any_sensor_valid){
        temp.dir = defloc->g_angle_looking_right - arbiter->me.heading;
        normalize_angle( temp.dir );
    }
    else temp.dir = 0.0;
    
    temp.spd = arbiter->defloc_speed();
    temp.valid = 1;
    arbiter->move(temp);
    return 0;
}

Gk_left_forward::Gk_left_forward(Arbiter *arbiterpointer,
                                         Gk_goto_defloc *gk_gd)
        :Prim_state(arbiterpointer){
    strcpy(name,"Gk_left_forward");
    defloc = gk_gd;
}

int Gk_left_forward::valid_conds()
{
    float temp = arbiter->defloc.loc.get_dir();
    if(  temp > 0.0 && temp < 0.5*M_PI ) return 1;
    return 0;
}
int Gk_left_forward::execute()
{
    static Move_cmd temp;
    if(!valid_conds()) return 1;
    if(arbiter->is_any_sensor_valid){
        temp.dir = defloc->g_angle_looking_left - arbiter->me.heading;
        normalize_angle( temp.dir );
    }
    else temp.dir = 0.0;
    
    temp.spd = arbiter->defloc_speed();
    temp.valid = 1;
    arbiter->move(temp);
    return 0;
}


Gk_right_reverse::Gk_right_reverse(Arbiter *arbiterpointer,
                                   Gk_goto_defloc *gk_gd)
        :Prim_state(arbiterpointer){
    strcpy(name,"Gk_right_reverse");
    defloc = gk_gd;
}

int Gk_right_reverse::valid_conds()
{
    float temp = arbiter->defloc.loc.get_dir();
    if( temp < -0.5*M_PI ) return 1;
    return 0;
}
int Gk_right_reverse::execute()
{
    static Move_cmd temp;
    if(!valid_conds()) return 1;
    if(arbiter->is_any_sensor_valid){
        temp.dir = defloc->g_angle_looking_left - arbiter->me.heading;
        normalize_angle( temp.dir );
    }
    else temp.dir = 0.0;
    // negative speed when reversing...
    temp.spd = -arbiter->defloc_speed();
    temp.valid = 1;
    arbiter->move(temp);
    return 0;
}

Gk_left_reverse::Gk_left_reverse(Arbiter *arbiterpointer,
                                         Gk_goto_defloc *gk_gd)
        :Prim_state(arbiterpointer){
    strcpy(name,"Gk_left_reverse");
    defloc = gk_gd;
}

int Gk_left_reverse::valid_conds()
{
    float temp = arbiter->defloc.loc.get_dir();
    if( temp > 0.5*M_PI ) return 1;
    return 0;
}
int Gk_left_reverse::execute()
{
    static Move_cmd temp;
    if(!valid_conds()) return 1;
    if(arbiter->is_any_sensor_valid){
        temp.dir = defloc->g_angle_looking_right - arbiter->me.heading;
        normalize_angle( temp.dir );
    }
    else temp.dir = 0.0;
    // negative speed when reversing...
    temp.spd = -arbiter->defloc_speed();
    temp.valid = 1;
    arbiter->move(temp);
    return 0;
}

Gk_hold_ball::Gk_hold_ball(Arbiter *arbiterpointer)
        :Prim_state(arbiterpointer)
{
    strcpy(name,"Gk_hold_ball");
}
int Gk_hold_ball::valid_conds()
{
    if(arbiter->referee.game_state != play_on) return 0;
    // if ball is too far we can't hold it
    if(arbiter->ball.loc.get_len() >= 1.9) return 0;
    if(arbiter->ball.cer < 0.7 ) return 0;
    if(arbiter->ball.g_vel.get_len() > 0.5) return 0;
    if(arbiter->me.catched) return 0;
    // next two clauses check if goalie is inside the penalty area
    if(arbiter->me.g_loc.get_x() > - MAX_X + 16.5 - 2.0) {
        
        cerr << "Not catching because x coord! "<<
            arbiter->me.g_loc.get_x() <<
            endl;
        return 0;
    }
    
    if(fabs( arbiter->me.g_loc.get_y() ) > 40.32/2.0 - 5){
        cerr << "Not catching because y coord!"<<endl;
        return 0;
    }
    
    return 1;
}

int Gk_hold_ball::execute()
{
    if(!valid_conds()) return 1;
    Hold_cmd hold;
    hold.valid = 1;
    hold.dir = arbiter->ball.loc.get_dir();
    arbiter->hold_ball(hold);
    return 0;
}
Gk_catch_ball::Gk_catch_ball(Arbiter *arbiterpointer)
        :Catch_ball(arbiterpointer)
{}

int Gk_catch_ball::valid_conds()
{
    return arbiter->gk_catch_ball_valid();
}

Gk_special_situation::Gk_special_situation(Arbiter *arbiterpointer)
        :State_machine(arbiterpointer){
    strcpy(name, "gk_special_situation_state_machine");
    
    state_list->add_state((Prim_state*)new KickFar(arbiter) );
    state_list->add_state((Prim_state*)new Turnto_goal(arbiter) );
    state_list->add_state((Prim_state*)new Turnto_ball(arbiter) );
    state_list->add_state((Prim_state*)new Goto_specloc(arbiter));
    state_list->add_state((Prim_state*)new Lookaround(arbiter) );
        
// in the beginning current state is special situation state ie. the state in
// the first node of the State_list
    curr_state = state_list->get_state();
}


int Gk_special_situation::valid_conds()
{
    if(arbiter->referee.game_state == play_on){
        return 0;
    }
    
    return 1;
}
