// file    : Vs_team_audio.cc   

// Purpose of team_audio virtual sensor is to collect information of
// objects on the field sent by teammates

#include "decls.h"
//#include "Soccer.h"
#include <Vs/Vs_team_audio.h>

extern char global_side_char;

const float cer_update = 0.4;


Vs_team_audio::Vs_team_audio()
{
     my_id = VS_TEAM_AUDIO_ID;
     
     for( int i = 0; i < MAX_HEARD_MSG; i++) messages[i].talker = TALKER_NONE;
     
     strcpy(teamname,"CAT");
     teamname[3] = Cons2::side_char();
     teamname[4] = 0;
     
     if(Cons2::debug_level_startup()) 
         cerr << "Vs_team_audio: CONSTRUCTOR executed" << endl;
}

Vs_team_audio::~Vs_team_audio()
{
    if(Cons2::debug_level_exit()) 
        cerr << "Vs_team_audio: DESTRUCTOR executed" << endl;
}

void Vs_team_audio::Init(EmptySamba *sambapointer)
{
     samba  = sambapointer;
     sensor = (Sensor*)samba->get_pointer( SENSOR_ID );
}

void Vs_team_audio::get_loc_and_vel(char *message, int &p, 
                                    Vector &loc, Vector &vel)
{
    static float x,y,vx,vy;
    static int length;
    if(sscanf(&(message[p]),"%f %f %f %f %n", &x, &y, &vx,
           &vy, &length) == 4){
        loc.set_xy(x, y);
        vel.set_xy(vx, vy);
        p += length;
    } else {
        cerr << "WRONG! " << __FILE__ << " " << __LINE__ << endl;
        exit(1);
    }
}


void Vs_team_audio::get_int(char *message, int &p, int &value)
{
    static int length;
    sscanf(&(message[p]),"%d %n", &value, &length);
    p += length;
}


void Vs_team_audio::analyze_message(Audioinfo& msg)
{
    char team_name[20];
         
    int sender_unum, send_time, check_value,
        length, p;
    
    p = 0;
    
    sscanf(msg.message,"%s %d %d %d %n",team_name, &sender_unum,
           &send_time, &check_value, &length);
    p += length;
    
    // check if this is our team's message
    if(strcmp(team_name,teamname)) {
        // cerr << __FILE__ << __LINE__ << endl;
        return;
    }
    
    // check if send time was near enough
    if( ( curr_time - send_time ) > 5 ){
        if(Cons2::debug_level() > 3) 
            cerr << "Player " << Cons2::me_unum() << ": Too old message: "
                 << (curr_time - send_time) << endl;
        return;
    }
    
    // read ball location and velocity
    get_loc_and_vel(msg.message, p, audio.ball_g_loc, audio.ball_g_vel);
    audio.ball_valid = 1;

    int unum = 0;
    
    while(unum < 12){
        get_int(msg.message, p, unum);
                
        if(unum < 12){
            get_loc_and_vel(msg.message, p, 
                            audio.opp_g_loc[unum-1],
                            audio.opp_g_vel[unum-1]);
            audio.opp_valid[unum-1] = 1;
//            cerr << "I heard: player " << unum << " stands at ";
//            audio.opp_g_loc[unum-1].err_print_xy();
//            cerr << endl;
        }
    }
}

void Vs_team_audio::invalidate_all()
{
    audio.ball_valid = 0;
    for(int k = 0; k< 11; k++){
        audio.opp_valid[k] = 0;
    }
    
}


void Vs_team_audio::read(Team_audio_vs &to)
{
    check_executed();
    to = audio;
}


int Vs_team_audio::transform()
{
    int i;
    
    sensor->read_audio_sensor(messages);
    // search for messages sent by referee 
    for( i = 0; i < MAX_HEARD_MSG; i++)
        if( messages[i].valid ){
            if( messages[i].talker == TALKER_PLAYER ){
                // analyze player's message and change
                analyze_message( messages[i] );
            }
            else if (messages[i].talker == TALKER_SELF )
                if(Cons2::debug_level()>1) 
                    cerr << "Own message received at time " << messages[i].time 
                         << endl << messages[i].message;
        }
    
    return 0;
}
