/* File    : cfg.c
 * Includes: Functions for processing the configuration file.
 */
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include "cfg.h"

/* a line can contain max 200 characters including newline */
#define CFG_MAXLINELEN 201

/* maximum length for parameter name string */
#define CFG_MAXPARLEN 50

/* maximum number of parameters */
#define CFG_MAXNPARS 500

/* one configuration item consists of a parameter string and a value */
struct cfg_item {
  char  parameter[CFG_MAXPARLEN];
  float value;
};

/* array for configureable parameters */
struct cfg_item Cfg[CFG_MAXNPARS];

/* number of parameters read from cfg file */
int Cfg_Count = 0;

/* global debug level variables in soccer */

//extern int debug_level_startup;



FILE *cfg_open (char *file)
{
  FILE *fp;
  
  if ((fp = fopen (file, "r")) == NULL)
    fprintf (stderr, "cfg_open: can't open cfg file %s\n", file);
  else ;//printf("File %s succesfully opened\n",file);
  
  return fp;
}

/* READ PARAMETERS FROM THE FILE
 */
void cfg_read (FILE *fp)
{
  char  line [CFG_MAXLINELEN];
  char *ptr;
  int   i,j,n;
  
  Cfg_Count = 0;
  for (i = 0; i < 10000 && Cfg_Count < CFG_MAXNPARS; i++) {
    if (fgets (line, CFG_MAXLINELEN, fp) == NULL)
      break;
    
    for (ptr = line; *ptr == ' '; ptr++)
      ;
    
    /* 1st non-white character of a comment line must be # */
    if (*ptr != '#' && *ptr != '\n') {
      /* what about too long parameter names? */
       n = sscanf (ptr, "%s %f",
	      Cfg[Cfg_Count].parameter,
	      &(Cfg[Cfg_Count].value));
       if(n != 2){
            cerr << "Something weird here in " << __FILE__ << " "
                 << __LINE__ << endl << "Read parameter: "
                 << Cfg[Cfg_Count].parameter << endl;

            cerr << "Previously read parameters:" << endl;
            for( i = 0; i< Cfg_Count; i++)
                 cerr << Cfg[i].parameter << endl;
            exit(1);
                        
            Cfg[Cfg_Count].parameter[0] = 0;
            Cfg[Cfg_Count].value = 0.0;
       }
       else Cfg_Count++;
    }
  }
  
  if (Cfg_Count >= CFG_MAXNPARS)
    fprintf (stderr,"cfg_read: too many configureable parameters\n");
  else
    for (j = Cfg_Count; j < CFG_MAXNPARS; j++) {
      Cfg[i].parameter[0] = '\0';
      Cfg[i].value = 0;
    }
  
/* DEBUG */
/*  if(debug_level_startup){
       printf ("\n*** CONFIGURATION FILE ***\n");
       for (i = 0; i < Cfg_Count; i++)
            printf ("%s = %.2f\n", Cfg[i].parameter, Cfg[i].value);
       printf ("\n");
  }*/
  
}


void cfg_close (FILE *fp)
{
  fclose (fp);
}


/* GET ONE PARAMETER
 *
 * This function returns 1, if the given string is a configuration
 * string; otherwise 0. If 1 is returned, value contains the
 * corresponding configuration value.
 */
int cfg_get (char *str, float *value)
{
     int i;
     
     for (i = 0; i < Cfg_Count; i++)
          if (strcmp (Cfg[i].parameter,str) == 0) {
               *value = Cfg[i].value;
               return 1;
          }
     cerr << "Variable " << str
          << "not in configuration file (or file not opened)!" << endl
          << "Exiting..." << endl;
     exit(1);
}

// FIND ONE PARAMETER
//
// The function returns 1 if parameter is in the configuration file
// else function returns 0
int cfg_find (char *str)
{
     int i;
     for (i = 0; i < Cfg_Count; i++)
          if (strcmp (Cfg[i].parameter,str) == 0) 
               return 1;
     return 0;
}

     

/* end of file */
