// -*- mode:C++ -*-
// file    : cpp_comm.h
// includes: class declaration for the cpp_comm
// 
// return values: 1 = sucess, 0 = failure

#ifndef _CPP_COMM_H
#define _CPP_COMM_H 

//#define CPP_FILE // if this is taken in to compilation, cpp will go to a file
#include <iostream.h>

#ifdef CPP_FILE
#include <fstream.h>
#else
#include <pfstream.h>
#endif
/* 25/March/1998: opfstream seems to work only with pipes,
   not with files. So, define cpp_ostream type as opfstream
   when you want to send data directly to gnuplot. Define
   cpp_ostream type as ofstream when you want to send data
   to a file.
*/



#ifdef CPP_FILE
typedef ofstream cpp_ostream;
#else
typedef opfstream cpp_ostream;
#endif
const int CPP_MAX_NAME_LEN = 200;
const int CPP_MAX_NDATA_ITEMS = 200;
const int CPP_MAX_NROWS = 50;
const int CPP_MAX_NCOLS = 50;

class cpp_comm {
public:
     cpp_comm();
     ~cpp_comm ();

     // methods for the producer:

     int send_init(cpp_ostream& out);

     int send_curve (cpp_ostream& out, 
                     char title[CPP_MAX_NAME_LEN],
                     int npoints, float *data,
                     int mark_end=1);
               
     int send_curve_to_epsfile (cpp_ostream& out,
                                char title[CPP_MAX_NAME_LEN],
                                int npoints, float *data,
                                const char* ps_filename);
     
     void dm (int ncols, int nrows, float max_mgn,
              float dir_vec[CPP_MAX_NROWS], float mgn_vec[CPP_MAX_NCOLS]);
     
     int send_surface (cpp_ostream& out, 
                       char title[CPP_MAX_NAME_LEN],
                       int nrows, int ncols, float max_mgn, float *data,
                       int mark_end=1);

     int send_surface_to_epsfile (cpp_ostream& out,
                                  char title[CPP_MAX_NAME_LEN],
                                  int nrows, int ncols, float max_mgn, float *data,
                                  const char* ps_filename);
     
     void send_quit (cpp_ostream& out);
  
     int rcv_and_draw (ifstream& in, cpp_ostream& out);
          
     void rcv_and_save_eps(ifstream& in, cpp_ostream& out,
                           const char* ps_filename);
                           
protected:
};


#endif

/* end of file */







