 // -*- mode:C++ -*-
#ifndef _FIELDOBJECT_H
#define _FIELDOBJECT_H

//#include <LEDA/basic.h>
#include <iostream.h>
#include "Const.h"

enum Side { Side_L, Side_R, Side_none } ;

const char Side_L_str[2] = "l" ;
const char Side_R_str[2] = "r" ;

/*
inline Side str_to_Side(string str) { 
     return str_to_Side((const char*)str) ;
}*/
inline Side str_to_Side(const char* str) {
     if(Side_L_str[0] == str[0]) 
	  return Side_L ;
     else if(Side_R_str[0] == str[0])
	  return Side_R ;
     else 
	  return Side_none ;
}



enum ViewAngle {VA_Narrow, VA_Normal, VA_Wide} ;
enum ViewQuality {VQ_High, VQ_Low} ;

struct Sense_body
{
     int time;
     int valid;
     float short_stamina;
     float long_stamina;
     float speed;
     ViewAngle view_angle;
     ViewQuality view_qual;
     int kicks;
     int dashes;
     int turns;
     int says;
};


enum TALKERS { TALKER_NONE, TALKER_REFEREE, TALKER_PLAYER, TALKER_SELF };

struct Audioinfo
{
     int time;
     int valid;
     float dir;
     TALKERS talker;
     char message[512]; // big enough
};

const MAX_HEARD_MSG = 5;


// Next classes are designed to ease handling of field objects;
// players, lines, flags, goals and ball

class FieldObject 
{
protected:     
     float D_dist;     // change of distance (delta_distance)
     int D_dir;        // change of direction
     
     int Dir;          // direction (degrees)
     float Dist;       // distance

     BOOL valid;
     int validation_time;
     
     int arraypos;
     
public:
     inline FieldObject(float d_dist = 0.0, int d_dir = 0, int dir = 0, float dist = 0.0,
                        BOOL v = FALSE)  // constructor
     {
          D_dir = d_dir;  
          D_dist = d_dist;
          Dir = dir;   
          Dist = dist;   
          valid = v;
          validation_time = 0;
     };
     virtual inline void init(int n){ arraypos = n; }

     virtual inline void invalidate(){ valid = FALSE; };
     virtual inline void validate(int time){

          valid = TRUE;
          validation_time = time;
     };
     virtual inline int get_D_dir(){ return D_dir; };
     virtual inline float get_D_dist(){ return D_dist; };
     virtual inline int get_dir(){ return Dir; };
     virtual inline float get_dist(){ return Dist; };
     virtual inline float get_spd() { return 0.0; }; // speed is not implemented yet
     
     virtual inline BOOL is_valid(){ return valid; };
     virtual void set_all(int time, int n_arguments, float dist,int dir, 
                          float d_dist,int d_dir);
};


class Player: public FieldObject
{
protected:
     Side side; // playing side of the player, Side_L, Side_R or Side_none
     int UNum;  // uniform number
     int heading;
     
public:
     inline Player(float d_dist = 0,int d_dir = 0, int dir = 0, float dist = 0.0,
                   BOOL v = FALSE, Side s = Side_none, int unum = 0, 
                   int head = 0)
               :FieldObject(d_dist, d_dir, dir, dist, v)
     {
          side = s;
          UNum = unum; // this is not needed anywhere!!!
          heading = head;
     };
     virtual int get_heading() { return heading; }
    
    virtual void set_all(int time, int n_arguments, float dist,int dir, 
                          float d_dist,int d_dir);
   
     // sets all the variables except Unum
     virtual void set_all(int time, int n_arguments, float dist, int dir, 
                          float d_dist, int d_dir, int head);
};


class Line: public FieldObject
{
protected:
     LINETYPES linetype; // left, center, right, top or bottom
     
public:
     inline Line(float d_dist = 0, int d_dir = 0, int dir = 0, float dist = 0.0,
                 BOOL v = FALSE, LINETYPES ltype = LEFT)
               :FieldObject(d_dist, d_dir, dir, dist, v) // call fieldobject's constructor first
     {
          linetype = ltype;
     };

     inline void init(int n) { arraypos = n; linetype = (LINETYPES)n; };
     
     inline int get_index(char* arg1){
          int k;
          for(k=0;k<NLINES;k++)
               if(LINECHARS[k] == arg1[0]){
                    return k;
               }
          cerr << "ERROR in class Line, get_index arg1 not a valid argument: " << arg1 << endl;
          return 0;
     }
     inline LINETYPES get_linetype(){ return linetype; }
          
     int get_robot_angle(); // returns robots angle measured from lineangles     
};

class Flag: public FieldObject
{
protected:
     float x,y;
     FLAGTYPES flagtype;
public:

     inline Flag(float d_dist = 0, int d_dir = 0, int dir = 0, float dist = 0.0,
                        BOOL v = FALSE)
               :FieldObject(d_dist, d_dir, dir, dist, v )
     {};
     
          
     int get_index(char* arg1, char* arg2, char* arg3);
     inline float get_x(){
          return x;
     };
     
     inline float get_y(){
          return y;
     };
     
     inline FLAGTYPES get_flagtype(){
          return flagtype;
     }
     
     void init(int arrayindex);
};
#endif // _FIELDOBJECT_H
