// interface.cpp
// created by Jussi Pajala
//

// Some functions in this file are originally created
//	by NODA, Itsuki
//

//Usage:
//  options: 
//	-h HOSTNAME 
//	-p PORTNUMBER
//  -H help page
//  -r Reconnect
//  -u Uniform number

//======================================================================
//Header files
//======================================================================
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <signal.h>
#include "assert.h"
#include "interface.h"

void *start_sensor(void *arg);

//=====================================================================
// Global variables
//=====================================================================


// set to one when interface has initialized itself, and set to zero when quitting
int up_and_running = 0;
volatile int signal_arrived = 0;

int global_me_unum = 0;

char global_side_char = 0;

pid_t interface_process_id;
Interface* P ;  // initialized in Interface::init() to point to the interface
extern void *read_user_command (void *arg);
extern Cons2 constants;
void *wait_alarm(void *p);
void *sender(void *arg);

const float MAXPOWER = 100.0;


//======================================================================
// Methods of class Interface
//======================================================================
// methods are in following order:
//   constructor
//   initialization methods
//   miscellaneous small methods called by other methods
//   methods for situation calculation
//   methods to handle sensor information
//   main loop

Interface::Interface(char* teamname,
                     char* host,
                     int port) {
//     next_msg_read = 0;
//     next_msg_write = 0;
//     sensor_data_mutex = PTHREAD_MUTEX_initIALIZER; // init mutex before any thread uses it
     pthread_mutex_init(&sensor_data_mutex,(pthread_mutexattr_t*)NULL);
     pthread_mutex_init(&send_data_mutex,(pthread_mutexattr_t*)NULL);
     
//     pthread_mutex_init(&hundred_ms_mutex,(pthread_mutexattr_t*)NULL);
     pthread_mutex_init(&mutex_10_ms,(pthread_mutexattr_t*)NULL);
     pthread_cond_init(&cond_10_ms, (pthread_condattr_t*)NULL);
     flag_10_ms = 0;

     pthread_mutex_init(&mutex_100_ms,(pthread_mutexattr_t*)NULL);
     pthread_cond_init(&cond_100_ms, (pthread_condattr_t*)NULL);
     flag_100_ms = 0;
     

     control_system_state = ALL_ON;
     cmd_buffer = new RoundBuffer<server_cmd>(10);
     audiobuffer = new RoundBuffer<Audioinfo>(MAX_HEARD_MSG);
   
     goalie = 0;
     req_old_protocol = 0;
     team = teamname;
     serverhost = host ;
     serverport = port ;
     bufsize = BUFFERSIZE;
     send_buffer = new char[bufsize] ;
     recv_buffer = new char[bufsize] ;
     move_command = new char[bufsize];
     
     side = Side_L ;
     last_visual_time = 0;
     time_after_visual = 0;
     curr_t = 0;
     print_sensor_data = 0;
          
     new_visual_information = FALSE;
          
     xv = yv = curr_speed = 0.0;
     angle = delta_angle = old_angle = 0.0;
     
     x = y = old_x = old_y = -10000.0;
     last_updated = 0;
     
     int i;

     for(i=0;i<NLINES;i++){
          lines[i].init(i);
     }
     for(i=0;i<NFLAGS;i++){
          flags[i].init(i);
     }
     
     reconnection = 0; // normal connection
     
     if(Const::debug_level_startup()) 
          cerr << "Interface: CONSTRUCTOR executed" 
               << " size of interface class: " << sizeof(Interface) << endl;
}
          
Interface::~Interface()
{
    if(!pthread_kill(sensor_tid,0)){
        sleep(1);
        if(!pthread_kill(sensor_tid,0)){
//            cerr << "Sensor thread not canceled, killing..." << endl;
            pthread_kill(sensor_tid,SIGKILL);
        }
    }
    // Close sockets
    close_connection(this->sock) ;
    
    up_and_running = 0;
    if(Const::debug_level_exit())
        cerr << "Interface: DESTRUCTOR executed" << endl;
}


//----------------
// Misc. methods
//----------------


//
// Prints usage message
//
void Interface::usage() {
     cerr << endl << "Usage:" << endl << endl
          << "client [options] TEAMNAME "<< endl
          << "options:"<< endl
          << "-h HOSTNAME"<< endl
          << "-p PORTNUMBER"<< endl 
          << "-o (requests server version 3.xx protocol)" << endl 
          << "-g (inits player as a goalie)" << endl
          << endl
          << " Following two options are meaningful only together:" << endl
          << "-r (reconnect)"<< endl 
          << "-u UNIFORM NUMBER"<< endl
          << endl
          << "-H THIS HELP PAGE" << endl
          << ends ;
}

          

//
// Moves robot anywhere on the field
// this can be used only before_kick_off
//
void Interface::command_move(int cmd_x, int cmd_y) {

     int sx,sy; // server coordinates

     // server coordinate system is different to normal system,
     // since y axel is just opposite. When sending move command
     // only y coordinate must be changed - server takes care
     // on which side player is playing
     
     sx=cmd_x;
     sy = -cmd_y;  
     lock_send_data();
     // locked section 
     sprintf(move_command,"(move %d %d)",sx,sy);
     // locked section ends
     unlock_send_data();
     
     switch(side){
     case Side_L:
          this->x = sx;
          this->y = sy;
          break;
     case Side_R:
          this->x = -sx;
          this->y = -sy;
          break;
     default: assert(0);
     }
}

//
// Turns the robot
// Input: turn command in radians
// returns void
void Interface::command_turn(float moment) {

     int command = -(int)(RAD2DEG*moment);
//HUOM - merkit!!
     if(!command) return;
#ifndef NO_PRINTS
     if(Const::debug_level() && fabs(moment) > M_PI) 
          cerr << "Remember: command_turn input is radians! "
               << " Sent command in degrees is" << moment << endl;
#endif
     // if moment is too big, it must be corrected
     if(command<-180) command = -180;
     else if(command>180) command = 180;
     lock_send_data();
     // locked section
     sprintf(move_command,"(turn %d)",command) ;
     // locked section ends
     unlock_send_data();
     
// here we estimate how much angle has changed after last_visual_time     
     delta_angle -= command/(1.0+0.5*curr_speed);
}

//
// Accelerates the robot
// Input: dashing power
// returns void
void Interface::command_dash(float power) {
     
     float p = power;

     if(p>MAXPOWER) p = MAXPOWER;
          
     lock_send_data();
     // locked section
     sprintf(move_command,"(dash %f)",p) ;
     // locked section ends
     unlock_send_data();
     float pr = Const::dash_power_rate();
     xv += cos((angle+delta_angle)*DEG2RAD)*p*pr;
     yv -= sin((angle+delta_angle)*DEG2RAD)*p*pr;
}

//
// Kicks the ball
// Input: Kick power and kick direction in radians
// returns void
void Interface::command_kick(float power, float dir) {

     if(fabs(dir) > M_PI) 
          cerr << "Remember: command_kick direction input is radians!" << endl;
     lock_send_data();
     // locked section
     dir = -RAD2DEG*dir;
     sprintf(move_command,"(kick %d %d)",(int)power, (int)dir) ;
     // locked section ends
     unlock_send_data();
}
//
// Catches the ball 
// Input: catching direction in radians
// returns void
void Interface::command_catch(float dir){
     
     if(fabs(dir) > M_PI)
          cerr << "Remember: command_catch input is radians!" << endl;

     lock_send_data();
     // locked section
     dir = -RAD2DEG*dir;
     sprintf(move_command,"(catch %d)",(int)dir);
     // locked section ends
     unlock_send_data();
}

//
// Sends audial information
//
void Interface::command_say(char *msg) {
     
//     cout << "Say:" << msg << endl ; 
     if(strlen(msg) > 512) {
          cerr << __FILE__ << " " << __LINE__ << endl
               << "too long message:" << msg << endl;
          exit(1);
     }
     lock_send_data();
     // locked section
     server_cmd temp;
     sprintf(temp.text,"(say %s)",msg) ;
     temp.time = curr_t;
     cmd_buffer->push(&temp);
     // locked section ends
     unlock_send_data();
}
//
// Sends request for body sensing
//
void Interface::command_sensebody(){
          
     lock_send_data();
     // locked section
     server_cmd temp;
     strcpy(temp.text,"(sense_body)") ;
     temp.time = curr_t;
     cmd_buffer->push(&temp);
     // locked section ends
     unlock_send_data();
}

//
// Changes type and quality of sensor information
//
void Interface::command_change_view(ViewAngle va, ViewQuality vq) {

     const char* ViewAngleStr[3] = {"narrow","normal","wide"} ;
     const char* ViewQualityStr[2] = {"high","low"} ;
#ifndef NOPRINTS     
     cout << "ChangeView: " << ViewAngleStr[va] << "," << ViewQualityStr[vq] 
          << endl ;
#endif 
     lock_send_data();
     // locked section
     server_cmd temp;
     sprintf(temp.text,"(change_view %s %s)",
             (const char*)ViewAngleStr[va],(const char*)ViewQualityStr[vq]);
     temp.time = curr_t;
     cmd_buffer->push(&temp);
     // locked section ends
     unlock_send_data();
}

//
// Sends any text to the server
//
void Interface::command_send_textbuffer(char* sending_buffer)
{
     lock_send_data();
     // locked section
     server_cmd temp;
     strcpy(temp.text,sending_buffer);
     temp.time = curr_t;
     cmd_buffer->push(&temp);
     // locked section ends
     unlock_send_data();
}

   

//-----------------
//initializations
//-----------------

//
// inits connection to the server
//
BOOL Interface::init(int argc, char **argv) {
        
// First Analyze Arguments
        my_get_opt(argc,argv) ;

// try to create connection with server
        sock = init_connection((char*)((const char*)serverhost),serverport) ;
        if(sock.socketfd == -1) {
                cerr << "Can't open socket." << endl ;
                return FALSE ;
        }

        ostrstream ost(send_buffer,bufsize) ;

        if(reconnection)
             ost << "(reconnect " << team << " " << unum << ")" << endl << ends;
        else{
             if(goalie && !req_old_protocol)
                  ost << "(init " << team << " (version 4.0) (goalie))" 
                      << endl << ends ;
             else if( !goalie && !req_old_protocol)
                  ost << "(init " << team << " (version 4.0))" << endl << ends ;
             else if( !goalie && req_old_protocol)
                  ost << "(init " << team << ")" << endl << ends ;
             else {
                  cerr << "Cannot have a goalie with old protocol!" << endl;
                  exit(1);
             }
        }

        if(send_message(send_buffer,sock) == -1) {
                cerr << "Can't connect to the server." << endl ;
                return FALSE ;
        }
     
//Receive init message
        int n ;
        while(!(n = receive_message(recv_buffer, bufsize, &sock))) {} ;
        if(n == -1) {
                cerr << "Network error!" << endl;
                return FALSE ;
        }
        int l ;
        char mode[BUFFERSIZE] ;
        char sidename[5] ;

// analyze server response to init command
        if(reconnection){
             if(2 > sscanf(recv_buffer,"(reconnect %[lr] %[^)])%n", 
                           sidename, mode, &l)) {
                  cerr << "Refused by the server: " << recv_buffer << endl ; 
                  return FALSE ;
             }
        } else {
             
             if(3 > sscanf(recv_buffer,"(init %[lr] %d %[^)])%n", 
                           sidename, &unum, mode, &l)) {
                  cerr << "Refused by the server: " << recv_buffer << endl ; 
                  return FALSE ;
             }
        }
// Setup Side
        side = str_to_Side(sidename) ;

        side_char = sidename[0];
        constants.set_side_char(side_char);
        
        if(side == Side_R) angle = 180;
        else angle = 0.0;
        
// Final Report
        if(unum == 1) {
            if(side) cout << "Playing on right" << endl;
            else cout << "Playing on left" << endl;
        }
        cout << "Player " << unum << " Connect with " << serverhost << " [" 
             << serverport << "]" << endl ;
        if(unum != GOAL_KEEPER_UNUM && goalie) {
            cout << "Player " << unum << " can't play as goal keeper!" << endl;
        }
        
#ifndef NOPRINTS
        if(unum == 1 && !goalie) 
            cerr << endl << endl 
                 << "GOALIE MUST BE INITIALIZED BY A COMMAND LINE OPTION!"
                 << endl << "For example: bin/sambaplayer -g -h tk10" << endl
                 << "To continue with goalie, kill all players and server"
                 << endl << endl << endl;
#endif
        

                
        Cons2::set_uniform_number(unum);

        // pointer P points to this interface. It is used by sensor cycle.
        P = this;
        send_buffer[0] = 0;
        
        // allow sensorcycle to send signals
        up_and_running = 1; 

        create_threads();
        
        for(int i = 0; i < 10;i++) wait_100ms();
        
        // set global variables
        global_side_char = side_char;
        global_me_unum = unum;

        if(Const::debug_level_startup()) 
             cerr << "Interface initialized" << endl;
        // invalidate all sensorinfo
        invalidate_all();
        
        // we'll come this far if connection is created and timer is running 
        return TRUE ;
}

// Interface::create processes() creates all LWPs of this program. No threads 
// are created anywhere else.
void Interface::create_threads()
{
     interface_process_id = getpid(); // get process id

     // create thread to send commands
     pthread_create(&sender_tid, (pthread_attr_t*)NULL, sender,(void *)NULL);
     // and to read user inputs
     pthread_create(&keyboard_read_tid, (pthread_attr_t*)NULL, read_user_command, 
                    (void *)NULL); 
     
     // create alarm thread to wakeup execution and sending
     pthread_attr_t custom_sched_attr;
     pthread_attr_init(&custom_sched_attr);
     pthread_attr_setscope(&custom_sched_attr, PTHREAD_SCOPE_SYSTEM);
     pthread_create(&alarm_tid, &custom_sched_attr, wait_alarm, NULL);

     // create a thread to read sensors
     pthread_create(&sensor_tid, (pthread_attr_t*)NULL, start_sensor,(void *)NULL);
}

 

void Interface::quit()
{
     // cancel threads only once at the first time this function is called
     if(up_and_running){
          pthread_cancel(sensor_tid);
          pthread_cancel(sender_tid);
          pthread_cancel(keyboard_read_tid);
          pthread_cancel(alarm_tid);
     }
     up_and_running = 0; // not up and running when quitting
}


//
// reads command line options and .ini file
//



void Interface::my_get_opt(int argc, char *argv[]) {
     int option_char;
     int i, j;
     
     while ((option_char = getopt(argc, argv, "h:p:ru:ogiH")) != EOF) {
          switch (option_char) {
          case 'h':
               serverhost = optarg;
            break;
          case 'p':
               serverport = atoi(optarg);
               break;
          case 'r':
               reconnection = 1;
               break;
          case 'u':
               unum = atoi(optarg);
               break;
          case 'o':
               req_old_protocol = 1;
               break;
          case 'g':
               goalie = 1;
               break;
          case 'i':
              // introduction on: show catch ball actionmaps 
              Const::set_introduction_on();
              break;
          case '?':  // unregognized option
          case 'H': usage();
            exit(0);
            break;
            
          }
     }
     
     for (i = 0, j = optind ; j < argc ; i++, j++) {
          switch(i) {
          case 0:
               team = argv[j];
               break;
          case 1:
      usage();
      break;                  // we can add cmd line arguments here
          default:
               usage();
               exit(-1);
          }
     }
}


//--------------------------
// Situation calculation
//--------------------------

//
// estimates situation basing to commanded movements
//
void Interface::estimate_situation()
{
     time_after_visual++;
     curr_t = get_time();

     old_x = x;
     old_y = y;

     x += xv; // position estimation
     y += yv;

     xv *= Const::player_decay();
     yv *= Const::player_decay();
          
     curr_speed = sqrt(xv*xv+yv*yv);
}

//
// computes situation from sensor information
//
void Interface::compute_situation()
{
     int delta_t;
     int i;
     int lineangle;
     float estimated_angle;
     
     estimated_angle = angle + delta_angle;
     delta_t = last_visual_time - curr_t;
     // curr_t was calculated with old last_visual_time, so if time_after_visual
     // was incremented too many times, delta_t > 0
     
     // loop if time is estimated wrong
     if(delta_t>0) for(i=0;i<delta_t;i++){
          // maybe it's not necessary to compute x:s and y:s here since they are computed accurately afterwards
          old_x = x;
          old_y = y;
          
          x += xv;
          y += xv;
          
          xv *= Const::player_decay();
          yv *= Const::player_decay();
          
          // curr_speed *=DECAY
     }
     curr_speed = sqrt(xv*xv+yv*yv);

     if(estimated_angle < 0) estimated_angle+=360.0;
     else if(estimated_angle > 360)estimated_angle -=360.0;

     lineangle = get_lineangle();

     if(lineangle < 1000){
          if(lineangle < 0) lineangle+=360;
          else if(lineangle > 360) lineangle -=360;
          angle = lineangle; // correct angle is angle measured from line's angle
          delta_angle = 0.0;
     }

     float ox=x,oy=y; // for error estimation
     if(correct_position()){
          ox-=x;
          oy-=y;
          ox*=ox;
          oy*=oy;
          ox=sqrt(ox+oy);

//          cout << "Position corrected, error: "<< ox << " Position: x:" << x << " y:" << y << endl;
     }

     time_after_visual++;
     curr_t = get_time();
}


//
// calculates angle of the robot
// Input: void
// returns robot heading in the field (angle to positive x-axis)
//
int Interface::get_lineangle()
{
     int i;
     float furthest_line_dist = -200;
     int furthest_line_i = 1000;
     
     for(i=0;i<NLINES;i++){
          if (lines[i].is_valid() && lines[i].get_dist()>furthest_line_dist){
               furthest_line_dist = lines[i].get_dist();
               furthest_line_i = i;
          }
          
     }
     if(furthest_line_i<NLINES) return lines[furthest_line_i].get_robot_angle();
     
     // jpr aug16 // cout << "No valid lines, " << __FILE__ << " line: " << __LINE__ << endl;
     return 1000;
}
// void Interface::get_flag
// writes information about flag given in the first parameter to variables
// referenced in other parameters
// Input: flag whose information is wanted
// returns void
void Interface::get_flag(FLAGTYPES flag,float &dir, float &dist, float &d_dir, float &d_dist, int &valid)
{
     // if playing on left side return flag as it is
     // else if on right:
     if(side == Side_R) {
          // if robot is playing on the right side (Side_R), opposite flag must be returned.
          // eg. when asking flag that is at the corner of top line and left line, we return the
          // flag at the corner of bottom and right line and so on...
          FLAGTYPES retflagtype;
          retflagtype = (FLAGTYPES)((NFLAGS-1)-(int)flag);
          flag = retflagtype;
     }
     dist = flags[flag].get_dist();
     dir = -DEG2RAD*flags[flag].get_dir();
     d_dist = flags[flag].get_D_dist();
     d_dir = -DEG2RAD*flags[flag].get_D_dir();
     valid = flags[flag].is_valid();
     return;
}

// void Interface::get_line   public
// writes information about line given in the first parameter to variables
// referenced in other parameters
// Input: line whose data is wanted
// returns void
void Interface::get_line(LINETYPES line,float &dir, float &dist, float &d_dir, float &d_dist, int &valid)
{
     if(side==Side_R){
          LINETYPES retlinetype;
          if(line < TOP) retlinetype = (LINETYPES)((int)RIGHT - (int)line);
          else retlinetype = (LINETYPES)((int)TOP + (int)BOTTOM - (int)line);
          line = retlinetype;
     }
     dist = lines[line].get_dist();
     dir = -DEG2RAD*lines[line].get_dir();
     d_dist = lines[line].get_D_dist();
     d_dir = -DEG2RAD*lines[line].get_D_dir();
     valid = lines[line].is_valid();
     return;
}

// int Interface::get_heard_msg     public
// writes heard message to referenced variable msg and returns
// Input: void
// returns 1 on success
//         0 and flushes msg on failure
//
int Interface::get_heard_msg(Audioinfo& msg)
{
    if(!audiobuffer->pop(&msg)){
        msg.valid = 0;
        return 0;
    }
    return 1;
}

//
// void Interface::put_heard_msg     protected
// pushes heard msg to audiobuffer (it is RoundBuffer)
// Input: message to push
// returns void
// 
void Interface::put_heard_msg(Audioinfo& msg)
{
    audiobuffer->push(&msg);
/*     heard_msg[next_msg_write] = msg;
     circular_inc(next_msg_write,MAX_HEARD_MSG-1);
     if(next_msg_write == next_msg_read) circular_inc(next_msg_read,MAX_HEARD_MSG-1);
*/
     return;
}

// void Interface::get_sense_body
// Fills sense body structure, information about robot body, speed and
// number of executed commands...
// Input: void
// returns void
void Interface::get_sense_body(Sense_body &body)
{
     body = sb;
}

// void Interface::get_team     public
// fills team mates' information to arrays given by pointers
// be sure that given pointers point to arrays size of NFRIENDS 
// (number of friends)
// returns void
void Interface::get_team(float *dir, float *dist, float *d_dir, float *d_dist,
                         float *head, int *Unum, int *valid)
{
     int i;
     for( i=0;i<NFRIENDS;i++ ){
          dist[i] = friends[i].get_dist();
          dir[i] = -DEG2RAD*friends[i].get_dir();
          
          d_dist[i] = friends[i].get_D_dist();
          d_dir[i] = -DEG2RAD*friends[i].get_D_dir();
          head[i] = -DEG2RAD*friends[i].get_heading();
          
          Unum[i] = i+1;
          valid[i] = friends[i].is_valid();
     }
}
// void Interface::get_uk_team     public
// writes information about unknown team mates to arrays given by pointers
// ( unknown means that uniform number is unknown )
// be sure that given pointers point to arrays size of NUKFRIENDS 
// (number of unknown friends)
// returns void
void Interface::get_uk_team(float *dir, float *dist, float *d_dir, 
                            float *d_dist, float *head, int *valid)
{
     int i;
     for( i = 0; i < NUKFRIENDS; i++ ){
          dist[i] = uk_friends[i].get_dist();
          dir[i] = -DEG2RAD*uk_friends[i].get_dir();
          
          d_dist[i] = uk_friends[i].get_D_dist();
          d_dir[i] = -DEG2RAD*uk_friends[i].get_D_dir();
          head[i] = -DEG2RAD*uk_friends[i].get_heading();

          valid[i] = uk_friends[i].is_valid();
     }
}
// void Interface::get_opp     public
// writes information about opponents to arrays given by pointers
// be sure that given pointers point to arrays size of NOPPS 
// (number of opponents)
// returns void
void Interface::get_opp(float *dir, float *dist, float *d_dir, float *d_dist,
                        float *head, int *Unum, int *valid)
{
     int i;
     for( i=0;i<NOPPS;i++ ){
          dist[i] = opponents[i].get_dist();
          dir[i] = -DEG2RAD*opponents[i].get_dir();
          
          d_dist[i] = opponents[i].get_D_dist();
          d_dir[i] = -DEG2RAD*opponents[i].get_D_dir();
          head[i] = -DEG2RAD*opponents[i].get_heading();

          Unum[i] = i+1;
          valid[i] = opponents[i].is_valid();
     }
}

//void Interface::get_uk_opp     public
// writes information about unknown opponents to arrays given by pointers
//  ( unknown means that uniform number is unknown )
// be sure that given pointers point to arrays size of NUKOPPS
// (number of unknown opponents)
// returns void
void Interface::get_uk_opp(float *dir, float *dist, float *d_dir, 
                           float *d_dist, float *head, int *valid)
{
     int i;
     for( i=0;i<NUKOPPS;i++ ){
          dist[i] = uk_opponents[i].get_dist();
          dir[i] = -DEG2RAD*uk_opponents[i].get_dir();
          
          d_dist[i] = uk_opponents[i].get_D_dist();
          d_dir[i] = -DEG2RAD*uk_opponents[i].get_D_dir();
          head[i] = -DEG2RAD*uk_opponents[i].get_heading();

          valid[i] = uk_opponents[i].is_valid();
     }
}
// void Interface::get_uk     public
// writes information about unknown players to arrays given by pointers
// both side and uniform number is unknown
// be sure that given pointers point to arrays size of NUNKNOWNS
// (number of unknown players)
// Input: void
// Returns void
void Interface::get_uk(float *dir, float *dist, float *d_dir, float *d_dist,
                       float *head, int *valid)
{
     int i;
     for( i=0;i<NUNKNOWNS;i++){
          dist[i] = unknowns[i].get_dist();
          dir[i] = -DEG2RAD*unknowns[i].get_dir();
          
          d_dist[i] = unknowns[i].get_D_dist();
          d_dir[i] = -DEG2RAD*unknowns[i].get_D_dir();
          head[i] = -DEG2RAD*unknowns[i].get_heading();

          valid[i] = unknowns[i].is_valid();
     }
}

// void Interface::get_fieldobject     protected
// reads parameters of a fieldobject given in the first parameter
// and writes them to referenced variables
// Input: Fieldobject to read (can be ball, opp_goal, own_goal)
// Returns void
void Interface::get_fieldobject(FieldObject &obj, float &dist, float &dir,
                                float &d_dist, float &d_dir, int &valid )
{
     dist = obj.get_dist();
     dir = -DEG2RAD*obj.get_dir();

     d_dist = obj.get_D_dist();
     d_dir = -DEG2RAD*obj.get_D_dir();

     valid = obj.is_valid();
}

// void Interface::get_ball     public
// writes parameters of the ball to referenced variables
// Input: void
// Returns void
void Interface::get_ball(float &dir, float &dist, float &d_dir, float &d_dist,
                         int &valid)
{
     get_fieldobject(ball,dist,dir,d_dist,d_dir,valid);
}

// void Interface::get_opp_goal     public
// writes parameters of the opponent goal to referenced variables
// Input: void
// Returns void
void Interface::get_opp_goal(float &dir, float &dist, float &d_dir,
                             float &d_dist, int &valid)
{
     get_fieldobject(opp_goal,dist,dir,d_dist,d_dir,valid);
}

// void Interface::get_own_goal     public
// writes parameters of the own goal to referenced variables
// Input: void
// Returns void
void Interface::get_own_goal(float &dir, float &dist, float &d_dir,
                             float &d_dist, int &valid)
{
     get_fieldobject(own_goal,dist,dir,d_dist,d_dir,valid);
}


//
// Calculates correct position of the robot
//
BOOL Interface::correct_position()
{
     int i;
     float closest_flag = 1000.0;
     int closest_flag_i = 1000;
     
     for(i=0;i<NFLAGS;i++)
          if(  flags[i].is_valid()
               && ( flags[i].get_dist() < closest_flag) ){
               closest_flag_i = i;
               closest_flag = flags[i].get_dist();
               
               /* break */
          }
     if(closest_flag_i>=NFLAGS) return FALSE; // couldn't find valid flags => can't calculate position

     i = closest_flag_i;

     int theta;
     theta = (int)angle - flags[i].get_dir();

     x = flags[i].get_x() - flags[i].get_dist()*cos(DEG2RAD*theta);
     y = -flags[i].get_y() + flags[i].get_dist()*sin(DEG2RAD*theta);

     return TRUE;
     
}


//-------------------------
//Sensor Process
//-------------------------

//
// waits messages from server and handles them
//
void Interface::check_message() {
     int r;
     int messagecounter = 0;
     int errorcounter = 0;
     
     while((r = receive_message(recv_buffer,bufsize,&sock))){
          pthread_testcancel();
          switch(r) {
               //     case 0: return ;
          case 1: 
               lock_sensor_data();  // lock data to prevent collisions
               analyze_message(); // put sensor information into the object arrays
               messagecounter++;
               unlock_sensor_data();
               break;
          default: 
               errorcounter++;
               cerr << __FILE__ << " " << __LINE__
                    << ": trouble in receive_message: errno is " << errno << endl ;
               return;
          }
          if(r==0) return;
     }
     return;
}

// this function is called when the sensor thread is initialized. This function
// calls check_message function of the interface and it loops forever or until
// thread is cancelled
void *start_sensor(void *arg)
#pragma argsused
{
#ifndef NO_PRINTS
     if(Const::debug_level_startup()) cerr << "Sensor process started!" << endl ;
#endif
     while(up_and_running){
          P->check_message(); // loop forever!
     }
#ifndef NO_PRINTS
     if(Const::debug_level_exit()) cerr << "Sensor thread ended!!" << endl;
#endif
               
// the next line is never reached
     return (void*) NULL;
}


void *sender(void *arg)
{
     while(1){
          pthread_testcancel();
          P->wait_10ms(); // wait ten ms cycles
          P->send_next_message();
     }
     return (void*)NULL;
}

void Interface::send_next_message()
{
     static int send_err = 0;
     server_cmd cmd;
     lock_send_data();
     if(cmd_buffer->pop(&cmd)){
          if(send_message(cmd.text,sock)) {
               cerr << "Network error while sending" << endl;
               send_err++;
               if(send_err > 99) exit(1);
          }
          
     }
     unlock_send_data();
}


//
// returns number of the fieldobject for given Object name and other sensor information
//
FIELDOBJECTS Interface::which_object(char* Objname,char* arg1,char* arg2,int number)
#pragma argsused
{
     int FO; // FO means fieldobject

//     if(arg2); // arg2 not needed!
     
     // first identify the main groups: player, ball, goal or flag
     for(FO=0;FO<5;FO++){
          if(strcasecmp(Objname,OBJ_NAMES[FO])) {
               // if we come here object was not identified!
               if(FO==4) { // if fiefldobject's number is four and it's not yet identified,
                           // something has gone wrong!!
                    cerr << __FILE__ << " line: "<<__LINE__<<": Unknown object:"
                         << Objname << " arg1: " << arg1 << " arg2: " << arg2 << endl;
                    cerr << recv_buffer << endl;
                    return (FIELDOBJECTS)-1;
                    
//                    exit(-1);
               }
               continue;
          }
          break;
     }

     if(strcmp(Objname,OBJ_NAMES[FO])){
          // if we come here, object's name is written with uppercase,
          // it means that the object
          // is somewhere nearby, but it is not seen.
          if(FO == OBJ_FLAG) {
               FO = OBJ_NEARBY_FLAG;
          }
          if(FO == OBJ_GOAL && arg1[0] != 0){
               cerr << __FILE__ << " " << __LINE__ << endl;
               cerr << "This doesn't function this way!!" << endl;
               exit(1);
               // if we come here, remember to repair goal identification
               // in analyze_message function!!!
          }
          
     }
     
     if(FO == (int)OBJ_PLAYER){
          if(number>=2){
               if(strcasecmp(arg1,team)){
                    if(number == 3) FO = (int)OBJ_OPPONENT;
                    else FO = (int)OBJ_SOMEOPPONENT;
               } else {
                    if(number == 3) FO = (int)OBJ_FRIEND;
                    else FO = (int)OBJ_SOMEFRIEND;
               } 
          }
          // else it's some unknown player;
          // let FO = OBJ_PLAYER
     }
     return (FIELDOBJECTS)FO;
}

//
// analyzes messages sent by the server, puts sensor information into arrays
//
void Interface::analyze_message() {

     int p,length,i ;
     int time;
     float dist = 0.0, d_dist = 0.0; 
     int dir= 0, d_dir = 0, heading = 0; 

     p = 0 ; // index to recv_buffer, how many bytes is already read

     char tmp[512],ObjInfo[256],ObjName[256];

     if(print_sensor_data) cerr << recv_buffer << endl;
     
     
     
//Which sense has sent the buffer     
     i  = sscanf(&(recv_buffer[p]),"(%s %d %n",&tmp[0],&time,&length);
//     cout << "sense: " << tmp << "  time: " << time << " length: " << length << " i: " << i << endl;

     if(strstr(&tmp[0],"error")){
#ifndef NOPRINTS
         cerr << "Warning: error in received buffer "<< recv_buffer << endl;
#endif
         return;
     }

     if(i>1){
          last_visual_time = time;
          time_after_visual = 0;
     }
     
//     curr_t = time;
     
//Increase index by amount of scanned bytes in recv_buffer 
     p += length;

     int buflen = strlen(recv_buffer);
     
     if(!strcasecmp(tmp,"see")){
         if( new_visual_information ){
#ifndef NO_PRINTS
             if(Const::debug_level()) 
                 cerr << "player " << unum 
                      << " new_visual_information is already new, "
                      << "invalidating _all_ information!" << endl;
#endif
             invalidate_all();
         }
// If we come here, data in recv_buffer is visual information
//          cout << "Seeing:";
          new_visual_information = TRUE;

          while(buflen-p > 2){ // loop while not end of buffer
// Separate object name  and it's information
              ObjName[0] = 0;
              
               i = sscanf(&(recv_buffer[p]),"((%[^)]) %[^)]) %n",&ObjName[0],
                          &ObjInfo[0],&length);
               // cout << "ObjName: " << ObjName << "   ObjInfo: " << ObjInfo << "    length: " << length << " i: " << i << endl;
               if(i<2) cout << " WARNING: Object name and/or it's information not received" << endl;
               if(i<=0) break;
               
               p+=length;
     
// Then examine which object it actually is
               char object[64], arg1[16], arg2[16], arg3[16];
               arg1[0] = arg2[0] = arg3[0] = '\0';
               
               i = sscanf( &(ObjName[0]), "%s %s %s %s %n", &object[0], &arg1[0], &arg2[0], &arg3[0], &length);

               // arg1 - arg3 contains for example teamname for players, flag ids, line ids
               
               FIELDOBJECTS obj_type;
               
               obj_type = which_object(object,arg1,arg2,i);

               if((int)obj_type < 0) { // error !!
                    cerr << "Object:" << object << " arg1: " << arg1 << " arg2 "<< arg2
                         << " p: " << p << " length: " << length << endl;
                    cerr << "ObjName: " << ObjName << endl;
                    
                    for(i=0; i< BUFFERSIZE; i++) recv_buffer[i] = 0;
                    p = 0;
                    exit(1);
                    return;
               }
               
               dist = 0.0;
               // Then save distance, direction and velocity direction     
               i = sscanf( &(ObjInfo[0]), "%f %d %f %d %d, %n", &dist, &dir, 
                           &d_dist, &d_dir, &heading, &length);
               if(i==3) cout << "i == 3 for object info: " << &(ObjInfo[0]) 
                             << "obj_type is " << object
                             << "(" << __FILE__ << ", line " << __LINE__ 
                             << ")" << endl;
               //cerr << __LINE__ << endl;
               // lets put information to corresponding arrays
               switch(obj_type){
               case OBJ_FRIEND:
                   friends[atoi(arg2)-1].set_all(time, i, dist, dir, d_dist,
                                                 d_dir,heading );
                    break;
               case OBJ_SOMEFRIEND:
                   uk_friends[c_ukfs++].set_all(time, i, dist, dir, d_dist,
                                                d_dir,heading );
                   break;
               case OBJ_OPPONENT:
                   opponents[atoi(arg2)-1].set_all(time, i, dist, dir, 
                                                   d_dist, d_dir, heading );
                   break;
               case OBJ_SOMEOPPONENT:
                   uk_opponents[c_ukopps++].set_all(time, i, dist, dir,
                                                    d_dist, d_dir,heading );
                   break;
               case OBJ_PLAYER:
                   unknowns[c_ukps++].set_all(time, i, dist, dir, d_dist, 
                                              d_dir, heading );
                   break;
               case OBJ_BALL:
                   ball.set_all(time, i, dist, dir, d_dist, d_dir );
                   break;
               case OBJ_FLAG:
                    flags[flags[0].get_index(arg1,arg2,arg3)].set_all(time, i, dist, dir, d_dist, d_dir );
/*                    cout << "flag index: " << flags[0].get_index(arg1,arg2,arg3)
                         << " dist: " << dist << " dir: " << dir << endl;
                    cout << &(ObjInfo[0]) << endl;
                    */                  
                    break;
               case OBJ_NEARBY_FLAG:
                    int f;
                    // NEARBY_FLAG is the flag which distance is estimated to 
                    // be under 5 meters
                    // (nearest flags are 10 meters distance to each other)
                    float x0,y0;
                    for(f=0;f<NFLAGS;f++){
                         Const::flag_coords((FLAGTYPES)f,x0,y0);
                         if( sqr(x-x0) + sqr(-y-y0) < 25.0)
                         {
                              flags[f].set_all(time, i, dist, dir, d_dist, d_dir );
                              break;
                              
                         }
                    }
                    if(f == NFLAGS){
#ifndef NOPRINTS
                         cout << " FLAG WAS NOT IDENTIFIED!!! " << endl;
                         cout << " Robot coords are: (" 
                              << x << ", " << y << ")" << endl;
#endif
                    }
                    break;
                    
               case OBJ_LINE:
                    lines[lines[0].get_index(arg1)].set_all(time, i, dist, dir, d_dist, d_dir );
                    break;
               case OBJ_GOAL:
                    if( arg1[0] == side_char
                        || (  (get_x() < -50.0)
                        && arg1[0] == 0)){
                         own_goal.set_all(time, i, dist, dir, d_dist, d_dir );
                    }
                    
                    else opp_goal.set_all(time, i, dist, dir, d_dist, d_dir );
                    break;
               }
          }
     }
     else if(!strcasecmp(tmp,"hear")) {
// if it's not visual, it must be audial information
          Audioinfo hear_struct;
          dir = 0;

          hear_struct.time = time;
//          cout << "heard message: " << recv_buffer << endl;
                    
          // if we come here, the robot has heard something
          sscanf(&(recv_buffer[p]),"%s %n",&tmp[0],&length);
          // if sender is referee, tmp holds string "referee" and if robot it self tmp holds "self",
          // else it holds a number
          if(strcasecmp(tmp,"referee") == 0){
               //               Referee is talking!!
               p += length; // it was referee increase index
               hear_struct.talker = TALKER_REFEREE;
               dir = 0;
          } else if(strcasecmp(tmp,"self") == 0){
               p += length; // it was "self" increase index
               hear_struct.talker = TALKER_SELF;
               dir = 0;
          } else {
               hear_struct.talker = TALKER_PLAYER;
               sscanf(tmp,"%d %n", &dir,&length);
               p+=length;
          }

          hear_struct.dir = -DEG2RAD*dir;
               
          char heard_message[512];
          strcpy(heard_message,&(recv_buffer[p]));
          // cut the last parenthese away from the message
          heard_message[strlen(heard_message)-2]= '\0'; 
          strcpy(hear_struct.message,heard_message);
          hear_struct.valid = 1;
          put_heard_msg(hear_struct);
     }

     else if(!strcasecmp(tmp,"sense_body")) {
          char view_qual[10],view_angle[10];
          i = sscanf(&(recv_buffer[p]),"(view_mode %s %[^)]) (stamina %f %f) (speed %f) (kick %d) (dash %d) (turn %d) (say %d))%n",
                 view_qual,view_angle,
                 &sb.short_stamina,&sb.long_stamina,&sb.speed,
                 &sb.kicks,&sb.dashes,&sb.turns,&sb.says,&length);
          sb.time = time;
          sb.valid = 1;
          if(i != 9) cerr << "WRONG" << i << endl;
          
          /*cerr << "i: " << i << endl;
          cerr << "qual: " << view_qual << " angle: " << view_angle << endl;

          if(Cons2::me_unum()== 11 ){
              
              cerr << recv_buffer ;
              cerr << "stamina: " << sb.short_stamina << " " << sb.long_stamina
               << " spd: " << sb.speed << " kicks " << sb.kicks 
               << " dashes " << sb.dashes << " turns " << sb.turns 
                   << " says " << sb.says << endl;
          }
          */
          if(!strcmp(view_qual,"high")) sb.view_qual = VQ_High;
          else if(!strcmp(view_qual,"low")) sb.view_qual = VQ_Low;
          else {
               cerr << __FILE__ << " " << __LINE__ << " View quality unknown!"
                    << endl;
               sb.view_qual = VQ_Low;
          }
          if(!strcmp(view_angle,"normal")) sb.view_angle = VA_Normal;
          else if(!strcmp(view_angle,"wide")) sb.view_angle = VA_Wide;
          else if(!strcmp(view_angle,"narrow")) sb.view_angle = VA_Narrow;
          else {
               cerr << __FILE__ << " " << __LINE__ << " View angle unknown!"
                    << endl;
               sb.view_angle = VA_Narrow;
          }
                   
     }
// if information not visual, audial nor body sense, what it can be?
     else{
          cerr << "WARNING in "<<__FILE__ << ", line " << __LINE__
               << ", sensor information is invalid: " << recv_buffer << endl;
          exit(1);
     }
     
     return;
}

//
// invalidates all sensor information
//
void Interface::invalidate_all()
{
     int i;
     // first zero counters of unknown players
     c_ukfs = c_ukopps = c_ukps = 0;

     new_visual_information = FALSE;
          
     for(i=0; i<NLINES; i++)
          lines[i].invalidate();

     for(i=0; i<NFRIENDS; i++)
          friends[i].invalidate();

     for(i=0; i<NUKFRIENDS; i++)
          uk_friends[i].invalidate();
    
     for(i=0; i<NOPPS; i++)
          opponents[i].invalidate();
    
     for(i=0; i<NUKOPPS; i++)
          uk_opponents[i].invalidate();
    
     for(i=0; i<NUNKNOWNS; i++)
          unknowns[i].invalidate();
    
     for(i=0; i<NFLAGS; i++)
          flags[i].invalidate();

     sb.valid = 0;
         
//     for(i=0;i<NGOALS;i++)
//          goals[i].invalidate();
     opp_goal.invalidate();
     own_goal.invalidate();
          
     ball.invalidate();
}


//
// Main function of class Interface
//
// pauses until a signal from sensor cycle is received
// analyzes new sensor information and calculates situation
// if there is no new information, estimates situation

void Interface::tick() {
     static int send_err = 0;
//     invalidate_all();  // invalidate sensor information, present sensor info is old for the next control cycle
     if(up_and_running){

         if(signal_arrived) cerr <<"Player " << unum << ": Control cycle takes too long time" << endl;

          if(Const::debug_level() > 4) cerr << "Interface.tick pausing...";
          // 100ms lock is unlocked every 100milliseconds. Since this main
          // main thread newer unlocks the lock, it pauses here until
          // alarm_thread unlocks the lock. (if control cycle <100ms)
          wait_100ms();   // wait for 100ms timer
          signal_arrived = 0;
     }
     // since we want to have sensebody information all the time, we 
     // put the request into roundbuffer where it is later sent 
     // to the server by send_next_message function
     command_sensebody();
     
     // Here we send the command calculated at last round. It is done here, because
     // sending interval should be approximately constant. If interval is changing
     // all the time, some commands might allways be overrun by the next command. 
     if(Const::debug_level() >4) cerr << " sending message...";

     lock_send_data();
     if(move_command[0])
          if(send_message(move_command,sock)){
               send_err++;
               cerr << send_err << " errors in move_command sending" 
                    << endl;
               if(send_err>10) exit(-1);
          }
     move_command[0] = 0;
     unlock_send_data();

     if(Const::debug_level() > 4) cerr << "Locking data" << endl;
     
     lock_sensor_data(); // lock sensor data, sensor data must not be overwritten while it is read
     // data must be unlocked after all sensor information is read, else sensor thread cannot
     // write new data
     
     if(new_visual_information){
          if(Const::debug_level() > 4)cerr << "Computing situation" << endl;
          compute_situation();    // calculate new position etc...
     }
     else {
          if(Const::debug_level() > 4) cerr << "Estimating situation" << endl;
          estimate_situation();   // estimate position etc...
     }
     
     if(Const::debug_level() > 4) cerr << " tick returning" << endl ;
}

     

extern "C" {
int usleep(unsigned int);
}

void Interface::wait_100ms(){
    pthread_mutex_lock(&mutex_100_ms);
    while(!flag_100_ms){
        pthread_cond_wait(&cond_100_ms, &mutex_100_ms);
    }
    flag_100_ms = 0;
    pthread_mutex_unlock(&mutex_100_ms);
}
void Interface::launch_100ms_wait()
{
    pthread_mutex_lock( &mutex_100_ms );
    flag_100_ms = 1;
    pthread_cond_broadcast( &cond_100_ms );
    pthread_mutex_unlock( &mutex_100_ms );
}

void Interface::wait_10ms(){

    pthread_mutex_lock( &mutex_10_ms );
    while(!flag_10_ms){
        pthread_cond_wait(&cond_10_ms, &mutex_10_ms);
    }
    flag_10_ms = 0;
    pthread_mutex_unlock( &mutex_10_ms );
}
void Interface::launch_10ms_wait()
{
    pthread_mutex_lock( &mutex_10_ms );
    flag_10_ms = 1;
    pthread_cond_broadcast( &cond_10_ms );
    pthread_mutex_unlock( &mutex_10_ms );
}

void *wait_alarm(void *p) 
{
     struct timeval timeout;
     timeout.tv_sec  = 0;
     timeout.tv_usec = 1000 * RECV_INTERVAL_MILLISEC;
     int ten_counter = 0;
    
     // TESTING - report on display how accurately the 100 ms timer works
     // hrtime_t prev;
     // hrtime_t current;
     // float diff;
     // prev = gethrtime();
     // END TESTING
     
     for ( ;; ) {
          // sleep approximately RECV_INTERVAL_MILLISEC milliseconds
          if (select(0, (fd_set *) 0, (fd_set *) 0, (fd_set *) 0, &timeout) < 0)
               cerr << "wait_alarm(): select error" << endl; 
          ten_counter++;
          
          if (ten_counter >= (100/RECV_INTERVAL_MILLISEC)) { 
               // if 100 ms has elapsed from the last time, it is time to wake up
               // control system execution (and send the control command)
               ten_counter = 0;
               signal_arrived = 1;
               P->launch_100ms_wait();
               pthread_testcancel();

               // TESTING
               // if(Cons2::me_unum() == GOAL_KEEPER_UNUM) {
               // current = gethrtime();
               // diff = (current - prev) / 1000000.0;
               // cout << "diff = " << diff << " should be 100" << endl;
               // prev = current;
               // }
               // END TESTING
          } else {
               // time to send supplementary commands; wake up sending thread.
               // we do not send supp. commands right after move command
               // has bees sent. That is why we do the test below
              if(ten_counter > 1) P->launch_10ms_wait();
               pthread_testcancel();
          }
     }
     return (void *)NULL;
}

void Interface::print_robots(Player *r,int N)
{
    int i;
    for( i=0;i<N;i++ ){
        cerr << i << " dis " << r[i].get_dist()
             << " dir "   << r[i].get_dir() 
             << " d_dis " << r[i].get_D_dist() 
             << " d_dir " << r[i].get_D_dir() 
             << " head "  << r[i].get_heading() 
             << " valid " << r[i].is_valid() << endl;
    }
}


void Interface::print_all_robots_and_exit()
{
    cerr << "Friends" << endl;
    print_robots(friends,NFRIENDS);     // coplayers, player's index to this array is his Uniform number
    cerr << "Uk_friends" << endl;
    print_robots(uk_friends,NUKFRIENDS);// max 10 unknown friends ( Unum is unknown )
    cerr << "opponents" << endl;
    print_robots(opponents,NOPPS);       // max 11 known opponents (in order of Unum)
    cerr << "uk_opponents" << endl;
    print_robots(uk_opponents,NUKOPPS);  // max 11 unknown opponents ( Unum is unknown )
    cerr << "unknowns" << endl;
    print_robots(unknowns,NUNKNOWNS);   // max 11+10 unknown players (even side unknown)
    cerr << "Number of unknowns: friends " << c_ukfs 
         << " opponents "<< c_ukopps 
         << " totally unknowns " << c_ukps<< endl;

    
    exit(1);
}
