/* File    : tools.c
 *
 * Functions ending to "2" are the new ones,
 * delete the 2's if using g++.
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <iostream.h>
#include "tools.h"
#include "icomm.h"
#include <assert.h>

extern double sign (double value)
{
  return (value < 0) ? -1 : 1;
}

extern float sign (float value)
{
  return (value < 0) ? -1 : 1;
}

extern int sign (int value)
{
  return (value < 0) ? -1 : 1;
}

extern float min (float a, float b)
{
  return (a <= b) ? a : b;
}

extern int min (int a, int b)
{
  return (a <= b) ? a : b;
}

extern float max (float a, float b)
{
  return (a >= b) ? a : b;
}

extern int max (int a, int b)
{
  return (a >= b) ? a : b;
}




int round (double value)
{
  return (int) (value + sign(value) * 0.5);
}

int round(float value)
{
  return (int) (value + sign(value) * 0.5);
}

double deg_to_rad (double deg)
{
  return (deg * M_PI / 180.0);
}

double rad_to_deg (double rad)
{
  return (rad * 180.0 / M_PI);
}


double calc_distance (double s_x, double s_y, double g_x, double g_y)
{
  double d_x = g_x - s_x;
  double d_y = g_y - s_y;
  
  return sqrt (d_x * d_x + d_y * d_y);
}

double calc_angle (double s_x, double s_y, double g_x, double g_y)
     /* <- angle of vector start->goal;
      *    0 if start == goal
      */
{
  double d_x = g_x - s_x;
  double d_y = g_y - s_y;

  if (d_x == 0.0 && d_y == 0.0)
    return 0;
  
  return atan2 (d_y, d_x);
}


float angle_diff (float alfa, float beta)
     /* <- -pi <= alfa-beta <= +pi
      */
{
  float gamma = alfa - beta;

  if (gamma > M_PI)         return gamma - (2 * M_PI);
  else if (gamma < (-M_PI)) return gamma + (2 * M_PI);
  else                      return gamma;
}


double calc_angle_difference (double alfa, double beta)
     /* <- -pi <= alfa-beta <= +pi
      */
{
  double gamma = alfa - beta;

  if (gamma > M_PI)         return gamma - (2 * M_PI);
  else if (gamma < (-M_PI)) return gamma + (2 * M_PI);
  else                      return gamma;
}


float calc_angle_sum (float alfa, float beta)
     /* <- -pi <= alfa+beta <= +pi
      */
{
  float gamma = alfa + beta;

  if (gamma > M_PI)         return gamma - (2 * M_PI);
  else if (gamma < (-M_PI)) return gamma + (2 * M_PI);
  else                      return gamma;
}


double calc_opposite_angle (double alfa)
     /* <- -pi <= opposite_angle <= +pi
      */
{
  return calc_angle_difference (alfa, (double) M_PI);
}


double rotate_angle (double alfa, double amount)
     /* -> angle from line A to line B and amount of
      *    rotation of line A, anticlockwise positive
      * <- new angle from line A to line B
      */
{
  /* If the angle from line A to line B is alfa and line A
     is rotated amount units anticlockwise, the new angle
     from line A to line B is alfa - amount.
     */
  return calc_angle_difference (alfa, amount);
}


void rotate_point (float *p_x, float *p_y, double amount)
     /* amount of translation along x-axis */
{
  double angle, dist;

  /* original angle */
  angle = calc_angle (0, 0, *p_x, *p_y);

  /* after rotation */
  angle = calc_angle_difference (angle, amount);
      
  dist  = calc_distance (0, 0, *p_x, *p_y);
  *p_x  = dist * cos (angle);
  *p_y  = dist * sin (angle);
}


void translate_point (float *p_x, float *p_y, double amount)
{
  /* Robot moves always in the direction of x-axis! */
  *p_x = *p_x - amount;
}


void add_vector_to_point (double x, double y, double dir, double len,
			  float *r_x, float *r_y)
{
  *r_x  = x + len * cos (dir);
  *r_y  = y + len * sin (dir); 
}


/* calculates intersection point of lines (a_x,a_y,a_dir) and (b_x,b_y,b_dir).
 * if intersection point exists, it is assigned to (i_x,i_y) and YES is returned;
 * otherwise (0,0) is assigned to (i_x,i_y) and NO is returned
 */
int calc_line_intersection (double a_x, double a_y, double a_dir,
			    double b_x, double b_y, double b_dir,
			    float *i_x, float *i_y)
{
  double /*x, y,*/ a_tan, b_tan;

  a_tan = tan (a_dir);
  b_tan = tan (b_dir);
    
  /* parallel lines */
  if (a_dir == b_dir || a_dir == calc_opposite_angle (b_dir))
    {
      *i_x = 0;
      *i_y = 0;
      return NO;
    }

  if (fabs (a_dir) == M_HALF_PI)
    {
      *i_x = a_x;
      *i_y = b_y + (*i_x - b_x) * b_tan;
    }
  else if (fabs (b_dir) == M_HALF_PI)
    {
      *i_x = b_x;
      *i_y = a_y + (*i_x - a_x) * a_tan;
    }
  else
    {
      *i_x = (b_y - a_y + a_x * a_tan - b_x * b_tan) / (a_tan - b_tan);
      *i_y = a_y + (*i_x - a_x) * a_tan;
    }
  return YES;
}

// This function calculates the intersections points between 
// the given line (x,y, dir) and the circle (x - c_x)^2 + (y - c_y)^2 = r^2.
// It returns the number of intersection points - 0, 1, or 2 - 
// and changes values of its intersection point
// parameters (i1_x, i1_y) and (i2_x, i2_y). 
int line_circle_intersection (float l_x, float l_y, float l_dir,
                              float c_x, float c_y, float r,
                              float& i1_x, float& i1_y,
                              float& i2_x, float& i2_y)
{
     float orig_c_x = c_x;
     float orig_c_y = c_y;
     
     // move origin to the center of the circle
     l_x -= c_x;
     l_y -= c_y;
     c_x = 0;
     c_y = 0;

     float m, b, discr;

     // line parameters, y = mx + b 
     m = tan (l_dir);
     b = l_y - m * l_x;

     // Substitute for y and and check the discriminant of the quadratic equation 
     discr = 4 * (-(b*b) + (r*r) * (1 + (m*m)));

     if (discr > 0) {
         float temp = sqrt(discr);
     
          i1_x = (((-2.0 * m * b) + temp)
                 / (2.0 * (1.0 + (m * m))));
          i1_y = (m * (i1_x - l_x) + l_y);

          i2_x = (((-2.0 * m * b) - temp)
                 / (2.0 * (1.0 + (m * m))));
          i2_y = (m * (i2_x - l_x) + l_y);
          
          i2_x += orig_c_x;
          i2_y += orig_c_y;
          i1_x += orig_c_x;          
          i1_y += orig_c_y;
          
          return 2; 
     }

     else if (discr == 0) {
     // we should calculate here the tangent 
          i2_x = i1_x = orig_c_x + r;
          i2_y = i1_y = orig_c_y;
          return 1;  
     }

     else 
       // if (discr < 0) 
       {
       // line and circle do not intersect 
          i2_x = i1_x = 0.0;
          i2_y = i1_y = 0.0;
          return 0;  
     }
} // End of line circle intersection.

/* This function calculates the distance of the point (p_x, p_y) from line
 * (l_x, l_y, l_dir), where l_dir is the direction of the line, that is,
 * tan (dir) is the slope of the line. Thus, the line equation is:
 * (l_y - y) = tan (l_dir) * (l_x - x)
 * Reference: Cairns, Edward J. (1967) Mathematics for applied engineering,
 *            Prentice-Hall, page 372.
 */
float calc_point_dist_from_line (float p_x, float p_y,
				 float l_x, float l_y, float l_dir)
{
  float m, result;
  
  if (fabs (l_dir) == M_HALF_PI)
    {
      result = l_y - p_y;
    }
  else
    {
      m = tan (l_dir);
      result = (m * p_x - p_y + l_y - m * l_x)
               /
               sqrt (m * m + 1);
    }
  return fabs (result);
}


/* This function checks whether the given line (x,y, dir) intersects the
 * circle x^2 + y^2 = r^2. It returns the number of intersection points.
 */
int line_intersects_circle_p (float l_x, float l_y, float l_dir, float r)
{
  float m, b, discr;
  
  /* line parameters, y = mx + b */
  m = tan (l_dir);
  b = l_y - m * l_x;

  /* Substitute for y and and check the discriminant of the quadratic equation */
  discr = 4 * (-(b*b) + (r*r) * (1 + (m*m)));

  if (discr > 0)
    return 2;  /* intersects two times */
  else if (discr == 0)
    return 1;  /* tangent */
  else /* if (discr < 0) */
    return 0;  /* does not intersect */
}

void circular_inc(int& value,int maxvalue)
{
     // increments by one, if value becomes greater than maxvalue
     // it is set to zero
     value++;
     if(value>maxvalue) value = 0;
}

void calc_dir_and_dist(float& dir, float& dist,float x, float y)
{
          if(fabs(x)<0.00001) x = (x<0?-0.00001:0.00001);
          if(fabs(y)<0.00001) y = (y<0?-0.00001:0.00001);
          
          dist = sqrt( x*x + y*y );
          dir = atan( y / x );
          // if x is negative, atan gives wrong result
          if(x<0) dir += M_PI;
          return;
}

void calc_x_and_y(float& x, float& y, float dir, float dist)
{
     x = cos(dir)*dist;
     y = sin(dir)*dist;
}

float sqr(float x)
{
     return x*x;
}

void normalize_angle(float& angle)
{
    if(fabs(angle)>=20.0) cerr << "ANGLE: " << angle << endl;
    assert( fabs(angle) < 20.0 );
    while(angle > M_PI) angle -= 2 * M_PI;
    while(angle < -M_PI) angle += 2 * M_PI;
}


int coords2gpos(float x, float y)
{
     int k,i;
     
     x += MAX_X;
     y += MAX_Y;
     
     k = 1 + (int)(9.0*x/(2*MAX_X));
     if(k<1) k = 1;
     else if(k>9) k=9;
     
     i = (int)(7.0*y/(2*MAX_Y));
     if(i<0) i = 0;
     else if(i>6) i=6;

     return (k+9*i);
}


int gpos2coords(int gpos, float& x, float& y)
{
     if(gpos > 63 || gpos < 1) return 0;
     
     y =  (2*MAX_Y)/7.0 * (0.5 +   ((gpos - 1) / 9));
     y -= MAX_Y;

     x =  (2*MAX_X)/9.0 * (-0.5 + ( (gpos - 1)%9 + 1));
     x -= MAX_X;

     return 1;
}


// this function returns a gain value for a given distance.
// also the parameters of the gain function are passed as arguments.
// the gain values form a step as a function of distance:
//
//    end_g->          +------- 
// x                  /        
// x                 /           
// begin_g -> ------+            
//           
//                  A  A     
//                  |  |         
//        left_step_d  |    
//                     right_step_d
// -jpr 10aug97


float dst2step_gain(float dst,
                    float lft_stepd, float rgt_stepd,
                    float begg, float endg)
{
     assert(rgt_stepd >= lft_stepd);
     
     float gain;
     
     if (dst <= lft_stepd)
          gain = begg;
     
     else if (dst >= rgt_stepd)
          gain = endg;
     
     else
          gain = begg +
               (endg - begg)*(dst - lft_stepd)/(rgt_stepd - lft_stepd);
     
     return(gain);
}


// this function returns a gain value for a given distance.
// also the parameters of the gain function are passed as arguments.
// the gain values form a hat as a function of distance:
//
// x         left_crown_d    right_crown_d
// x                   |       |
// x                   V       V
// x
// x       top_g->     +-------+
// x                  |         |
// x                 |           |
// begin_g -> ------+             +-------- <- end_g
// x         
// x                A             A
// x                |             |
// x            left_brim_d      right_brimd
// x
// -jpr 10aug97

float dst2hat_gain(float dst, float lft_brimd, float lft_crownd,
                   float rgt_crownd, float rgt_brimd,
                   float begg, float topg, float endg)
{
     assert(rgt_brimd >= rgt_crownd && rgt_crownd >= lft_crownd && lft_crownd >= lft_brimd);
     
     float gain;
     
     if (dst <= lft_brimd)
          gain = begg;
     
     else if (dst <= lft_crownd)
          gain = begg
               + (topg - begg)*(dst - lft_brimd)/(lft_crownd - lft_brimd);
     
     else if (dst <= rgt_crownd)
          gain = topg;
     
     else if (dst <= rgt_brimd)
          gain = topg
               + (endg - topg)*(dst - rgt_crownd)/(rgt_brimd - rgt_crownd);
     
     else
          gain = endg;
     
     return(gain);
}

// Converts global location to local(relative) position
// alse converts global velocity to local velocity
Vector g_loc2loc(Vector g_loc,Vector me_g_loc,float me_head)
{
     Vector temp;
     
     temp = g_loc - me_g_loc;
     temp.turn( -me_head );
     return temp;
}

// converts local location to global
// also converts local velocity to global velocity
Vector loc2g_loc(Vector loc, Vector me_g_loc, float me_head)
{
     Vector temp;
     temp = loc;
     temp.turn( me_head );
     temp = temp + me_g_loc;
     return temp;
}
Vector vel2mix_vel(Vector g_vel, float me_head)
{
    Vector temp;
    temp = g_vel;
    temp.turn( -me_head );
    return temp;
}

     

/* end of file */

