// -*- mode:C++ -*-                                     
// file     : tools.h
// includes : declarations for application independent
//            basic constants, data types, and functions
///

#ifndef H_TOOLS
#define H_TOOLS

#include <math.h>

#define M_HALF_PI   M_PI_2 // 1.570796327
#define M_DOUBLE_PI 2.0*M_PI //6.283185307

#include "Cons2.h"
#include "Vector.h"


#define MESSAGE(m) {fprintf(stderr,m); fflush(stderr);}

// default weight: no prior knowledge
//
#define DEFAULT_CERTAINTY (0.5)
#define DEFAULT_WGT (0.5)

#define MAX(a, b)  ((a) > (b)) ? (a) : (b)
#define MIN(a, b)  ((a) < (b)) ? (a) : (b)

#define RAD2DEG (float)(180.0 / M_PI) 
#define DEG2RAD (float)(M_PI / 180.0)


extern double sign (double value);
extern float sign (float value);
extern int sign (int value);

extern float min (float a, float b);
extern int min (int a, int b);

extern float max (float a, float b);
extern int max (int a, int b);

extern int round (double value);
extern int round(float value);
extern double deg_to_rad (double deg);
extern double rad_to_deg (double rad);
extern double calc_distance (double s_x, double s_y, double g_x, double g_y);
extern double calc_angle (double s_x, double s_y, double g_x, double g_y);
extern float angle_diff (float alfa, float beta);
extern double calc_angle_difference (double alfa, double beta);
extern float calc_angle_sum (float alfa, float beta);
extern double calc_opposite_angle (double alfa);
extern double rotate_angle (double alfa, double amount);
extern void rotate_point (float *p_x, float *p_y, double amount);
extern void translate_point (float *p_x, float *p_y, double amount);
extern void add_vector_to_point (double x, double y, double dir, double len,
				 float *r_x, float *r_y);
extern int calc_line_intersection (double a_x, double a_y, double a_dir,
				   double b_x, double b_y, double b_dir,
				   float *i_x, float *i_y);
extern int line_circle_intersection (float l_x, float l_y, float l_dir,
                                     float c_x, float c_y, float r,
                                     float& i1_x, float& i1_y, 
                                     float& i2_x, float& i2_y);

extern float calc_point_dist_from_line (float p_x, float p_y,
					float l_x, float l_y, float l_dir);
extern int line_intersects_circle_p (float l_x, float l_y,
				     float l_dir, float r);

extern void circular_inc(int& value,int maxvalue);
extern float sqr(float);
extern void normalize_angle(float& angle);

extern int coords2gpos(float x, float y);
extern int gpos2coords(int gpos, float& x, float& y);


float dst2step_gain(float dst,
                    float lft_stepd, float rgt_stepd,
                    float begg, float endg);

float dst2hat_gain(float dst, float lft_brimd, float lft_crownd,
                   float rgt_crownd, float rgt_brimd,
                   float begg, float topg, float endg);

// Converts global location to local(relative) position
// alse converts global velocity to local velocity
Vector g_loc2loc(Vector g_loc,Vector me_gloc,float me_head);

// converts local location to global
// also converts local velocity to global velocity
Vector loc2g_loc(Vector loc, Vector me_g_loc, float me_head);

// converts global velocity vector to a vector where length is length
// of global velocity and direction is relative to robot's direction
Vector vel2mix_vel(Vector g_vel, float me_head);

#endif // H_TOOLS 

// end of file 
