// -*- mode:C++ -*-
//Header:
//Title: UDP interface program for Soccer Server
//File: udpif.C
//Creator: NODA, Itsuki
//Date: 1996/05/19
//EndHeader:

//History:
// This file is based on client.C in the package of SoccerServer.
//	by NODA, Itsuki. [1996/06/06] (Ver. 1.0)
//
// Change O_NONBLOCK to O_NDELAY for fcntl.
// (suggestion from David E. Moriaty <moriarty@AIC.NRL.Naby.Mul>
//  and Peter Stone <pstone@cs.cmu.edu>)
//	by NODA, Itsuki. [1996/06/07] (Ver. 1.1)
//
//EndHistory:

#include "udpif.h"
#include <iostream.h>

//======================================================================
// Open UDP connection
// init_connection(char *host, int port)
//  host: hostname or IP address of host
//  port: the number of UDP port
//
//  Result:
//	Return assigned Socket struct.  
//	If an error occurs, Socket.socketfd = -1.
//======================================================================

Socket init_connection(char *host, int port)
{
     struct hostent 	*host_ent ;
     struct in_addr     *addr_ptr ;
     struct sockaddr_in	cli_addr ;
     int			    sockfd ;
     //int                val ;
     Socket		        sock ;
     
     sock.socketfd = -1 ;
     
     if((host_ent = (struct hostent *)gethostbyname(host)) == NULL) {
          /* Check if a numeric address */
          if(inet_addr(host) == -1){
               return sock ;
          }
     }
     else{
          addr_ptr = (struct in_addr *) *host_ent->h_addr_list ;
          host = inet_ntoa(*addr_ptr) ;
     }
     
//  Open UDP socket.
     
     if( (sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
          return sock ;  	/* Can't open socket. */
     }
     
/*     val = fcntl(sockfd, F_GETFL, 0) ;

     val |= O_NDELAY ;
     fcntl(sockfd, F_SETFL, val) ;
     */        
// Bind any local address.
     
     bzero((char *) &cli_addr, sizeof(cli_addr)) ;
     cli_addr.sin_family		= AF_INET ;
     cli_addr.sin_addr.s_addr	= htonl(INADDR_ANY) ;
     cli_addr.sin_port		= htons(0) ;
     
     
     if(bind(sockfd, (struct sockaddr *) &cli_addr,
             sizeof(cli_addr)) < 0){
          return sock ;  /* Can't bind local address */
     }
     
     
//  Fill in the structure with the address of the server.
     
     sock.socketfd = sockfd ;
     
     bzero((char *) &sock.serv_addr, sizeof(sock.serv_addr)) ;
     sock.serv_addr.sin_family		= AF_INET ;
     sock.serv_addr.sin_addr.s_addr	= inet_addr(host) ;
     sock.serv_addr.sin_port		= htons(port) ;
     
     return sock ;
}

//======================================================================
// Send message to socket
// int send_message(char *buf, Socket sock)
//	buf: content of the message to send
//	sock: socket 
//	
//	Result: Return -1 if it fails to send.
//======================================================================

int send_message(char *buf, Socket sock)
{
     int n ;
     
     n = strlen(buf) ;
#ifdef DEBUG
     printf( "send %d : %s",
             ntohs(sock.serv_addr.sin_port),buf);
#endif
     if( sendto(sock.socketfd, buf, n, 0,
                (struct sockaddr *)&sock.serv_addr, sizeof(sock.serv_addr))
         != n )
          return (-1) ;
     else
          return 0 ;
}

//======================================================================
// int receive_message(char *buf, int size, Socket *sock)
//	buf: buffer to receive a message
//	size: size of buf
//	sock: socket
//
//  Result:
//	Return -1 if it fails to receive.
//	Return 0 if there is no message in the socket.
//	Return 1 if a message is received successfully.
//======================================================================

int receive_message(char *buf, int size, Socket *sock)
{
     int			n,servlen ;
     struct sockaddr_in	serv_addr ;
     
     servlen = sizeof(serv_addr) ;
     n = recvfrom(sock->socketfd, buf, size, 0,
                  (struct sockaddr *)&serv_addr, &servlen);
     
     if(n < 0)
          if( n == -1 && errno == EWOULDBLOCK){
               buf[0] = '\0' ;
               return 0 ;
          }
          else {
               perror("perskele!");
               return (-1) ;
          }
     
     
     else{
#ifdef DEBUG
          printf( "recv %d : ", ntohs(serv_addr.sin_port));
#endif
          sock->serv_addr.sin_port = serv_addr.sin_port ;
          buf[n] = '\0' ;
          
	if(n == 0)
         return 0 ;
	else 
         return 1 ;
    }
}


//======================================================================
// void message_loop(FILE* fpin, FILE* fpout, Socket sock)
//	fpin: file descripter for input data to send
//	fpout: file descripter for output data to receive
//	sock: socket to the server
//
//  Result:
//	It loops eternally
//======================================================================
/*
void message_loop(FILE* fpin, FILE* fpout, Socket sock)
{
     fd_set	readfds ;
     int		in, max_fd, n ;
     char	buf[MAXMESG];
     
     in = fileno(fpin) ;
     while(1){
          FD_ZERO(&readfds) ;
          FD_SET(in, &readfds) ;
          FD_SET(sock.socketfd, &readfds) ;
          
          max_fd = ((in > sock.socketfd) ? in : sock.socketfd) + 1 ;
          if(select(max_fd, &readfds, 0, 0, 0) == -1){
               perror("select") ;
               break ;
          }
          
          if(FD_ISSET(in, &readfds)){
               fgets(buf, MAXMESG, fpin) ;
               if(send_message(buf, sock) == -1)
                    break ;
          }
          
          if(FD_ISSET(sock.socketfd, &readfds))
               {
                    n = receive_message(buf, MAXMESG, &sock) ;
                    if(n == -1)
                         break ;
                    else if(n > 0)
                         fputs(buf, fpout) ;
                    fflush(stdout) ;
               }
     }
}
*/
//======================================================================
// void close_connection(Socket sock)
//	sock: Socket to close
//======================================================================


void close_connection(Socket sock)
{
    close(sock.socketfd) ;
}
