// -*- mode:C++ -*-
/* file     : icomm.h
 * includes : declarations for data types and functions
 *            used in internal communication
 */

#ifndef _ICOMM_H
#define _ICOMM_H 1

#include <strings.h>
#include "trace.h"

// boolean values 
typedef enum {NOT_VALID = (-1), NO = 0, YES = 1} FLAG;

// certainty describes, how reliable data is. value 0
// describes totally unreliable data and 1.0 completely
// reliable
//
typedef float CERTAINTY;

// System states
//  
// After initialization the system goes automatically to SENSORS_ON
// state. After that, the state is changed according to user
// commands. Behaviors are inhibited based on the state information.
//

// System states
//  
// After initialization the system goes automatically to SENSORS_ON
// state. After that, the state is changed according to user
// commands. Behaviors are inhibited based on the state information.
//
typedef enum {
  INITIALIZING_SOCCER = 1,
  SENSORS_ON        = 2,
  LOOKAROUND_ON     = 3,
  BALL_BEHAVIORS_ON = 4,
  GOAL_BEHAVIORS_ON = 5,
  ALL_ON            = 6,
  QUIT_SOCCER       = 7,
  RESET_SOCCER      = 8,
  PAUSE_SOCCER      = 9
} SYSTEM_STATE;


// id numbers for modules and signals
typedef enum
{
     // HERE_THEY_ARE
     DEFAULT_ID = 0,
// actuators
     AC_BASE_ID, AC_KICK_ID, AC_SUPPLY_ID,
// arbiters
     AR_MOVE_ID, AR_KICK_ID, AR_META_ID,
// behaviors
     BEHAVIOR_ID,         
     BE_KICKTOGOAL_ID, BE_LOOKAROUND_ID, BE_TURNTOBALL_ID,
     BE_MOVETOBALL_ID, BE_TURNTOGOAL_ID, BE_GOTO_BALL_ID,
     BE_GOTO_BORDER_ID, BE_GOTO_BRD_FLAG_ID, BE_GOTO_TMATE_ID,
     BE_GOTO_OPP_ID, BE_GOTO_OPPG_FLAG_ID, BE_GOTO_OWNG_FLAG_ID,
     BE_GOTO_OPPGOAL_ID, BE_GOTO_OWNGOAL_ID,
     BE_GOTO_NEAR_ID, BE_GOTO_BALL_DIR_ID, BE_GOTO_DEFLOC_ID,

     BE_KICKTO_BORDER_ID, BE_KICKTO_BRD_FLAG_ID, BE_KICKTO_TMATE_ID,
     BE_KICKTO_OPP_ID, BE_KICKTO_OPPG_FLAG_ID, BE_KICKTO_OWNG_FLAG_ID,
     BE_KICKTO_OPPGOAL_ID, BE_KICKTO_OWNGOAL_ID,
     BE_KICKTO_NEAR_ID, 

// behaviors for special situations
     BE_GOTO_SPECIAL_LOC_ID,
     BE_JUMPTO_DEFLOC_ID, BE_SHOUT_ID, BE_KICK_SPECIAL_ID,
// sensor
     SENSOR_ID,
// virtual sensors
     VS_REFEREE_ID, VS_TEAM_AUDIO_ID, VS_DEFENCE_WGT_ID, VS_BALL_POSSD_ID,
     VS_RISK_LVL_ID,
// markers
     MA_ME_ID, MA_BALL_ID, MA_GOAL_ID, MA_BORDER_ID, MA_TEAMMATE_ID, MA_OPPONENT_ID,
     MA_FLAGS_ID, MA_DEFLOC_ID, MA_SPECIAL_LOC_ID,
// signals
     SI_ME_MARKER_ID, SI_BALL_MARKER_ID, SI_OPP_GOAL_MARKER_ID,
     SI_OWN_GOAL_MARKER_ID, SI_BORDER_MARKER_ID, SI_TEAMMATE_MARKER_ID,
     SI_OPPONENT_MARKER_ID, SI_BRD_FLAGS_MARKER_ID, SI_OPPG_FLAGS_MARKER_ID,
     SI_OWNG_FLAGS_MARKER_ID, SI_DEFLOC_MARKER_ID, SI_SPEC_LOC_MARKER_ID,
     SI_SMO_ID, SI_AC_BASE_IMPULSE_ID, SI_AC_KICK_IMPULSE_ID,
// sensor signals
     SI_SENSOR_ID,
     SI_BALL_SSIG_ID, SI_OPP_GOAL_SSIG_ID, SI_OWN_GOAL_SSIG_ID,
     SI_PLAYERS_SSIG_ID, SI_LINES_SSIG_ID, SI_FLAGS_SSIG_ID,
     SI_AUDIO_SSIG_ID,
     SI_MOTION_CMD_ID, SI_KICK_CMD_ID, SI_SUPPLY_CMD_ID,
// virtual sensor signals
     SI_REFEREE_VS_ID, SI_BALL_POSSD_VS_ID, SI_RISK_LVL_VS_ID,
     SI_TEAM_AUDIO_VS_ID, SI_DEFENCE_WGT_VS_ID,
// signals from primitive behaviors, PAM = Primitive Action Map
     // be_goto_border
     SI_GOTO_TOP_BORDER_PAM_ID, SI_GOTO_BOTTOM_BORDER_PAM_ID, SI_GOTO_LEFT_TOP_BORDER_PAM_ID,
     SI_GOTO_LEFT_BOTTOM_BORDER_PAM_ID, SI_GOTO_RIGHT_TOP_BORDER_PAM_ID, SI_GOTO_RIGHT_BOTTOM_BORDER_PAM_ID,
     // be_goto_ball
     SI_GOTO_BALL_PAM_ID,
     // be_goto_brd_flag
     SI_GOTO_TOP_RIGHT_FLAG_PAM_ID, SI_GOTO_TOP_CENTER_FLAG_PAM_ID, SI_GOTO_TOP_LEFT_FLAG_PAM_ID,
     SI_GOTO_BOTTOM_RIGHT_FLAG_PAM_ID, SI_GOTO_BOTTOM_CENTER_FLAG_PAM_ID, SI_GOTO_BOTTOM_LEFT_FLAG_PAM_ID,
     // be_goto_tmate
     SI_GOTO_TMATE1_PAM_ID, SI_GOTO_TMATE2_PAM_ID, SI_GOTO_TMATE3_PAM_ID,
     SI_GOTO_TMATE4_PAM_ID, SI_GOTO_TMATE5_PAM_ID,
     // be_goto_opp
     SI_GOTO_OPP1_PAM_ID, SI_GOTO_OPP2_PAM_ID, SI_GOTO_OPP3_PAM_ID,
     SI_GOTO_OPP4_PAM_ID, SI_GOTO_OPP5_PAM_ID,
     // be_goto_oppg_flag
     SI_GOTO_OPPG_TOP_FLAG_PAM_ID, SI_GOTO_OPPG_CENTER_FLAG_PAM_ID, SI_GOTO_OPPG_BOTTOM_FLAG_PAM_ID,
     // be_goto_owng_flag
     SI_GOTO_OWNG_TOP_FLAG_PAM_ID,              SI_GOTO_OWNG_CENTER_FLAG_PAM_ID, SI_GOTO_OWNG_BOTTOM_FLAG_PAM_ID,
     // be_goto_opp_goal and be_goto_own_goal       
     SI_GOTO_OPPGOAL_PAM_ID, SI_GOTO_OWNGOAL_PAM_ID,      
     // be_goto_near       
     SI_GOTO_NEAR_PAM_ID,              
     // be_lookaround
     SI_LOOKAROUND_PAM_ID,
     // be_goto_ball_dir
     SI_GOTO_BALL_DIR_PAM_ID,                         
     // be_goto_deflog
     SI_GOTO_DEFLOC_PAM_ID,
     // be_goto_special_loc
     SI_GOTO_SPECIAL_LOC_PAM_ID,

     // KICKING SIGNALS:
     SI_KICKTO_TMATE1_PAM_ID, SI_KICKTO_TMATE2_PAM_ID, SI_KICKTO_TMATE3_PAM_ID,
     SI_KICKTO_TMATE4_PAM_ID, SI_KICKTO_TMATE5_PAM_ID,
     SI_KICKTO_OPP1_PAM_ID, SI_KICKTO_OPP2_PAM_ID, SI_KICKTO_OPP3_PAM_ID,
     SI_KICKTO_OPP4_PAM_ID, SI_KICKTO_OPP5_PAM_ID,
     SI_KICKTO_TOP_BORDER_PAM_ID, SI_KICKTO_BOTTOM_BORDER_PAM_ID,
     SI_KICKTO_LEFT_TOP_BORDER_PAM_ID, SI_KICKTO_LEFT_BOTTOM_BORDER_PAM_ID,
     SI_KICKTO_RIGHT_TOP_BORDER_PAM_ID, SI_KICKTO_RIGHT_BOTTOM_BORDER_PAM_ID,
     SI_KICKTO_OWNGOAL_PAM_ID, SI_KICKTO_OPPGOAL_PAM_ID, SI_KICKTO_NEAR_PAM_ID,
     SI_KICKTO_TOP_LEFT_FLAG_PAM_ID, SI_KICKTO_TOP_CENTER_FLAG_PAM_ID,
     SI_KICKTO_TOP_RIGHT_FLAG_PAM_ID, SI_KICKTO_BOTTOM_LEFT_FLAG_PAM_ID ,
     SI_KICKTO_BOTTOM_CENTER_FLAG_PAM_ID, SI_KICKTO_BOTTOM_RIGHT_FLAG_PAM_ID,
     SI_KICKTO_OWNG_TOP_FLAG_PAM_ID, SI_KICKTO_OWNG_CENTER_FLAG_PAM_ID,
     SI_KICKTO_OWNG_BOTTOM_FLAG_PAM_ID , SI_KICKTO_OPPG_TOP_FLAG_PAM_ID ,
     SI_KICKTO_OPPG_CENTER_FLAG_PAM_ID ,SI_KICKTO_OPPG_BOTTOM_FLAG_PAM_ID ,
     
// Signals from special behaviors
     SI_KICK_SPECIAL_MA_CMD_ID, SI_KICK_SPECIAL_AR_MOVE_CMD_ID,
     SI_KICK_SPECIAL_AR_KICK_CMD_ID,
// miscellanous
     USER_ID, VISUAL_ID,
     IDLE_ID, INHIBITED_ID, ACTIVE_ID, LAST_ID
} ID;

typedef enum 
{
     NONE, MOVE_CMD_ID, CHANGE_VQ_CMD_ID, SAY_CMD_ID
} SUPP_ID;


#endif 

/* end of file */
