// file    : v_comm.cc
// includes: functions for the v_comm class

#include "v_comm.h"


v_comm::v_comm()
{
}


v_comm::~v_comm()
{
}


int v_comm::rcv_msg_type ()
{
     int type;
     cin >> type;

     // cout << "Received: type(" << type << ")" << endl;
     
     if (type != V_INIT_MSG 
         && type != V_REQ_MSG 
         && type != V_QUIT_MSG)
          {
               cerr << "VISUALIZING ERROR: type of the rcvd msg (";
               cerr << type << ") not valid" << endl;
               return V_CORRUPTED_MSG;
          }
     return type;
}


int v_comm::send_init (opfstream& out, float min_x, float max_x, 
                       float min_y, float max_y,
                       float scaling, char *title)
{
     if (!out.good())
          {
               cerr << "VISUALIZING ERROR: could not create pipe to visualizing process" << endl;
               return 0;
          }

     out << V_INIT_MSG << ' ';
     out << min_x << ' ';
     out << max_x << ' ';
     out << min_y << ' ';
     out << max_y << ' ';
     out << scaling << ' ';

     // replace whitespaces with underlines on the fly

     int cnt=0;
     for (; cnt<V_MAX_TITLE_LEN-1 && title[cnt] != '\0'; cnt++)
          if (title[cnt] == ' ') out << '_';
          else out << title[cnt];

     // if empty string, send an underline
     if (cnt == 0) out << '_';
     out << ' ' << endl;

     if (!out.good())
          {
               cerr << "VISUALIZING ERROR: could not send init msg" << endl;
               return 0;
          }
     return 1;
}


int v_comm::rcv_init_body (float& min_x, float& max_x, 
                           float& min_y, float& max_y,
                           float& scaling, char title[V_MAX_TITLE_LEN])
{
     cin >> min_x;
     cin >> max_x;
     cin >> min_y;
     cin >> max_y;
     cin >> scaling;

     // replace underlines with whitespaces on the fly

     int cnt=0;
     cin >> title[cnt];
     while (cnt<V_MAX_TITLE_LEN-1 && !isspace(title[cnt]))
          {
               // cout << "RCVD(" << title[cnt] << ')' << endl;
               if (title[cnt] == '_') title[cnt] = ' ';
               cin.get (title[++cnt]);
          }

     // if first char is an underline, this is an empty string
     if (title[0] == '_') title[0] = '\0';
     else title[cnt] = '\0';               

     /*
     cout << "          x(" << min_x << ',' << max_x << ") y(" 
          << min_y << ',' << max_y << ") scaling(" << scaling 
          << ") title(" << title << ')' << endl;
     */

     if (!cin.good())
          {
               cerr << "VISUALIZING ERROR: receiving init msg failed" << endl;
               return 0;
          }

     return 1;
}


int v_comm::rcv_init (float& min_x, float& max_x, 
                      float& min_y, float& max_y,
                      float& scaling, char title[V_MAX_TITLE_LEN])
{
     if (rcv_msg_type () != V_INIT_MSG)
          {
               cerr << "VISUALIZING ERROR: init msg was expected but not received" << endl;
               return 0;
          }

     return rcv_init_body (min_x, max_x, min_y, max_y, scaling, title);
}


int v_comm::send_req (opfstream& out, int objn, v_obj_type type, 
                      float x, float y, float dir, float dim, 
                      char *name, v_color clr)
{
     out << V_REQ_MSG << ' ';
     out << objn << ' ';
     out << type << ' ';
     out << x << ' ';
     out << y << ' ';
     out << dir << ' ';
     out << dim << ' ';

     // replace whitespaces with underlines on the fly

     int cnt=0;
     for (; cnt<V_MAX_NAME_LEN-1 && name[cnt] != '\0'; cnt++)
          if (name[cnt] == ' ') out << '_';
          else out << name[cnt];
     
     // if empty string, send an underline
     if (cnt == 0) out << '_';
     out << ' ';

     out << clr << ' ' << endl;

     if (!out.good())
          {
               cerr << "VISUALIZING ERROR: sending request failed" << endl;
               return 0;
          }
     return 1;
}


int v_comm::rcv_req_body (int& objn, v_obj_type& type, float& x, float& y,
                          float& dir, float& dim, char name[V_MAX_NAME_LEN], 
                          v_color& clr)
{
     cin >> objn;

     int raw_obj_type = 0;
     cin >> raw_obj_type;
     type = v_obj_type(raw_obj_type);

     cin >> x;
     cin >> y;
     cin >> dir;
     cin >> dim;

     // replace underlines with whitespaces on the fly

     int cnt=0;
     cin >> name[cnt];
     while (cnt<V_MAX_NAME_LEN-1 && !isspace(name[cnt]))
          {
               // cout << "RCVD(" << name[cnt] << ')' << endl;
               if (name[cnt] == '_') name[cnt] = ' ';
               cin.get (name[++cnt]);
          }

     // if first char is an underline, this is an empty string
     if (name[0] == '_') name[0] = '\0';
     else name[cnt] = '\0';               

     int raw_clr = 0;
     cin >> raw_clr;
     clr = v_color(raw_clr);

/*
     cout << "          x(" << x << ") y(" << y << ") dir(" << dir 
          << ") dim(" << dim << ") name(" << name << ')' << endl;
*/
     
     if (!cin.good())
          {
               cerr << "VISUALIZING ERROR: receiving request msg failed" << endl;
               return 0;
          }
     
     return 1;
}


int v_comm::rcv_req (int& objn, v_obj_type& type, float& x, float& y,
                     float& dir, float& dim, char name[V_MAX_NAME_LEN], 
                     v_color& clr)
{
     int msg_type = rcv_msg_type();
     if (msg_type != V_REQ_MSG)
          {
               // do not report a quit msg as an error
               if (msg_type != V_QUIT_MSG)
                    cerr << "VISUALIZING ERROR: request msg was expected but not received" << endl;
               return 0;
          }
     
     return rcv_req_body (objn, type, x, y, dir, dim, name, clr);
}


void v_comm::send_quit (opfstream& out)
{
     out << V_QUIT_MSG << endl;
}

/* end of file */


