// -*- mode:C++ -*-
// file    : v_producer.h
// includes: class declaration for the v_producer
// 
// v_producer produces drawing requests. These requests are handled
// by the visualizer. v_produzer initializes the visualizer and sends
// the requests to it through a pipe.
//
// return values: 1 = success; 0 = failure

#ifndef _V_PRODUCER_H
#define _V_PRODUCER_H 1

#include "v_comm.h"

class v_comm;

class v_producer {
public:
     // the 1st argument is the name of the visualizer program,
     // min_x, min_y, max_x, and max_y determine the limits of
     // coordinate system, scaling defines how many pixels is
     // one unit in the coordinate system, the last argument
     // is the window title

     v_producer (char* program_pipe,
                 float min_x, float max_x, 
                 float min_y, float max_y,
                 float scaling, char *title);
     ~v_producer ();

     // producer sends one drawing request with this method

     int send_req (int objn, v_obj_type type, float x, float y,
                   float dir, float dim, char *name, v_color clr);

     // producer quits the visualizer with this method

     int quit ();

     int max_dyn_obj_id () {return V_NOBJS-1;}
     int stat_obj_id() {return V_NOBJS;}
     

protected:
     v_comm comm;   // communication object
     opfstream out; // pipe to the drawing program
     int active;    // 0 = not, 1 = yes (i.e. initialized)
};

#endif _V_PRODUCER_H

/* end of file */







